@@ -49,10 +49,20 @@
 #include "intrface.h"
 #include "io_redir.h"
 #include "log.h"
+#include "guid_cmp.h"
+#include "dimage.h"
+
+extern const arch_fnct_t arch_gpt;
+extern const arch_fnct_t arch_i386;
+extern const arch_fnct_t arch_mac;
+extern const arch_fnct_t arch_none;
+extern const arch_fnct_t arch_sun;
+extern const arch_fnct_t arch_xbox;
 
 #define INTER_ADV_X	0
 #define INTER_ADV_Y	23
 #define INTER_ADV	15
+#define DEFAULT_IMAGE_NAME "image.dd"
 
 static int is_hfs(const partition_t *partition);
 static int is_hfsp(const partition_t *partition);
@@ -67,7 +77,7 @@ static int is_hfs(const partition_t *partition)
 
 static int is_hfsp(const partition_t *partition)
 {
-  return (is_part_hfsp(partition) || partition->upart_type==UP_HFSP);
+  return (is_part_hfsp(partition) || partition->upart_type==UP_HFSP || partition->upart_type==UP_HFSX);
 }
 
 static int is_linux(const partition_t *partition)
@@ -107,6 +117,8 @@ static int is_part_hfs(const partition_t *partition)
     case PMAC_HFS:
       return 1;
   }
+  if(guid_cmp(partition->part_type_gpt,GPT_ENT_TYPE_MAC_HFS)==0)
+    return 1;
   return 0;
 }
 
@@ -122,30 +134,39 @@ static int is_part_hfsp(const partition_t *partition)
     case PMAC_HFS:
       return 1;
   }
+  if(guid_cmp(partition->part_type_gpt,GPT_ENT_TYPE_MAC_HFS)==0)
+    return 1;
   return 0;
 }
 
 int is_part_linux(const partition_t *partition)
 {
-  switch(partition->part_type_i386)
+  if(partition->arch==&arch_i386)
   {
-    case P_LINUX:
+    if(partition->part_type_i386==P_LINUX)
       return 1;
   }
-  switch(partition->part_type_sun)
+  else if(partition->arch==&arch_sun)
   {
-    case PSUN_LINUX:
+    if(partition->part_type_sun==PSUN_LINUX)
       return 1;
   }
-  switch(partition->part_type_mac)
+  else if(partition->arch==&arch_mac)
   {
-    case PMAC_LINUX:
+    if(partition->part_type_mac==PMAC_LINUX)
       return 1;
   }
+  /*
+  else if(partition->arch==&arch_gpt)
+  {
+    if(guid_cmp(partition->part_type_gpt,GPT_ENT_TYPE_LINUX_DATA)==0)
+      return 1;
+  }
+  */
   return 0;
 }
 
-void interface_adv(disk_t *disk_car, const int debug,const int dump_ind, const unsigned int expert, char**current_cmd)
+void interface_adv(disk_t *disk_car, const int verbose,const int dump_ind, const unsigned int expert, char**current_cmd)
 {
   int quit;
   int offset=0;
@@ -156,7 +177,7 @@ void interface_adv(disk_t *disk_car, const int debug,const int dump_ind, const u
   list_part_t *list_part;
   list_part_t *current_element;
   log_info("\nInterface Advanced\n");
-  list_part=disk_car->arch->read_part(disk_car,debug,0);
+  list_part=disk_car->arch->read_part(disk_car,verbose,0);
   current_element=list_part;
   for(element=list_part;element!=NULL;element=element->next)
   {
@@ -169,6 +190,7 @@ void interface_adv(disk_t *disk_car, const int debug,const int dump_ind, const u
       {'t',"Type","Change type, this setting will not be saved on disk"},
       {'b',"Boot","Boot sector recovery"},
       {'s',"Superblock",NULL},
+      {'c',"Image creation", "Create an image"},
 //      {'a',"Add", "Add temporary partition (Expert only)"},
       {'q',"Quit","Return to main menu"},
       {0,NULL,NULL}
@@ -176,11 +198,12 @@ void interface_adv(disk_t *disk_car, const int debug,const int dump_ind, const u
     int menu=0;
     int i;
     int command;
+#ifdef HAVE_NCURSES
     if(rewrite!=0)
     {
       aff_copy(stdscr);
       wmove(stdscr,4,0);
-      wdoprintf(stdscr,"%s",disk_car->description(disk_car));
+      wprintw(stdscr,"%s",disk_car->description(disk_car));
       if(list_part!=NULL)
 	mvwaddstr(stdscr,6,0,msg_PART_HEADER_LONG);
       rewrite=0;
@@ -193,62 +216,65 @@ void interface_adv(disk_t *disk_car, const int debug,const int dump_ind, const u
       if(element==current_element)
       {
 	wattrset(stdscr, A_REVERSE);
-	aff_part(stdscr,AFF_PART_ORDER,disk_car,element->part);
+	aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,element->part);
 	wattroff(stdscr, A_REVERSE);
       } else
       {
-	aff_part(stdscr,AFF_PART_ORDER,disk_car,element->part);
+	aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,element->part);
       }
     }
+#endif
     menu=0;
     if(current_element==NULL)
     {
       options="q";
+#ifdef HAVE_NCURSES
       wmove(stdscr,5+2,0);
       wattrset(stdscr, A_REVERSE);
-      wdoprintf(stdscr,"No partition available.");
+      wprintw(stdscr,"No partition available.");
       wattroff(stdscr, A_REVERSE);
+#endif
     }
     else
     {
       if(is_part_fat(current_element->part) ||
 	  is_part_ntfs(current_element->part))
       {
-	options="tbq";
+	options="tbcq";
 	menu=1;
       }
       else if(is_part_linux(current_element->part))
       {
-	options="tsq";
+	options="tscq";
 	menuAdv[2].desc="Locate EXT2/EXT3 backup superblock";
 	menu=1;
       }
       else if(is_part_hfs(current_element->part) || is_part_hfsp(current_element->part))
       {
-	options="tsq";
+	options="tscq";
 	menuAdv[2].desc="Locate HFS/HFS+ backup superblock";
 	menu=1;
       }
       else if(is_fat(current_element->part) ||
 	  is_ntfs(current_element->part))
       {
-	options="tbq";
+	options="tbcq";
 	menu=1;
       }
       else if(is_linux(current_element->part))
       {
-	options="tsq";
+	options="tscq";
 	menuAdv[2].desc="Locate EXT2/EXT3 backup superblock";
 	menu=1;
       }
       else if(is_hfs(current_element->part) || is_hfsp(current_element->part))
       {
-	options="tsq";
+	options="tscq";
 	menuAdv[2].desc="Locate HFS/HFS+ backup superblock";
 	menu=1;
       }
       else
-	options="tq";
+	options="tcq";
     }
     quit=0;
     if(*current_cmd!=NULL)
@@ -270,6 +296,11 @@ void interface_adv(disk_t *disk_car, const int debug,const int dump_ind, const u
 	  (*current_cmd)+=4;
 	  command='b';
 	}
+	else if(strncmp(*current_cmd,"copy",4)==0)
+	{
+	  (*current_cmd)+=4;
+	  command='c';
+	}
 	else if(strncmp(*current_cmd,"superblock",10)==0)
 	{
 	  (*current_cmd)+=10;
@@ -291,10 +322,17 @@ void interface_adv(disk_t *disk_car, const int debug,const int dump_ind, const u
       } while(keep_asking>0);
     }
     else
+    {
+#ifdef HAVE_NCURSES
       command = wmenuSelect(stdscr,INTER_ADV_Y, INTER_ADV_X, menuAdv, 8, options,
 	  MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu);
+#else
+      command = 'q';
+#endif
+    }
     switch(command)
     {
+#ifdef HAVE_NCURSES
       case KEY_UP:
 	if(current_element!=NULL)
 	{
@@ -319,6 +357,7 @@ void interface_adv(disk_t *disk_car, const int debug,const int dump_ind, const u
 	    offset++;
 	}
 	break;
+#endif
       case 'q':
       case 'Q':
 	quit=1;
@@ -326,7 +365,7 @@ void interface_adv(disk_t *disk_car, const int debug,const int dump_ind, const u
       case 'a':
       case 'A':
 	{
-	  list_part=disk_car->arch->add_partition(disk_car,list_part, debug, current_cmd);
+	  list_part=disk_car->arch->add_partition(disk_car,list_part, verbose, current_cmd);
 	  current_element=list_part;
 	  rewrite=1;
 	}
@@ -335,51 +374,75 @@ void interface_adv(disk_t *disk_car, const int debug,const int dump_ind, const u
       case 'B':
 	if(current_element!=NULL)
 	{
-	  if(is_part_fat32(current_element->part))
+	  partition_t *partition=current_element->part;
+	  if(is_part_fat32(partition))
 	  {
-	    fat32_boot_sector(disk_car, current_element->part, debug, dump_ind, expert,current_cmd);
+	    fat32_boot_sector(disk_car, partition, verbose, dump_ind, expert,current_cmd);
 	    rewrite=1;
 	  }
-	  else if(is_part_fat12(current_element->part) || is_part_fat16(current_element->part))
+	  else if(is_part_fat12(partition) || is_part_fat16(partition))
 	  {
-	    fat1x_boot_sector(disk_car, current_element->part, debug, dump_ind,expert,current_cmd);
+	    fat1x_boot_sector(disk_car, partition, verbose, dump_ind,expert,current_cmd);
 	    rewrite=1;
 	  }
-	  else if(is_part_ntfs(current_element->part))
+	  else if(is_part_ntfs(partition))
 	  {
-	    ntfs_boot_sector(disk_car, current_element->part, debug, dump_ind, expert, current_cmd);
+	    ntfs_boot_sector(disk_car, partition, verbose, dump_ind, expert, current_cmd);
 	    rewrite=1;
 	  }
-	  else if(current_element->part->upart_type==UP_FAT32)
+	  else if(partition->upart_type==UP_FAT32)
 	  {
-	    fat32_boot_sector(disk_car, current_element->part, debug, dump_ind, expert,current_cmd);
+	    fat32_boot_sector(disk_car, partition, verbose, dump_ind, expert,current_cmd);
 	    rewrite=1;
 	  }
-	  else if(current_element->part->upart_type==UP_FAT12 || current_element->part->upart_type==UP_FAT16)
+	  else if(partition->upart_type==UP_FAT12 || partition->upart_type==UP_FAT16)
 	  {
-	    fat1x_boot_sector(disk_car, current_element->part, debug, dump_ind,expert,current_cmd);
+	    fat1x_boot_sector(disk_car, partition, verbose, dump_ind,expert,current_cmd);
 	    rewrite=1;
 	  }
-	  else if(current_element->part->upart_type==UP_NTFS)
+	  else if(partition->upart_type==UP_NTFS)
 	  {
-	    ntfs_boot_sector(disk_car, current_element->part, debug, dump_ind, expert, current_cmd);
+	    ntfs_boot_sector(disk_car, partition, verbose, dump_ind, expert, current_cmd);
 	    rewrite=1;
 	  }
 	}
 	break;
+      case 'c':
+      case 'C':
+	if(current_element!=NULL)
+	{
+	  char *image_dd;
+	  menu=0;
+	  image_dd=ask_location("Do you want to save disk file image.dd in %s%s ? [Y/N]","");
+	  if(image_dd!=NULL)
+	  {
+	    char *new_recup_dir=MALLOC(strlen(image_dd)+1+strlen(DEFAULT_IMAGE_NAME)+1);
+	    strcpy(new_recup_dir,image_dd);
+	    strcat(new_recup_dir,"/");
+	    strcat(new_recup_dir,DEFAULT_IMAGE_NAME);
+	    free(image_dd);
+	    image_dd=new_recup_dir;
+	  }
+	  if(image_dd!=NULL)
+	  {
+	    disk_image(disk_car, current_element->part, image_dd);
+	    free(image_dd);
+	  }
+	}
+	break;
       case 's':
       case 'S':
 	if(current_element!=NULL)
 	{
 	  if(is_linux(current_element->part))
 	  {
-	    list_part_t *list_sb=search_superblock(disk_car,current_element->part,debug,dump_ind,1);
+	    list_part_t *list_sb=search_superblock(disk_car,current_element->part,verbose,dump_ind,1);
 	    interface_superblock(disk_car,list_sb,current_cmd);
-	    delete_list_part(list_sb);
+	    part_free_list(list_sb);
 	  }
 	  if(is_hfs(current_element->part) || is_hfsp(current_element->part))
 	  {
-	    HFS_HFSP_boot_sector(disk_car, current_element->part, debug, dump_ind, expert, current_cmd);
+	    HFS_HFSP_boot_sector(disk_car, current_element->part, verbose, dump_ind, expert, current_cmd);
 	  }
 	  rewrite=1;
 	}
@@ -394,10 +457,39 @@ void interface_adv(disk_t *disk_car, const int debug,const int dump_ind, const u
 	break;
     }
   } while(quit==0);
-  delete_list_part(list_part);
+  part_free_list(list_part);
 }
 
-int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd)
+#ifdef HAVE_NCURSES
+static void dump_fat1x_ncurses(disk_t *disk_car, partition_t *partition, const unsigned char *buffer_bs)
+{
+  WINDOW *window=newwin(0,0,0,0);	/* full screen */
+  keypad(window, TRUE); /* Need it to get arrow key */
+  aff_copy(window);
+  wmove(window,4,0);
+  wprintw(window,"%s",disk_car->description(disk_car));
+  wmove(window,5,0);
+  aff_part(window,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+  mvwaddstr(window,6,0, "Boot sector");
+  dump(window,buffer_bs,FAT1x_BOOT_SECTOR_SIZE);
+  delwin(window);
+  (void) clearok(stdscr, TRUE);
+#ifdef HAVE_TOUCHWIN
+  touchwin(stdscr);
+#endif
+}
+#endif
+
+static void dump_fat1x(disk_t *disk_car, partition_t *partition, const unsigned char *buffer_bs)
+{
+  log_info("Boot sector\n");
+  dump_log(buffer_bs, FAT1x_BOOT_SECTOR_SIZE);
+#ifdef HAVE_NCURSES
+  dump_fat1x_ncurses(disk_car, partition, buffer_bs);
+#endif
+}
+
+int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int verbose, const int dump_ind, const unsigned int expert, char **current_cmd)
 {
   unsigned char *buffer_bs;
   const char *options="DR";
@@ -407,6 +499,7 @@ int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
     { 'P', "Previous",""},
     { 'N', "Next","" },
     { 'Q', "Quit","Return to Advanced menu"},
+    { 'L', "List", "List directories and files" },
     { 'R', "Rebuild BS","Rebuild boot sector"},
     { 'D', "Dump","Dump boot sector and backup boot sector"},
     { 'C', "Repair FAT","Very Dangerous! Expert only"},
@@ -416,37 +509,46 @@ int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
   buffer_bs=(unsigned char*)MALLOC(FAT1x_BOOT_SECTOR_SIZE);
   while(1)
   {
+    unsigned int menu=0;
     int command;
     aff_buffer(BUFFER_RESET,"Q");
     if(rescan==1)
     {
+#ifdef HAVE_NCURSES
       aff_copy(stdscr);
       wmove(stdscr,4,0);
-      wdoprintf(stdscr,"%s",disk_car->description(disk_car));
+      wprintw(stdscr,"%s",disk_car->description(disk_car));
       mvwaddstr(stdscr,5,0,msg_PART_HEADER_LONG);
       wmove(stdscr,6,0);
-      aff_part(stdscr,AFF_PART_ORDER,disk_car,partition);
+      aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+#endif
       log_info("\nfat1x_boot_sector\n");
       log_partition(disk_car,partition);
-      if(expert==0)
-	options="DRC";
-      else
-	options="DRCI";
       aff_buffer(BUFFER_ADD,"Boot sector\n");
       if(disk_car->read(disk_car,FAT1x_BOOT_SECTOR_SIZE, buffer_bs, partition->part_offset)!=0)
       {
 	aff_buffer(BUFFER_ADD,"fat1x_boot_sector: Can't read boot sector.\n");
 	memset(buffer_bs,0,FAT1x_BOOT_SECTOR_SIZE);
       }
-      if(test_FAT(disk_car,(const struct fat_boot_sector *)buffer_bs,partition,debug,0)==0)
+      if(test_FAT(disk_car,(const struct fat_boot_sector *)buffer_bs,partition,verbose,0)==0)
       {
 	aff_buffer(BUFFER_ADD,"OK\n");
+	if(expert==0)
+	  options="DRCL";
+	else
+	  options="DRCIL";
+      }
+      else
+      {
+	aff_buffer(BUFFER_ADD,"Bad\n");
+	options="DRC";
       }
       aff_buffer(BUFFER_ADD,"\n");
       aff_buffer(BUFFER_ADD,"A valid FAT Boot sector must be present in order to access\n");
       aff_buffer(BUFFER_ADD,"any data; even if the partition is not bootable.\n");
       rescan=0;
     }
+    screen_buffer_to_log();
     if(*current_cmd!=NULL)
     {
       command=0;
@@ -462,6 +564,12 @@ int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	(*current_cmd)+=4;
 	command='D';
       }
+      else if(strncmp(*current_cmd,"list",4)==0)
+      {
+	(*current_cmd)+=4;
+	if(strchr(options,'L')!=NULL)
+	  command='L';
+      }
       else if(strncmp(*current_cmd,"repairfat",8)==0)
       {
 	(*current_cmd)+=8;
@@ -474,49 +582,72 @@ int fat1x_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	if(strchr(options,'I')!=NULL)
 	    command='I';
       }
-      screen_buffer_to_log();
     }
     else
-      command=screen_buffer_display(stdscr,options,menu_fat1x);
+    {
+      log_flush();
+#ifdef HAVE_NCURSES
+      command=screen_buffer_display_ext(stdscr, options, menu_fat1x, &menu);
+#else
+      command=0;
+#endif
+    }
     switch(command)
     {
       case 0:
 	free(buffer_bs);
 	return 0;
       case 'R': /* R : rebuild boot sector */
-	rebuild_FAT_BS(disk_car,partition,debug,dump_ind,1,expert,current_cmd);
+	rebuild_FAT_BS(disk_car,partition,verbose,dump_ind,1,expert,current_cmd);
 	rescan=1;
 	break;
       case 'D':
-	{
-	  WINDOW *window=newwin(0,0,0,0);	/* full screen */
-	  keypad(window, TRUE); /* Need it to get arrow key */
-	  aff_copy(window);
-	  wmove(window,4,0);
-	  wdoprintf(window,"%s",disk_car->description(disk_car));
-	  wmove(window,5,0);
-	  aff_part(window,AFF_PART_ORDER,disk_car,partition);
-	  log_info("Boot sector\n");
-	  mvwaddstr(window,6,0, "Boot sector");
-	  dump(window,buffer_bs,FAT1x_BOOT_SECTOR_SIZE);
-	  delwin(window);
-	  (void) clearok(stdscr, TRUE);
-#ifdef HAVE_TOUCHWIN
-	  touchwin(stdscr);
-#endif
-	}
+	dump_fat1x(disk_car, partition, buffer_bs);
 	break;
       case 'C':
-	repair_FAT_table(disk_car,partition,debug);
+	repair_FAT_table(disk_car,partition,verbose);
 	break;
       case 'I':
-	FAT_init_rootdir(disk_car,partition,debug);
+	FAT_init_rootdir(disk_car,partition,verbose);
+	break;
+      case 'L':
+	dir_partition(disk_car, partition, 0,current_cmd);
 	break;
     }
   }
 }
 
-int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd)
+#ifdef HAVE_NCURSES
+static void dump_fat32_ncurses(disk_t *disk_car, const partition_t *partition, const unsigned char *buffer_bs, const unsigned char *buffer_backup_bs)
+{
+  WINDOW *window=newwin(0,0,0,0);	/* full screen */
+  keypad(window, TRUE); /* Need it to get arrow key */
+  aff_copy(window);
+  wmove(window,4,0);
+  wprintw(window,"%s",disk_car->description(disk_car));
+  wmove(window,5,0);
+  aff_part(window,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+  mvwaddstr(window,6,0, "Boot sector                        Backup boot sector");
+  dump2(window, buffer_bs, buffer_backup_bs, 3*disk_car->sector_size);
+  delwin(window);
+  (void) clearok(stdscr, TRUE);
+#ifdef HAVE_TOUCHWIN
+  touchwin(stdscr);
+#endif
+}
+#endif
+
+static void dump_fat32(disk_t *disk_car, const partition_t *partition, const unsigned char *buffer_bs, const unsigned char *buffer_backup_bs)
+{
+  log_info("Boot sector                        Backup boot sector\n");
+  dump2_log(buffer_bs, buffer_backup_bs, 3*disk_car->sector_size);
+  log_fat2_info((const struct fat_boot_sector*)buffer_bs,(const struct fat_boot_sector*)buffer_backup_bs,UP_FAT32,disk_car->sector_size);
+#ifdef HAVE_NCURSES
+  dump_fat32_ncurses(disk_car, partition, buffer_bs, buffer_backup_bs);
+#endif
+}
+
+int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int verbose, const int dump_ind, const unsigned int expert, char **current_cmd)
 {
   unsigned char *buffer_bs;
   unsigned char *buffer_backup_bs;
@@ -527,6 +658,7 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
     { 'P', "Previous",""},
     { 'N', "Next","" },
     { 'Q', "Quit","Return to Advanced menu"},
+    { 'L', "List", "List directories and files" },
     { 'O', "Org. BS","Copy boot sector over backup sector"},
     { 'B', "Backup BS","Copy backup boot sector over boot sector"},
     { 'R', "Rebuild BS","Rebuild boot sector"},
@@ -538,6 +670,7 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
   buffer_backup_bs=(unsigned char*)MALLOC(3*disk_car->sector_size);
   while(1)
   {
+    unsigned int menu=0;
     int command;
     aff_buffer(BUFFER_RESET,"Q");
     if(rescan==1)
@@ -546,12 +679,14 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
       int opt_B=0;
       int opt_O=0;
       options="DRC";
+#ifdef HAVE_NCURSES
       aff_copy(stdscr);
       wmove(stdscr,4,0);
-      wdoprintf(stdscr,"%s",disk_car->description(disk_car));
+      wprintw(stdscr,"%s",disk_car->description(disk_car));
       mvwaddstr(stdscr,5,0,msg_PART_HEADER_LONG);
       wmove(stdscr,6,0);
-      aff_part(stdscr,AFF_PART_ORDER,disk_car,partition);
+      aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+#endif
       log_info("\nfat32_boot_sector\n");
       log_partition(disk_car,partition);
       aff_buffer(BUFFER_ADD,"Boot sector\n");
@@ -560,18 +695,22 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	aff_buffer(BUFFER_ADD,"fat32_boot_sector: Can't read boot sector.\n");
 	memset(buffer_bs,0,3*disk_car->sector_size);
       }
-      if(test_FAT(disk_car,(struct fat_boot_sector *)buffer_bs,partition,debug,0)==0)
+      if(test_FAT(disk_car,(struct fat_boot_sector *)buffer_bs,partition,verbose,0)==0)
       {
-	aff_buffer(BUFFER_ADD,"OK\n");
-	if(partition->upart_type==UP_FAT32)
-	{
-	  opt_O=1;
-	  opt_over=1;
-	}
-	else
-	{
-	  aff_buffer(BUFFER_ADD,"Warning: valid FAT bootsector but not a FAT32 one!");
-	}
+        aff_buffer(BUFFER_ADD,"OK\n");
+        if(partition->upart_type==UP_FAT32)
+        {
+          opt_O=1;
+          opt_over=1;
+        }
+        else
+        {
+          aff_buffer(BUFFER_ADD,"Warning: valid FAT bootsector but not a FAT32 one!");
+        }
+      }
+      else
+      {
+        aff_buffer(BUFFER_ADD,"Bad\n");
       }
       aff_buffer(BUFFER_ADD,"\nBackup boot sector\n");
       if(disk_car->read(disk_car,3*disk_car->sector_size, buffer_backup_bs, partition->part_offset+6*disk_car->sector_size)!=0)
@@ -579,7 +718,7 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	aff_buffer(BUFFER_ADD,"fat32_boot_sector: Can't read backup boot sector.\n");
 	memset(buffer_backup_bs,0,3*disk_car->sector_size);
       }
-      if(test_FAT(disk_car,(struct fat_boot_sector *)buffer_backup_bs,partition,debug,0)==0)
+      if(test_FAT(disk_car,(struct fat_boot_sector *)buffer_backup_bs,partition,verbose,0)==0)
       {
 	aff_buffer(BUFFER_ADD,"OK\n");
 	if(partition->upart_type==UP_FAT32)
@@ -592,6 +731,10 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	  aff_buffer(BUFFER_ADD,"Warning: valid FAT backup bootsector but not a FAT32 one!");
 	}
       }
+      else
+      {
+        aff_buffer(BUFFER_ADD,"Bad\n");
+      }
       aff_buffer(BUFFER_ADD,"\n");
       if((memcmp(buffer_bs,buffer_backup_bs,0x3E8)==0)&&(memcmp(buffer_bs+0x3F0,buffer_backup_bs+0x3F0,0x600-0x3F0))==0)
       {
@@ -614,14 +757,28 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
       if(opt_over!=0)
       {
 	if(opt_B!=0 && opt_O!=0)
-	  options="DOBR";
+	  options="DOBRL";
 	else if(opt_B!=0)
-	  options="DBR";
+	{
+	  menu=5;
+	  options="DBRL";
+	}
 	else if(opt_O!=0)
-	  options="DOR";
+	{
+	  menu=4;
+	  options="DORL";
+	}
+      }
+      else
+      {
+	if(opt_B!=0)
+	  options="DRCL";
+	else
+	  options="DR";
       }
       rescan=0;
     }
+    screen_buffer_to_log();
     if(*current_cmd!=NULL)
     {
       command=0;
@@ -637,6 +794,12 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	(*current_cmd)+=4;
 	command='D';
       }
+      else if(strncmp(*current_cmd,"list",4)==0)
+      {
+	(*current_cmd)+=4;
+	if(strchr(options,'L')!=NULL)
+	  command='L';
+      }
       else if(strncmp(*current_cmd,"repairfat",8)==0)
       {
 	(*current_cmd)+=8;
@@ -655,10 +818,16 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	if(strchr(options,'B')!=NULL)
 	    command='B';
       }
-      screen_buffer_to_log();
     }
     else
-      command=screen_buffer_display(stdscr,options,menu_fat32);
+    {
+      log_flush();
+#ifdef HAVE_NCURSES
+      command=screen_buffer_display_ext(stdscr, options, menu_fat32, &menu);
+#else
+      command=0;
+#endif
+    }
     switch(command)
     {
       case 0:
@@ -673,6 +842,7 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	  {
 	    display_message("Write error: Can't overwrite FAT32 backup boot sector\n");
 	  }
+          disk_car->sync(disk_car);
 	  rescan=1;
 	}
 	break;
@@ -684,41 +854,64 @@ int fat32_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	  {
 	    display_message("Write error: Can't overwrite FAT32 boot sector\n");
 	  }
+          disk_car->sync(disk_car);
 	  rescan=1;
 	}
 	break;
-      case 'R': /* R : rebuild boot sector */
-	rebuild_FAT_BS(disk_car,partition,debug,dump_ind,1,expert,current_cmd);
-	rescan=1;
+      case 'C':
+	repair_FAT_table(disk_car,partition,verbose);
 	break;
       case 'D':
-	{
-	  WINDOW *window=newwin(0,0,0,0);	/* full screen */
-	  keypad(window, TRUE); /* Need it to get arrow key */
-	  aff_copy(window);
-	  wmove(window,4,0);
-	  wdoprintf(window,"%s",disk_car->description(disk_car));
-	  wmove(window,5,0);
-	  aff_part(window,AFF_PART_ORDER,disk_car,partition);
-	  log_info("Boot sector                        Backup boot sector\n");
-	  mvwaddstr(window,6,0, "Boot sector                        Backup boot sector");
-	  dump2(window,buffer_bs,buffer_backup_bs,3*disk_car->sector_size);
-	  delwin(window);
-	  (void) clearok(stdscr, TRUE);
-#ifdef HAVE_TOUCHWIN
-	  touchwin(stdscr);
-#endif
-	  log_fat2_dump((const struct fat_boot_sector*)buffer_bs,(const struct fat_boot_sector*)buffer_backup_bs,UP_FAT32,disk_car->sector_size);
-	}
+	dump_fat32(disk_car, partition, buffer_bs, buffer_backup_bs);
 	break;
-      case 'C':
-	repair_FAT_table(disk_car,partition,debug);
+      case 'L':
+	if(strchr(options,'O')==NULL && strchr(options,'B')!=NULL)
+	{
+	  io_redir_add_redir(disk_car,partition->part_offset,3*disk_car->sector_size,0,buffer_backup_bs);
+	  dir_partition(disk_car, partition, 0,current_cmd);
+	  io_redir_del_redir(disk_car,partition->part_offset);
+	}
+	else
+	  dir_partition(disk_car, partition, 0,current_cmd);
+	break;
+      case 'R': /* R : rebuild boot sector */
+	rebuild_FAT_BS(disk_car,partition,verbose,dump_ind,1,expert,current_cmd);
+	rescan=1;
 	break;
     }
   }
 }
 
-int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd)
+#ifdef HAVE_NCURSES
+static void dump_NTFS_ncurses(disk_t *disk_car, const partition_t *partition, const unsigned char *buffer_bs, const unsigned char *buffer_backup_bs)
+{
+  WINDOW *window=newwin(0,0,0,0);	/* full screen */
+  keypad(window, TRUE); /* Need it to get arrow key */
+  aff_copy(window);
+  wmove(window,4,0);
+  wprintw(window,"%s",disk_car->description(disk_car));
+  wmove(window,5,0);
+  aff_part(window,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+  mvwaddstr(window,6,0, "Boot sector                        Backup boot sector");
+  dump2(window, buffer_bs, buffer_backup_bs, NTFS_BOOT_SECTOR_SIZE);
+  delwin(window);
+  (void) clearok(stdscr, TRUE);
+#ifdef HAVE_TOUCHWIN
+  touchwin(stdscr);
+#endif
+}
+#endif
+
+static void dump_NTFS(disk_t *disk_car, const partition_t *partition, const unsigned char *buffer_bs, const unsigned char *buffer_backup_bs)
+{
+  log_info("Boot sector                        Backup boot sector\n");
+  dump2_log(buffer_bs, buffer_backup_bs, NTFS_BOOT_SECTOR_SIZE);
+#ifdef HAVE_NCURSES
+  dump_NTFS_ncurses(disk_car, partition, buffer_bs, buffer_backup_bs);
+#endif
+}
+
+int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int verbose, const int dump_ind, const unsigned int expert, char **current_cmd)
 {
   unsigned char *buffer_bs;
   unsigned char *buffer_backup_bs;
@@ -745,21 +938,22 @@ int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 
   while(1)
   {
+    unsigned int menu=0;
     int command;
     aff_buffer(BUFFER_RESET,"Q");
     if(rescan==1)
     {
-      int opt_over=0;
+      int identical_sectors=0;
       int opt_B=0;
       int opt_O=0;
-//      options="DRMLU";
-      options="DRML";
+#ifdef HAVE_NCURSES
       aff_copy(stdscr);
       wmove(stdscr,4,0);
-      wdoprintf(stdscr,"%s",disk_car->description(disk_car));
+      wprintw(stdscr,"%s",disk_car->description(disk_car));
       mvwaddstr(stdscr,5,0,msg_PART_HEADER_LONG);
       wmove(stdscr,6,0);
-      aff_part(stdscr,AFF_PART_ORDER,disk_car,partition);
+      aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+#endif
       log_info("\nntfs_boot_sector\n");
       log_partition(disk_car,partition);
       aff_buffer(BUFFER_ADD,"Boot sector\n");
@@ -768,11 +962,10 @@ int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	aff_buffer(BUFFER_ADD,"ntfs_boot_sector: Can't read boot sector.\n");
 	memset(buffer_bs,0,NTFS_BOOT_SECTOR_SIZE);
       }
-      if(test_NTFS(disk_car,(struct ntfs_boot_sector*)buffer_bs,partition,debug,0)==0)
+      if(test_NTFS(disk_car,(struct ntfs_boot_sector*)buffer_bs,partition,verbose,0)==0)
       {
 	aff_buffer(BUFFER_ADD,"Status: OK\n");
 	opt_O=1;
-	opt_over=1;
       }
       else
       {
@@ -784,11 +977,10 @@ int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	aff_buffer(BUFFER_ADD,"ntfs_boot_sector: Can't read backup boot sector.\n");
 	memset(buffer_backup_bs,0,NTFS_BOOT_SECTOR_SIZE);
       }
-      if(test_NTFS(disk_car,(struct ntfs_boot_sector*)buffer_backup_bs,partition,debug,0)==0)
+      if(test_NTFS(disk_car,(struct ntfs_boot_sector*)buffer_backup_bs,partition,verbose,0)==0)
       {
 	aff_buffer(BUFFER_ADD,"Status: OK\n");
 	opt_B=1;
-	opt_over=1;
       }
       else
       {
@@ -797,29 +989,42 @@ int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
       aff_buffer(BUFFER_ADD,"\n");
       if(memcmp(buffer_bs,buffer_backup_bs,NTFS_BOOT_SECTOR_SIZE)==0)
       {
-	log_ntfs_dump((const struct ntfs_boot_sector *)buffer_bs);
+	log_ntfs_info((const struct ntfs_boot_sector *)buffer_bs);
 	aff_buffer(BUFFER_ADD,"Sectors are identical.\n");
-	opt_over=0;
+	identical_sectors=1;
       }
       else
       {
-	log_ntfs2_dump((const struct ntfs_boot_sector *)buffer_bs, (const struct ntfs_boot_sector *)buffer_backup_bs);
+	log_ntfs2_info((const struct ntfs_boot_sector *)buffer_bs, (const struct ntfs_boot_sector *)buffer_backup_bs);
 	aff_buffer(BUFFER_ADD,"Sectors are not identical.\n");
+	identical_sectors=0;
       }
       aff_buffer(BUFFER_ADD,"\n");
       aff_buffer(BUFFER_ADD,"A valid NTFS Boot sector must be present in order to access\n");
       aff_buffer(BUFFER_ADD,"any data; even if the partition is not bootable.\n");
-      if(opt_over!=0)
+      if(opt_B!=0 && opt_O!=0)
       {
-	if(opt_B!=0 && opt_O!=0)
+//      options="DRMLU";
+	if(identical_sectors==0)
 	  options="DOBRL";
-	else if(opt_B!=0)
-	  options="DBRL";
-	else if(opt_O!=0)
-	  options="DORL";
+	else
+	  options="DRML";
       }
+      else if(opt_B!=0)
+      {
+	menu=5;
+	options="DBRL";
+      }
+      else if(opt_O!=0)
+      {
+	menu=4;
+	options="DORL";
+      }
+      else
+	options="DR";
       rescan=0;
     }
+    screen_buffer_to_log();
     if(*current_cmd!=NULL)
     {
       command=0;
@@ -858,10 +1063,16 @@ int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	if(strchr(options,'M')!=NULL)
 	    command='M';
       }
-      screen_buffer_to_log();
     }
     else
-      command=screen_buffer_display(stdscr,options,menu_ntfs);
+    {
+      log_flush();
+#ifdef HAVE_NCURSES
+      command=screen_buffer_display_ext(stdscr, options, menu_ntfs, &menu);
+#else
+      command=0;
+#endif
+    }
     switch(command)
     {
       case 0:
@@ -876,6 +1087,7 @@ int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	  {
 	    display_message("Write error: Can't overwrite NTFS backup boot sector\n");
 	  }
+          disk_car->sync(disk_car);
 	  rescan=1;
 	}
 	break;
@@ -887,13 +1099,14 @@ int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	  {
 	    display_message("Write error: Can't overwrite NTFS boot sector\n");
 	  }
+          disk_car->sync(disk_car);
 	  rescan=1;
 	}
 	break;
       case 'L':
 	if(strchr(options,'O')==NULL && strchr(options,'B')!=NULL)
 	{
-	  io_redir_add_redir(disk_car,partition->part_offset,sizeof(buffer_backup_bs),0,buffer_backup_bs);
+	  io_redir_add_redir(disk_car,partition->part_offset,NTFS_BOOT_SECTOR_SIZE,0,buffer_backup_bs);
 	  dir_partition(disk_car, partition, 0,current_cmd);
 	  io_redir_del_redir(disk_car,partition->part_offset);
 	}
@@ -901,35 +1114,19 @@ int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
 	  dir_partition(disk_car, partition, 0,current_cmd);
 	break;
       case 'M':
-        repair_MFT(disk_car, partition, debug, current_cmd);
+        repair_MFT(disk_car, partition, verbose, expert, current_cmd);
 	break;
       case 'R': /* R : rebuild boot sector */
-	rebuild_NTFS_BS(disk_car,partition,debug,dump_ind,1,expert,current_cmd);
+	rebuild_NTFS_BS(disk_car,partition,verbose,dump_ind,1,expert,current_cmd);
 	rescan=1;
 	break;
       case 'D':
-	{
-	  WINDOW *window=newwin(0,0,0,0);	/* full screen */
-	  keypad(window, TRUE); /* Need it to get arrow key */
-	  aff_copy(window);
-	  wmove(window,4,0);
-	  wdoprintf(window,"%s",disk_car->description(disk_car));
-	  wmove(window,5,0);
-	  aff_part(window,AFF_PART_ORDER,disk_car,partition);
-	  log_info("Boot sector                        Backup boot sector\n");
-	  mvwaddstr(window,6,0, "Boot sector                        Backup boot sector");
-	  dump2(window,buffer_bs,buffer_backup_bs,NTFS_BOOT_SECTOR_SIZE);
-	  delwin(window);
-	  (void) clearok(stdscr, TRUE);
-#ifdef HAVE_TOUCHWIN
-	  touchwin(stdscr);
-#endif
-	}
+	dump_NTFS(disk_car, partition, buffer_bs, buffer_backup_bs);
 	break;
 #if 0
 #ifdef HAVE_LIBNTFS
       case 'U':
-	ntfs_undelete_part(disk_car, partition, debug);
+	ntfs_undelete_part(disk_car, partition, verbose);
 	break;
 #endif
 #endif
@@ -937,7 +1134,36 @@ int ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int debug,
   }
 }
 
-int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd)
+#ifdef HAVE_NCURSES
+static void hfs_dump_ncurses(disk_t *disk_car, const partition_t *partition, const unsigned char *buffer_bs, const unsigned char *buffer_backup_bs)
+{
+  WINDOW *window=newwin(0,0,0,0);	/* full screen */
+  keypad(window, TRUE); /* Need it to get arrow key */
+  aff_copy(window);
+  wmove(window,4,0);
+  wprintw(window,"%s",disk_car->description(disk_car));
+  wmove(window,5,0);
+  aff_part(window,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+  mvwaddstr(window,6,0, "Superblock                        Backup superblock");
+  dump2(window, buffer_bs, buffer_backup_bs, HFSP_BOOT_SECTOR_SIZE);
+  delwin(window);
+  (void) clearok(stdscr, TRUE);
+#ifdef HAVE_TOUCHWIN
+  touchwin(stdscr);
+#endif
+}
+#endif
+
+static void hfs_dump(disk_t *disk_car, const partition_t *partition, const unsigned char *buffer_bs, const unsigned char *buffer_backup_bs)
+{
+  log_info("Superblock                        Backup superblock\n");
+  dump2_log(buffer_bs, buffer_backup_bs, HFSP_BOOT_SECTOR_SIZE);
+#ifdef HAVE_NCURSES
+  hfs_dump_ncurses(disk_car, partition, buffer_bs, buffer_backup_bs);
+#endif
+}
+
+int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int verbose, const int dump_ind, const unsigned int expert, char **current_cmd)
 {
   unsigned char *buffer_bs;
   unsigned char *buffer_backup_bs;
@@ -958,6 +1184,7 @@ int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int deb
 
   while(1)
   {
+    unsigned int menu=0;
     int command;
     aff_buffer(BUFFER_RESET,"Q");
     if(rescan==1)
@@ -966,12 +1193,14 @@ int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int deb
       int opt_B=0;
       int opt_O=0;
       options="D";
+#ifdef HAVE_NCURSES
       aff_copy(stdscr);
       wmove(stdscr,4,0);
-      wdoprintf(stdscr,"%s",disk_car->description(disk_car));
+      wprintw(stdscr,"%s",disk_car->description(disk_car));
       mvwaddstr(stdscr,5,0,msg_PART_HEADER_LONG);
       wmove(stdscr,6,0);
-      aff_part(stdscr,AFF_PART_ORDER,disk_car,partition);
+      aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+#endif
       log_info("\nHFS_HFSP_boot_sector\n");
       log_partition(disk_car,partition);
       aff_buffer(BUFFER_ADD,"Superblock\n");
@@ -980,13 +1209,13 @@ int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int deb
 	aff_buffer(BUFFER_ADD,"Bad: can't read HFS/HFS+ superblock.\n");
 	memset(buffer_bs,0,HFSP_BOOT_SECTOR_SIZE);
       }
-      else if(test_HFSP(disk_car,(const struct hfsp_vh*)buffer_bs,partition,debug,0)==0)
+      else if(test_HFSP(disk_car,(const struct hfsp_vh*)buffer_bs,partition,verbose,0)==0)
       {
 	aff_buffer(BUFFER_ADD,"HFS+ OK\n");
 	opt_O=1;
 	opt_over=1;
       }
-      else if(test_HFS(disk_car,(const hfs_mdb_t*)buffer_bs,partition,debug,0)==0)
+      else if(test_HFS(disk_car,(const hfs_mdb_t*)buffer_bs,partition,verbose,0)==0)
       {
 	aff_buffer(BUFFER_ADD,"HFS Ok\n");
 	opt_O=1;
@@ -1000,13 +1229,13 @@ int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int deb
 	aff_buffer(BUFFER_ADD,"Bad: can't read HFS/HFS+ backup superblock.\n");
 	memset(buffer_backup_bs,0,HFSP_BOOT_SECTOR_SIZE);
       }
-      else if(test_HFSP(disk_car,(const struct hfsp_vh*)buffer_backup_bs,partition,debug,0)==0)
+      else if(test_HFSP(disk_car,(const struct hfsp_vh*)buffer_backup_bs,partition,verbose,0)==0)
       {
 	aff_buffer(BUFFER_ADD,"HFS+ OK\n");
 	opt_B=1;
 	opt_over=1;
       }
-      else if(test_HFS(disk_car,(const hfs_mdb_t*)buffer_backup_bs,partition,debug,0)==0)
+      else if(test_HFS(disk_car,(const hfs_mdb_t*)buffer_backup_bs,partition,verbose,0)==0)
       {
 	aff_buffer(BUFFER_ADD,"HFS Ok\n");
 	opt_B=1;
@@ -1035,6 +1264,7 @@ int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int deb
       }
       rescan=0;
     }
+    screen_buffer_to_log();
     if(*current_cmd!=NULL)
     {
       command=0;
@@ -1057,10 +1287,16 @@ int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int deb
 	if(strchr(options,'B')!=NULL)
 	    command='B';
       }
-      screen_buffer_to_log();
     }
     else
-      command=screen_buffer_display(stdscr,options,menu_hfsp);
+    {
+      log_flush();
+#ifdef HAVE_NCURSES
+      command=screen_buffer_display_ext(stdscr, options, menu_hfsp, &menu);
+#else
+      command=0;
+#endif
+    }
     switch(command)
     {
       case 0:
@@ -1075,6 +1311,7 @@ int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int deb
 	  {
 	    display_message("Write error: Can't overwrite HFS/HFS+ backup superblock\n");
 	  }
+          disk_car->sync(disk_car);
 	  rescan=1;
 	}
 	break;
@@ -1086,29 +1323,13 @@ int HFS_HFSP_boot_sector(disk_t *disk_car, partition_t *partition, const int deb
 	  {
 	    display_message("Write error: Can't overwrite HFS/HFS+ main superblock\n");
 	  }
+          disk_car->sync(disk_car);
 	  rescan=1;
 	}
 	break;
       case 'D':
-	{
-	  WINDOW *window=newwin(0,0,0,0);	/* full screen */
-	  keypad(window, TRUE); /* Need it to get arrow key */
-	  aff_copy(window);
-	  wmove(window,4,0);
-	  wdoprintf(window,"%s",disk_car->description(disk_car));
-	  wmove(window,5,0);
-	  aff_part(window,AFF_PART_ORDER,disk_car,partition);
-	  log_info("Superblock                        Backup superblock\n");
-	  mvwaddstr(window,6,0, "Superblock                        Backup superblock");
-	  dump2(window,buffer_bs,buffer_backup_bs,HFSP_BOOT_SECTOR_SIZE);
-	  delwin(window);
-	  (void) clearok(stdscr, TRUE);
-#ifdef HAVE_TOUCHWIN
-	  touchwin(stdscr);
-#endif
-	}
+	hfs_dump(disk_car, partition, buffer_bs, buffer_backup_bs);
 	break;
     }
   }
 }
-
