@@ -165,8 +165,8 @@ struct disklabel {
 #define TST_FS_VINUM        14              /* Vinum drive */
 #define TST_FS_RAID         15              /* RAIDFrame drive */
 #define TST_FS_JFS2         21              /* IBM JFS2 */
-int check_BSD(disk_t *disk_car,partition_t *partition,const int debug,const unsigned int max_partitions);
-int test_BSD(disk_t *disk_car, const struct disklabel*bsd_header,partition_t *partition,const int debug, const int dump_ind, const unsigned int max_partitions);
-int recover_BSD(disk_t *disk_car, const struct disklabel*bsd_header,partition_t *partition,const int debug, const int dump_ind);
+int check_BSD(disk_t *disk_car,partition_t *partition,const int verbose,const unsigned int max_partitions);
+int test_BSD(disk_t *disk_car, const struct disklabel*bsd_header,partition_t *partition,const int verbose, const int dump_ind, const unsigned int max_partitions);
+int recover_BSD(disk_t *disk_car, const struct disklabel*bsd_header,partition_t *partition,const int verbose, const int dump_ind);
 
 #endif
