@@ -26,6 +26,9 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "lang.h"
@@ -34,6 +37,10 @@
 #include "fnctdsk.h"
 #include "chgtype.h"
 #include "log.h"
+#include "guid_cmp.h"
+
+extern const arch_fnct_t arch_gpt;
+extern const arch_fnct_t arch_none;
 
 struct part_name_struct
 {
@@ -41,8 +48,22 @@ struct part_name_struct
   const char *name;
 };
 
-int change_part_type(const disk_t *disk_car,partition_t *partition, char **current_cmd)
+static void change_part_type_cli(const disk_t *disk_car,partition_t *partition, char **current_cmd)
 {
+  while(*current_cmd[0]==',')
+    (*current_cmd)++;
+  {
+    int tmp_val= strtol(*current_cmd, NULL, 16);
+    while(*current_cmd[0]!=',' && *current_cmd[0]!='\0')
+      (*current_cmd)++;
+    partition->arch->set_part_type(partition,tmp_val);
+  }
+}
+
+#ifdef HAVE_NCURSES
+static void change_part_type_ncurses(const disk_t *disk_car,partition_t *partition)
+{
+  partition_t *new_partition;
   char response[100];
   int size=0;
   int i;
@@ -56,13 +77,13 @@ int change_part_type(const disk_t *disk_car,partition_t *partition, char **curre
     { 0, NULL, NULL }
   };
   /* Create an index of all partition type except Intel extended */
-  partition_t *new_partition=partition_new();
+  new_partition=partition_new(NULL);
   dup_partition_t(new_partition,partition);
   for(i=0;i<=0xFF;i++)
   {
-    if(disk_car->arch->set_part_type(new_partition,i)==0)
+    if(partition->arch->set_part_type(new_partition,i)==0)
     {
-      part_name[size].name=disk_car->arch->get_partition_name(new_partition);
+      part_name[size].name=new_partition->arch->get_partition_typename(new_partition);
       if(part_name[size].name!=NULL)
 	part_name[size++].index=i;
     }
@@ -75,41 +96,61 @@ int change_part_type(const disk_t *disk_car,partition_t *partition, char **curre
   for (i = 2; i >= 0; i--)
     last[2 - i] = done += (size + i - done) / (i + 1);
   i = done = 0;
-  do {
+  while (done < last[0])
+  {
     aff_buffer(BUFFER_ADD, "%02x %-20s%c",  part_name[next].index, part_name[next].name,(i==2 ? '\n' : ' '));
     next = last[i++] + done;
     if (i > 2 || next >= last[i]) {
       i = 0;
       next = ++done;
     }
-  } while (done < last[0]);
+  }
 
   /* Ask for the new partition type*/
   aff_copy(stdscr);
   wmove(stdscr,4,0);
-  aff_part(stdscr,AFF_PART_ORDER,disk_car,partition);
-  if(*current_cmd==NULL)
-    screen_buffer_display(stdscr,"",menuType);
+  aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+  screen_buffer_display(stdscr,"",menuType);
   wmove(stdscr,23,0);
-  wdoprintf(stdscr,"New partition type [current %02x] ? ",disk_car->arch->get_part_type(partition));
-  if(*current_cmd!=NULL)
-  {
-    while(*current_cmd[0]==',')
-      (*current_cmd)++;
-    {
-      int tmp_val= strtol(*current_cmd, NULL, 16);
-      while(*current_cmd[0]!=',' && *current_cmd[0]!='\0')
-	(*current_cmd)++;
-      disk_car->arch->set_part_type(partition,tmp_val);
-    }
+  wprintw(stdscr,"New partition type [current %02x] ? ",partition->arch->get_part_type(partition));
+  if (get_string(response, sizeof(response), NULL) > 0) {
+    int tmp_val = strtol(response, NULL, 16);
+    partition->arch->set_part_type(partition,tmp_val);
   }
+}
+#endif
+
+void change_part_type(const disk_t *disk_car,partition_t *partition, char **current_cmd)
+{
+  const arch_fnct_t *arch=partition->arch;
+  if(partition->arch==NULL)
+  {
+    log_error("change_part_type arch==NULL\n");
+    return;
+  }
+  if(partition->arch==&arch_gpt)
+    partition->arch=&arch_none;
+  if(partition->arch->set_part_type==NULL)
+  {
+    log_error("change_part_type set_part_type==NULL\n");
+    partition->arch=arch;
+    return;
+  }
+  if(*current_cmd!=NULL)
+    change_part_type_cli(disk_car, partition, current_cmd);
   else
   {
-    if (get_string(response, sizeof(response), NULL) > 0) {
-      int tmp_val = strtol(response, NULL, 16);
-      disk_car->arch->set_part_type(partition,tmp_val);
-    }
+#ifdef HAVE_NCURSES
+    change_part_type_ncurses(disk_car, partition);
+#endif
   }
+  if(arch==&arch_gpt)
+  {
+    partition->arch=arch;
+    /* TODO: use upart_type to set part_type_gpt */
+    if(guid_cmp(partition->part_type_gpt, GPT_ENT_TYPE_UNUSED)==0)
+      partition->part_type_gpt=GPT_ENT_TYPE_MS_BASIC_DATA;
+  }
+  log_info("Change partition type:\n");
   log_partition(disk_car,partition);
-  return 0;
 }
