@@ -2,7 +2,7 @@
 
     File: chgtype.h
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,5 +21,5 @@
  */
 
 void change_geometry(disk_t *disk_car, char **current_cmd);
-int change_part_type(const disk_t *disk_car,partition_t *partition, char **current_cmd);
+void change_part_type(const disk_t *disk_car,partition_t *partition, char **current_cmd);
 
