@@ -25,6 +25,7 @@
 #include <config.h>
 #endif
  
+#include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -55,7 +56,9 @@ void *MALLOC(size_t size)
 #if defined(HAVE_POSIX_MEMALIGN)
   if(size>=512)
   {
-    /* Warning, memory leak checker must be posix_memalign aware, otherwise reports may look strange */
+    /* Warning, memory leak checker must be posix_memalign aware, otherwise  *
+     * reports may look strange. Aligned memory is required if the buffer is *
+     * used for read/write operation with a file opened with O_DIRECT        */
     if(posix_memalign(&res,4096,size)==0)
     {
       memset(res,0,size);
@@ -139,43 +142,110 @@ static unsigned int up2power_aux(const unsigned int number)
 	return(1+up2power_aux(number/2));
 }
 
-int check_volume_name(const char *name,const unsigned int max_size)
-{
-  unsigned int i;
-  for(i=0;name[i]!='\0' && i<max_size;i++)
-  {
-    if((name[i]>=0x6 && name[i]<=0x1f)||
-	(name[i]>=0x3A && name[i]<=0x3f))
-      return 1;
-    switch(name[i])
-    {
-      case 0x1:
-      case 0x2:
-      case 0x3:
-      case 0x4:
-      case 0x22:
-      case 0x2A:
-      case 0x2B:
-      case 0x2C:
-/*     case 0x2E: Pas sur */
-      case 0x2F:
-      case 0x5B:
-      case 0x5C:
-      case 0x5D:
-      case 0x7C:
-/*     case 'a' ... 'z': */
-	return 1;
-    }
-  }
-  return 0; /* Ok */
-}
-
 void set_part_name(partition_t *partition,const char *src,const int max_size)
 {
   int i;
   for(i=0;(i<max_size) && (src[i]!=(char)0);i++)
-    partition->name[i]=src[i];
-  partition->name[i--]='\0';
+    partition->fsname[i]=src[i];
+  partition->fsname[i--]='\0';
+}
+
+static inline unsigned char convert_char(unsigned char car)
+{
+#ifdef DJGPP
+  if(car<0x20)
+    return '_';
+  switch(car)
+  {
+    /* Forbidden */
+    case '<':
+    case '>':
+    case ':':
+    case '"':
+    /* case '/': subdirectory */
+    case '\\':
+    case '|':
+    case '?':
+    case '*':
+    /* Not recommanded */
+    case '[':
+    case ']':
+    case ';':
+    case ',':
+    case '+':
+    case '=':
+      return '_';
+  }
+  /* 'a' */
+  if(car>=224 && car<=230)      
+    return 'a';
+  /* 'c' */
+  if(car==231)
+    return 'c';
+  /* 'e' */
+  if(car>=232 && car<=235)
+    return 'e';
+  /* 'i' */
+  if(car>=236 && car<=239)
+    return 'n';
+  /* n */
+  if(car==241)
+    return 'n';
+  /* 'o' */
+  if((car>=242 && car<=246) || car==248)
+    return 'o';
+  /* 'u' */
+  if(car>=249 && car<=252)
+    return 'u';
+  /* 'y' */
+  if(car>=253)
+    return 'y';
+#elif defined(__CYGWIN__) || defined(__MINGW32__)
+  if(car<0x20)
+    return '_';
+  switch(car)
+  {
+    /* Forbidden */
+    case '<':
+    case '>':
+    case ':':
+    case '"':
+    /* case '/': subdirectory */
+    case '\\':
+    case '|':
+    case '?':
+    case '*':
+    /* Not recommanded */
+    case '[':
+    case ']':
+    case ';':
+    case ',':
+    case '+':
+    case '=':
+      return '_';
+  }
+#endif
+  return car;
+}
+
+unsigned int filename_convert(char *dst, const char*src, const unsigned int n)
+{
+  unsigned int i;
+  for(i=0;i<n-1 && src[i]!='\0';i++)
+  {
+    if(i==1 && src[i]==':')
+      dst[i]=src[i];
+    else
+      dst[i]=convert_char(src[i]);
+  }
+#if defined(DJGPP) || defined(__CYGWIN__) || defined(__MINGW32__)
+  while(i>0 && (dst[i-1]==' '||dst[i-1]=='.'))
+    i--;
+  if(i==0 && (dst[i]==' '||dst[i]=='.'))
+    dst[i++]='_';
+#endif
+  dst[i]='\0';
+  return i;
 }
 
 void create_dir(const char *dir_name, const unsigned int is_dir_name)
@@ -204,7 +274,7 @@ void create_dir(const char *dir_name, const unsigned int is_dir_name)
       mkdir(path, 0775);
 #endif
 #else
-#error You need a mkdir function!
+#warn You need a mkdir function!
 #endif
     }
   } while(pos!=NULL);
