@@ -36,10 +36,10 @@
 #include "fnctdsk.h"
 #include "log.h"
 
-static int set_cramfs_info(const disk_t *disk_car, const struct cramfs_super *sb,partition_t *partition, const int debug, const int dump_ind);
-static int test_cramfs(const disk_t *disk_car, const struct cramfs_super *sb,partition_t *partition,const int debug, const int dump_ind);
+static int set_cramfs_info(const disk_t *disk_car, const struct cramfs_super *sb,partition_t *partition, const int verbose, const int dump_ind);
+static int test_cramfs(const disk_t *disk_car, const struct cramfs_super *sb,partition_t *partition,const int verbose, const int dump_ind);
 
-int check_cramfs(disk_t *disk_car,partition_t *partition,const int debug)
+int check_cramfs(disk_t *disk_car,partition_t *partition,const int verbose)
 {
   unsigned char *buffer=(unsigned char*)MALLOC(CRAMFS_SUPERBLOCK_SIZE);
   if(disk_car->read(disk_car,CRAMFS_SUPERBLOCK_SIZE, buffer, partition->part_offset+0x200)!=0)
@@ -47,9 +47,9 @@ int check_cramfs(disk_t *disk_car,partition_t *partition,const int debug)
     free(buffer);
     return 1;
   }
-  if(test_cramfs(disk_car,(struct cramfs_super*)buffer,partition,debug,0)==0)
+  if(test_cramfs(disk_car,(struct cramfs_super*)buffer,partition,verbose,0)==0)
   {
-    set_cramfs_info(disk_car,(struct cramfs_super*)buffer,partition,debug,0);
+    set_cramfs_info(disk_car,(struct cramfs_super*)buffer,partition,verbose,0);
     free(buffer);
     return 0;
   }
@@ -57,24 +57,24 @@ int check_cramfs(disk_t *disk_car,partition_t *partition,const int debug)
   return 1;
 }
 
-static int test_cramfs(const disk_t *disk_car, const struct cramfs_super *sb,partition_t *partition,const int debug, const int dump_ind)
+static int test_cramfs(const disk_t *disk_car, const struct cramfs_super *sb,partition_t *partition,const int verbose, const int dump_ind)
 {
-  if (sb->magic==CRAMFS_MAGIC)
+  if (sb->magic==le32(CRAMFS_MAGIC))
   {
     partition->upart_type = UP_CRAMFS;
   }
   else
     return 1;
-  if(debug>0)
+  if(verbose>0)
     log_info("\ncramfs Marker at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
   return 0;
 }
 
-int recover_cramfs(disk_t *disk_car, const struct cramfs_super *sb,partition_t *partition,const int debug, const int dump_ind)
+int recover_cramfs(disk_t *disk_car, const struct cramfs_super *sb,partition_t *partition,const int verbose, const int dump_ind)
 {
-  if(test_cramfs(disk_car,sb,partition,debug,dump_ind)!=0)
+  if(test_cramfs(disk_car,sb,partition,verbose,dump_ind)!=0)
     return 1;
-  if(debug>0 || dump_ind!=0)
+  if(verbose>0 || dump_ind!=0)
   {
     log_trace("\nrecover_cramfs\n");
     if(dump_ind!=0)
@@ -86,11 +86,12 @@ int recover_cramfs(disk_t *disk_car, const struct cramfs_super *sb,partition_t *
   partition->part_type_i386 = P_LINUX;
   partition->part_type_mac= PMAC_LINUX;
   partition->part_type_sun= PSUN_LINUX;
-  set_cramfs_info(disk_car,sb,partition,debug,dump_ind);
+  partition->part_type_gpt= GPT_ENT_TYPE_LINUX_DATA;
+  set_cramfs_info(disk_car,sb,partition,verbose,dump_ind);
   return 0;
 }
 
-static int set_cramfs_info(const disk_t *disk_car,const struct cramfs_super *sb,partition_t *partition, const int debug, const int dump_ind)
+static int set_cramfs_info(const disk_t *disk_car,const struct cramfs_super *sb,partition_t *partition, const int verbose, const int dump_ind)
 {
   set_part_name(partition,sb->name,16);
   switch(partition->upart_type)
