@@ -79,5 +79,5 @@ struct cramfs_super {
 };
 
 
-int check_cramfs(disk_t *disk_car,partition_t *partition,const int debug);
-int recover_cramfs(disk_t *disk_car, const struct cramfs_super *sb,partition_t *partition,const int debug, const int dump_ind);
+int check_cramfs(disk_t *disk_car,partition_t *partition,const int verbose);
+int recover_cramfs(disk_t *disk_car, const struct cramfs_super *sb,partition_t *partition,const int verbose, const int dump_ind);
