@@ -2,7 +2,7 @@
 
     File: dir.c
 
-    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,25 +36,36 @@
 #include <sys/stat.h>
 #endif
 #include "types.h"
+#ifdef HAVE_UTIME_H
+#include <utime.h>
+#endif
 #include "common.h"
 #include "fat.h"
 #include "lang.h"
 #include "fnctdsk.h"
 #include "testdisk.h"
 #include "intrf.h"
+#ifdef HAVE_NCURSES
+#include "intrfn.h"
+#else
+#include <stdio.h>
+#endif
 #include "dir.h"
 #include "ext2_dir.h"
 #include "fat_dir.h"
 #include "ntfs_dir.h"
 #include "rfs_dir.h"
+#include "log.h"
 
-static int dir_partition_aux(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode, const int first_time, char **current_cmd);
-static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const t_file_data*dir_list, const unsigned long int inode, const int first_time, char**current_cmd);
-static int copy_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode);
-
-static const char *monstr[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
+const char *monstr[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
 				"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
+
+static int dir_partition_aux(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode, const int first_time, char **current_cmd);
+static int copy_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t *dir);
 static char ftypelet (unsigned int bits);
+#ifdef HAVE_NCURSES
+static long int dir_aff_ncurses(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t*dir_list, const unsigned long int inode, const int first_time);
+#endif
 
 static char ftypelet (unsigned int bits)
 {
@@ -154,14 +165,14 @@ void mode_string (const unsigned int mode, char *str)
 #endif
 }
 
-int dir_aff_log(const t_param_disk *disk_car, const t_partition *partition, const t_dir_data *dir_data, const t_file_data*dir_list)
+int dir_aff_log(const disk_t *disk_car, const partition_t *partition, const dir_data_t *dir_data, const file_data_t*dir_list)
 {
   int test_date=0;
-  const t_file_data *current_file;
-  aff_part_rapport(disk_car,partition);
+  const file_data_t *current_file;
+  log_partition(disk_car,partition);
   if(dir_data!=NULL)
   {
-    ecrit_rapport("Directory %s\n",dir_data->current_directory);
+    log_info("Directory %s\n",dir_data->current_directory);
   }
   for(current_file=dir_list;current_file!=NULL;current_file=current_file->next)
   {
@@ -185,16 +196,17 @@ int dir_aff_log(const t_param_disk *disk_car, const t_partition *partition, cons
       strncpy(datestr, "                 ",sizeof(datestr));
     }
     mode_string(current_file->filestat.st_mode,str);
-    ecrit_rapport("%7lu ",(unsigned long int)current_file->filestat.st_ino);
-    ecrit_rapport("%s %5u  %5u   ", 
+    log_info("%7lu ",(unsigned long int)current_file->filestat.st_ino);
+    log_info("%s %5u  %5u   ", 
 	str, (unsigned int)current_file->filestat.st_uid, (unsigned int)current_file->filestat.st_gid);
-    ecrit_rapport("%7llu", (long long unsigned int)current_file->filestat.st_size);
-    ecrit_rapport(" %s %s\n", datestr, current_file->name);
+    log_info("%7llu", (long long unsigned int)current_file->filestat.st_size);
+    log_info(" %s %s\n", datestr, current_file->name);
   }
   return test_date;
 }
 
-static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const t_file_data*dir_list, const unsigned long int inode, const int first_time, char **current_cmd)
+#ifdef HAVE_NCURSES
+static long int dir_aff_ncurses(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t*dir_list, const unsigned long int inode, const int first_time)
 {
   /* Return value
    * -1: quit
@@ -204,62 +216,59 @@ static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_
   int quit=0;
   int offset=0;
   int pos_num=0;
-  const t_file_data *current_file;
-  const t_file_data *pos=dir_list;
-  aff_copy(dir_data->window);
-  wmove(dir_data->window,4,0);
-  aff_part(dir_data->window,AFF_PART_ORDER,disk_car,partition);
-  mvwaddstr(dir_data->window,5,0,"Use ");
+  const file_data_t *current_file;
+  const file_data_t *pos=dir_list;
+  WINDOW *window=(WINDOW*)dir_data->display;
+  aff_copy(window);
+  wmove(window,4,0);
+  aff_part(window,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
+  mvwaddstr(window,5,0,"Use ");
   if(first_time==0)
   {
     if(has_colors())
-      wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(0));
-    waddstr(dir_data->window, "Left");
+      wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
+    waddstr(window, "Left");
     if(has_colors())
-      wbkgdset(dir_data->window,' ' | COLOR_PAIR(0));
-    waddstr(dir_data->window," arrow to go back, ");
+      wbkgdset(window,' ' | COLOR_PAIR(0));
+    waddstr(window," arrow to go back, ");
   }
   if(has_colors())
-    wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(0));
-  waddstr(dir_data->window,"Right");
+    wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
+  waddstr(window,"Right");
   if(has_colors())
-    wbkgdset(dir_data->window,' ' | COLOR_PAIR(0));
-  waddstr(dir_data->window," arrow to change directory, ");
+    wbkgdset(window,' ' | COLOR_PAIR(0));
+  waddstr(window," arrow to change directory, ");
   if(dir_data->copy_file!=NULL)
   {
     if(has_colors())
-      wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(0));
-    waddstr(dir_data->window,"c");
+      wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
+    waddstr(window,"c");
     if(has_colors())
-      wbkgdset(dir_data->window,' ' | COLOR_PAIR(0));
-    waddstr(dir_data->window," to copy, ");
+      wbkgdset(window,' ' | COLOR_PAIR(0));
+    waddstr(window," to copy, ");
   }
   if(has_colors())
-    wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(0));
-  waddstr(dir_data->window,"q");
+    wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
+  waddstr(window,"q");
   if(has_colors())
-    wbkgdset(dir_data->window,' ' | COLOR_PAIR(0));
-  waddstr(dir_data->window," to quit");
-  wmove(dir_data->window,6,0);
-  wdoprintf(dir_data->window,"Directory %s\n",dir_data->current_directory);
+    wbkgdset(window,' ' | COLOR_PAIR(0));
+  waddstr(window," to quit");
+  wmove(window,6,0);
+  wprintw(window,"Directory %s\n",dir_data->current_directory);
   do
   {
     int i;
     int car;
     for(i=0,current_file=dir_list;(current_file!=NULL) && (i<offset);current_file=current_file->next,i++);
-    wmove(dir_data->window, 8-1, 4);
-    wclrtoeol(dir_data->window);
-    if(offset>0)
-      wdoprintf(dir_data->window, "Previous");
     for(i=offset;(current_file!=NULL) &&((i-offset)<INTER_DIR);i++,current_file=current_file->next)
     {
       struct tm		*tm_p;
       char str[11];
       char		datestr[80];
-      wmove(dir_data->window, 8+i-offset, 0);
-      wclrtoeol(dir_data->window);	/* before addstr for BSD compatibility */
+      wmove(window, 8+i-offset, 0);
+      wclrtoeol(window);	/* before addstr for BSD compatibility */
       if(current_file==pos)
-	wattrset(dir_data->window,A_STANDOUT);
+	wattrset(window, A_REVERSE);
       if(current_file->filestat.st_mtime!=0)
       {
 	tm_p = localtime(&current_file->filestat.st_mtime);
@@ -272,35 +281,36 @@ static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_
 	strncpy(datestr, "                 ",sizeof(datestr));
       }
       mode_string(current_file->filestat.st_mode,str);
-      wdoprintf(dir_data->window, "%s %5u %5u   ", 
+      wprintw(window, "%s %5u %5u   ", 
 	  str, (unsigned int)current_file->filestat.st_uid, (unsigned int)current_file->filestat.st_gid);
-      wdoprintf(dir_data->window, "%7llu", (long long unsigned int)current_file->filestat.st_size);
+      wprintw(window, "%7llu", (long long unsigned int)current_file->filestat.st_size);
       /* screen may overlap due to long filename */
-      wdoprintf(dir_data->window, " %s %s", datestr, current_file->name);
+      wprintw(window, " %s %s", datestr, current_file->name);
       if(current_file==pos)
-	wattroff(dir_data->window,A_STANDOUT);
+	wattroff(window, A_REVERSE);
     }
+    wmove(window, 8-1, 51);
+    wclrtoeol(window);
+    if(offset>0)
+      wprintw(window, "Previous");
     /* Clear the last line, useful if overlapping */
-    wmove(dir_data->window,8+i-offset,0);
-    wclrtoeol(dir_data->window);
-    wmove(dir_data->window, 8+INTER_DIR, 4);
-    wclrtoeol(dir_data->window);
+    wmove(window,8+i-offset,0);
+    wclrtoeol(window);
+    wmove(window, 8+INTER_DIR, 51);
+    wclrtoeol(window);
     if(current_file!=NULL)
-      wdoprintf(dir_data->window, "Next");
+      wprintw(window, "Next");
     if(dir_list==NULL)
     {
-      wmove(dir_data->window,8,0);
-      wdoprintf(dir_data->window,"No file found, filesystem seems damaged.");
+      wmove(window,8,0);
+      wprintw(window,"No file found, filesystem seems damaged.");
     }
-    wrefresh(dir_data->window);
+    wrefresh(window);
     /* Using gnome terminal under FC3, TERM=xterm, the screen is not always correct */
-    wredrawln(dir_data->window,0,getmaxy(dir_data->window));	/* redrawwin def is boggus in pdcur24 */
-    if(*current_cmd!=NULL)
-      car='q';
-    else
-      car=wgetch(dir_data->window);
-    wmove(dir_data->window,7,0);
-    wclrtoeol(dir_data->window);
+    wredrawln(window,0,getmaxy(window));	/* redrawwin def is boggus in pdcur24 */
+    car=wgetch(window);
+    wmove(window,7,0);
+    wclrtoeol(window);
     switch(car)
     {
       case key_ESC:
@@ -407,41 +417,41 @@ static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_
 	      if(dir_data->local_dir!=NULL)
 	      {
 		int res=-1;
-		wmove(dir_data->window,7,0);
-		wclrtoeol(dir_data->window);
+		wmove(window,7,0);
+		wclrtoeol(window);
 		if(has_colors())
-		  wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(1));
-		wdoprintf(dir_data->window,"Copying, please wait...");
+		  wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(1));
+		wprintw(window,"Copying, please wait...");
 		if(has_colors())
-		  wbkgdset(dir_data->window,' ' | COLOR_PAIR(0));
-		wrefresh(dir_data->window);
+		  wbkgdset(window,' ' | COLOR_PAIR(0));
+		wrefresh(window);
 		if(LINUX_S_ISDIR(pos->filestat.st_mode)!=0)
 		{
-		  res=copy_dir(disk_car, partition, dir_data, (unsigned long int)pos->filestat.st_ino);
+		  res=copy_dir(disk_car, partition, dir_data, pos);
 		}
 		else if(LINUX_S_ISREG(pos->filestat.st_mode)!=0)
 		{
-		  res=dir_data->copy_file(disk_car, partition, dir_data, (unsigned long int)pos->filestat.st_ino);
+		  res=dir_data->copy_file(disk_car, partition, dir_data, pos);
 		}
-		wmove(dir_data->window,7,0);
-		wclrtoeol(dir_data->window);
-		if(res<0)
+		wmove(window,7,0);
+		wclrtoeol(window);
+		if(res < -1)
 		{
 		  if(has_colors())
-		    wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(1));
-		  wdoprintf(dir_data->window,"Copy failed!");
+		    wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(1));
+		  wprintw(window,"Copy failed!");
 		}
 		else
 		{
 		  if(has_colors())
-		    wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(2));
-		  if(res>0)
-		    wdoprintf(dir_data->window,"Copy done! (Failed to copy some files)");
+		    wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(2));
+		  if(res < 0)
+		    wprintw(window,"Copy done! (Failed to copy some files)");
 		  else
-		    wdoprintf(dir_data->window,"Copy done!");
+		    wprintw(window,"Copy done!");
 		}
 		if(has_colors())
-		  wbkgdset(dir_data->window,' ' | COLOR_PAIR(0));
+		  wbkgdset(window,' ' | COLOR_PAIR(0));
 	      }
 	      dir_data->current_directory[current_directory_namelength]='\0';
 	    }
@@ -452,61 +462,43 @@ static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_
   } while(quit==0);
   return -1;
 }
+#endif
 
-void dir_aff_entry(WINDOW *window, struct file_info *dir_info)
+void delete_list_file(file_data_t *file_list)
 {
-  struct tm		*tm_p;
-  char str[11];
-  char		datestr[80];
-  if(dir_info->stat.st_mtime!=0)
-  {
-    tm_p = localtime(&dir_info->stat.st_mtime);
-    snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
-	tm_p->tm_mday, monstr[tm_p->tm_mon],
-	1900 + tm_p->tm_year, tm_p->tm_hour,
-	tm_p->tm_min);
-    /* May have to use %d instead of %e */
-  } else {
-    strncpy(datestr, "                 ",sizeof(datestr));
-  }
-  mode_string(dir_info->stat.st_mode,str);
-  wdoprintf(window, "%s %5u %5u   ", 
-      str, (unsigned int)dir_info->stat.st_uid, (unsigned int)dir_info->stat.st_gid);
-  wdoprintf(window, "%7llu", (long long unsigned int)dir_info->stat.st_size);
-  /* screen may overlap due to long filename */
-  wdoprintf(window, " %s %s", datestr, dir_info->name);
-}
-
-void delete_list_file(t_file_data *file_list)
-{
-  t_file_data *current_file=file_list;
+  file_data_t *current_file=file_list;
   while(current_file!=NULL)
   {
-    t_file_data *next=current_file->next;
+    file_data_t *next=current_file->next;
     free(current_file);
     current_file=next;
   }
 }
 
-int dir_partition_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode, char **current_cmd)
+int dir_partition_aff(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode, char **current_cmd)
 {
   if(dir_data==NULL)
     return -1;
   return dir_partition_aux(disk_car,partition,dir_data,inode,1,current_cmd);
 }
 
-static int dir_partition_aux(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode, const int first_time, char**current_cmd)
+static int dir_partition_aux(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode, const int first_time, char**current_cmd)
 {
-  t_file_data *dir_list;
-  long int new_inode;
-  if(dir_data->debug>0)
-    ecrit_rapport("\ndir_partition inode=%ld\n",inode);
+  file_data_t *dir_list;
+  long int new_inode=-1;
+  if(dir_data->verbose>0)
+    log_info("\ndir_partition inode=%ld\n",inode);
   dir_list=dir_data->get_dir(disk_car,partition,dir_data,inode);
   dir_aff_log(disk_car, partition, dir_data, dir_list);
   do
   {
     unsigned int current_directory_namelength=strlen(dir_data->current_directory);
-    new_inode=dir_aff(disk_car,partition,dir_data,dir_list,inode,first_time,current_cmd);
+    if(*current_cmd==NULL)
+    {
+#ifdef HAVE_NCURSES
+      new_inode=dir_aff_ncurses(disk_car,partition,dir_data,dir_list,inode,first_time);
+#endif
+    }
     if(new_inode==0 || new_inode>1)
     {
       if(dir_partition_aux(disk_car, partition, dir_data, (unsigned long int)new_inode,0,current_cmd)<0)
@@ -522,25 +514,31 @@ static int dir_partition_aux(t_param_disk *disk_car, const t_partition *partitio
   return new_inode;
 }
 
-static int copy_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode)
+/*
+Returns
+-2: no file copied
+-1: failed to copy some files
+0: all files has been copied
+*/
+static int copy_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t *dir)
 {
-  unsigned int file_failed=0;
-  unsigned int file_copied=0;
-  t_file_data *dir_list;
+  file_data_t *dir_list;
   unsigned int current_directory_namelength=strlen(dir_data->current_directory);
-  t_file_data *current_file;
+  file_data_t *current_file;
+  char *dir_name;
+  int copy_bad=0;
+  int copy_ok=0;
   if(dir_data->get_dir==NULL || dir_data->copy_file==NULL)
-    return -1;
-  ecrit_rapport("copy_dir %s\n",dir_data->current_directory);
+    return -2;
   {
-    char *dir_name=MALLOC(strlen(dir_data->local_dir)+strlen(dir_data->current_directory)+1);
-    strcpy(dir_name,dir_data->local_dir);
-    strcat(dir_name,dir_data->current_directory);
-    create_dir(dir_name,1);
-    free(dir_name);
+    int l1=strlen(dir_data->local_dir);
+    int l2=strlen(dir_data->current_directory);
+    dir_name=MALLOC(l1+l2+1);
+    memcpy(dir_name,dir_data->local_dir,l1);
+    filename_convert(dir_name+l1,dir_data->current_directory,l2+1);
   }
-  dir_list=dir_data->get_dir(disk_car,partition,dir_data,inode);
-//  dir_aff_log(disk_car, partition, dir_data, dir_list);
+  create_dir(dir_name,1);
+  dir_list=dir_data->get_dir(disk_car, partition,dir_data, (const unsigned long int)dir->filestat.st_ino);
   for(current_file=dir_list;current_file!=NULL;current_file=current_file->next)
   {
     if(strlen(dir_data->current_directory)+1+strlen(current_file->name)<sizeof(dir_data->current_directory)-1)
@@ -550,23 +548,32 @@ static int copy_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_
       strcat(dir_data->current_directory,current_file->name);
       if(LINUX_S_ISDIR(current_file->filestat.st_mode)!=0)
       {
-	if((unsigned long int)current_file->filestat.st_ino!=inode &&
+	int tmp=0;
+	if(current_file->filestat.st_ino != dir->filestat.st_ino &&
 	    strcmp(current_file->name,"..")!=0 && strcmp(current_file->name,".")!=0)
-	  copy_dir(disk_car, partition, dir_data, (unsigned long int)current_file->filestat.st_ino);
+	  tmp=copy_dir(disk_car, partition, dir_data, current_file);
+	if(tmp>=-1)
+	  copy_ok=1;
+	if(tmp<0)
+	  copy_bad=1;
       }
       else if(LINUX_S_ISREG(current_file->filestat.st_mode)!=0)
       {
-//	ecrit_rapport("copy_file %s\n",dir_data->current_directory);
-	if(dir_data->copy_file(disk_car, partition, dir_data, (unsigned long int)current_file->filestat.st_ino))
-	  file_failed++;
+//	log_trace("copy_file %s\n",dir_data->current_directory);
+	int tmp;
+	tmp=dir_data->copy_file(disk_car, partition, dir_data, current_file);
+	if(tmp==0)
+	  copy_ok=1;
 	else
-	  file_copied++;
+	  copy_bad=1;
       }
       dir_data->current_directory[current_directory_namelength]='\0';
     }
   }
   delete_list_file(dir_list);
-  return (file_copied>0?(file_failed>0?1:0):-1);
+  set_date(dir_name, dir->filestat.st_atime, dir->filestat.st_mtime);
+  free(dir_name);
+  return (copy_bad>0?(copy_ok>0?-1:-2):0);
 }
 
 FILE *create_file(const char *filename)
@@ -580,3 +587,32 @@ FILE *create_file(const char *filename)
   }
   return f_out;
 }
+
+/**
+ * set_date - Set the file's date and time
+ * @pathname:  Path and name of the file to alter
+ * @actime:    Date and time to set
+ * @modtime:   Date and time to set
+ *
+ * Give a file a particular date and time.
+ *
+ * Return:  1  Success, set the file's date and time
+ *	    0  Error, failed to change the file's date and time
+ */
+int set_date(const char *pathname, time_t actime, time_t modtime)
+{
+#ifdef HAVE_UTIME
+  struct utimbuf ut;
+  if (!pathname)
+    return 0;
+  ut.actime  = actime;
+  ut.modtime = modtime;
+  if (utime(pathname, &ut)) {
+    log_error("ERROR: Couldn't set the file's date and time for %s\n", pathname);
+    return 0;
+  }
+  return 1;
+#else
+  return 0;
+#endif
+}
