@@ -2,7 +2,7 @@
 
     File: edit.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,21 +34,27 @@
 #include "common.h"
 #include "lang.h"
 #include "intrf.h"
+#ifdef HAVE_NCURSES
 #include "intrfn.h"
+#else
+#include <stdio.h>
+#endif
 #include "fnctdsk.h"
 #include "edit.h"
 #include "log.h"
 
+#ifdef HAVE_NCURSES
 static void interface_editor_position(const disk_t *disk_car, uint64_t *lba);
 static int dump_editor(const unsigned char *nom_dump,const unsigned int lng, const int menu_pos);
+static void interface_editor_ncurses(disk_t *disk_car);
 
-void interface_editor(disk_t *disk_car)
+static void interface_editor_ncurses(disk_t *disk_car)
 {
-  int done = FALSE;
+  int done = 0;
   uint64_t hd_offset=0;
   unsigned char *buffer=MALLOC(disk_car->sector_size);
   log_info("%s\n",disk_car->description(disk_car));
-  while (done==FALSE)
+  while (done==0)
   {
     static struct MenuItem menuEditor[]=
     {
@@ -71,11 +77,11 @@ void interface_editor(disk_t *disk_car)
 	  {
 	    wmove(stdscr,4,0);
 	    wclrtoeol(stdscr);
-	    wdoprintf(stdscr,"%lu ", (unsigned long)(hd_offset/disk_car->sector_size));
+	    wprintw(stdscr,"%lu ", (unsigned long)(hd_offset/disk_car->sector_size));
 	    aff_LBA2CHS(disk_car,hd_offset/disk_car->sector_size);
 	    if(disk_car->read(disk_car,disk_car->sector_size, buffer, hd_offset))
 	    {
-	      wdoprintf(stdscr,msg_PART_RD_ERR);
+	      wprintw(stdscr,msg_PART_RD_ERR);
 	    }
 	    {
 	      menu_pos=dump_editor(buffer,disk_car->sector_size,menu_pos);
@@ -94,18 +100,18 @@ void interface_editor(disk_t *disk_car)
 		    menu_pos=KEY_UP;
 		  break;
 		default:
-		  done = TRUE;
+		  done = 1;
 		  break;
 	      }
 	    }
 	  }
-	  done = FALSE;
+	  done = 0;
 	}
 	break;
       case key_ESC:
       case 'q':
       case 'Q':
-	done = TRUE;
+	done = 1;
 	break;
     }
   }
@@ -115,7 +121,7 @@ void interface_editor(disk_t *disk_car)
 static void interface_editor_position(const disk_t *disk_car,uint64_t *lba)
 {
   CHS_t position;
-  int done = FALSE;
+  int done = 0;
   char def[LINE_LENGTH];
   char response[LINE_LENGTH];
   unsigned int tmp_val;
@@ -123,7 +129,7 @@ static void interface_editor_position(const disk_t *disk_car,uint64_t *lba)
   position.cylinder=offset2cylinder(disk_car,*lba);
   position.head=offset2head(disk_car,*lba);
   position.sector=offset2sector(disk_car,*lba);
-  while (done==FALSE) {
+  while (done==0) {
 	static struct MenuItem menuGeometry[]=
 	{
 	  { 'c', "Cylinders", "Change cylinder" },
@@ -146,7 +152,7 @@ static void interface_editor_position(const disk_t *disk_car,uint64_t *lba)
 		  if (tmp_val <= disk_car->CHS.cylinder) {
 			position.cylinder = tmp_val;
 		  } else
-			wdoprintf(stdscr,"Illegal cylinders value");
+			wprintw(stdscr,"Illegal cylinders value");
 		}
 		break;
 	  case 'h':
@@ -158,7 +164,7 @@ static void interface_editor_position(const disk_t *disk_car,uint64_t *lba)
 		  if (tmp_val <= disk_car->CHS.head) {
 			position.head = tmp_val;
 		  } else
-			wdoprintf(stdscr,"Illegal heads value");
+			wprintw(stdscr,"Illegal heads value");
 		}
 		break;
 	  case 's':
@@ -170,13 +176,13 @@ static void interface_editor_position(const disk_t *disk_car,uint64_t *lba)
 		  if (tmp_val > 0 && tmp_val <= disk_car->CHS.sector ) {
 			position.sector = tmp_val;
 		  } else
-			wdoprintf(stdscr,"Illegal sectors value");
+			wprintw(stdscr,"Illegal sectors value");
 		}
 		break;
 	  case key_ESC:
 	  case 'd':
 	  case 'D':
-		done = TRUE;
+		done = 1;
 		break;
 	}
   }
@@ -209,22 +215,22 @@ static int dump_editor(const unsigned char *nom_dump,const unsigned int lng, con
 	{
 	  wmove(stdscr,DUMP_Y+i-pos,DUMP_X);
 	  wclrtoeol(stdscr);
-	  wdoprintf(stdscr,"%04X ",i*0x10);
+	  wprintw(stdscr,"%04X ",i*0x10);
 	  for(j=0; j< 0x10;j++)
 	  {
 		car=*(nom_dump+i*0x10+j);
-		wdoprintf(stdscr,"%02x", car);
+		wprintw(stdscr,"%02x", car);
 		if(j%4==(4-1))
-		  wdoprintf(stdscr," ");
+		  wprintw(stdscr," ");
 	  }
-	  wdoprintf(stdscr,"  ");
+	  wprintw(stdscr,"  ");
 	  for(j=0; j< 0x10;j++)
 	  {
 		car=*(nom_dump+i*0x10+j);
 		if ((car<32)||(car >= 127))
-		  wdoprintf(stdscr,".");
+		  wprintw(stdscr,".");
 		else
-		  wdoprintf(stdscr,"%c",  car);
+		  wprintw(stdscr,"%c",  car);
 	  }
 	}
 	switch (wmenuSelect(stdscr,INTER_DUMP_Y, INTER_DUMP_X, menuDump, 8, "PNQ", MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu))
@@ -274,4 +280,11 @@ static int dump_editor(const unsigned char *nom_dump,const unsigned int lng, con
   } while(done==0);
   return done;
 }
+#endif
 
+void interface_editor(disk_t *disk_car)
+{
+#ifdef HAVE_NCURSES
+  interface_editor_ncurses(disk_car);
+#endif
+}
