@@ -34,12 +34,12 @@
 #include "ext2.h"
 #include "fnctdsk.h"
 #include "log.h"
+#include "guid_cpy.h"
 
-#define EXT2_BLOCK_SIZE(s)      (EXT2_MIN_BLOCK_SIZE << (s)->s_log_block_size)
-static int test_EXT2(disk_t *disk_car, const struct ext2_super_block *sb,partition_t *partition,const int debug, const int dump_ind);
-static int set_EXT2_info(disk_t *disk_car, const struct ext2_super_block *sb,partition_t *partition,const int debug, const int dump_ind);
+static int test_EXT2(disk_t *disk_car, const struct ext2_super_block *sb,partition_t *partition,const int verbose, const int dump_ind);
+static int set_EXT2_info(disk_t *disk_car, const struct ext2_super_block *sb,partition_t *partition,const int verbose, const int dump_ind);
 
-int check_EXT2(disk_t *disk_car,partition_t *partition,const int debug)
+int check_EXT2(disk_t *disk_car,partition_t *partition,const int verbose)
 {
   unsigned char *buffer=(unsigned char*)MALLOC(EXT2_SUPERBLOCK_SIZE);
   if(disk_car->read(disk_car,EXT2_SUPERBLOCK_SIZE, buffer, partition->part_offset+0x400)!=0)
@@ -47,17 +47,17 @@ int check_EXT2(disk_t *disk_car,partition_t *partition,const int debug)
     free(buffer);
     return 1;
   }
-  if(test_EXT2(disk_car,(struct ext2_super_block*)buffer,partition,debug,0)!=0)
+  if(test_EXT2(disk_car,(struct ext2_super_block*)buffer,partition,verbose,0)!=0)
   {
     free(buffer);
     return 1;
   }
-  set_EXT2_info(disk_car,(struct ext2_super_block*)buffer,partition,debug,0);
+  set_EXT2_info(disk_car,(struct ext2_super_block*)buffer,partition,verbose,0);
   free(buffer);
   return 0;
 }
 
-static int set_EXT2_info(disk_t *disk_car, const struct ext2_super_block *sb,partition_t *partition,const int debug, const int dump_ind)
+static int set_EXT2_info(disk_t *disk_car, const struct ext2_super_block *sb,partition_t *partition,const int verbose, const int dump_ind)
 {
   set_part_name(partition,sb->s_volume_name,16);
   /* sb->s_last_mounted seems to be unemployed in kernel 2.2.16 */
@@ -76,7 +76,7 @@ static int set_EXT2_info(disk_t *disk_car, const struct ext2_super_block *sb,par
   if(le16(sb->s_block_group_nr)!=0)
   {
     strcat(partition->info," Backup superblock");
-    if(debug>0)
+    if(verbose>0)
     {
       log_warning("\nblock_group_nr %u\n",le16(sb->s_block_group_nr));
     }
@@ -89,51 +89,58 @@ static int set_EXT2_info(disk_t *disk_car, const struct ext2_super_block *sb,par
 Primary superblock is at 1024 (SUPERBLOCK_OFFSET)
 Group 0 begin at s_first_data_block
 */
-int recover_EXT2(disk_t *disk_car, const struct ext2_super_block *sb,partition_t *partition,const int debug, const int dump_ind)
+int recover_EXT2(disk_t *disk_car, const struct ext2_super_block *sb,partition_t *partition,const int verbose, const int dump_ind)
 {
-  if(test_EXT2(disk_car,sb,partition,debug,dump_ind)!=0)
+  if(test_EXT2(disk_car,sb,partition,verbose,dump_ind)!=0)
     return 1;
   if(partition==NULL)
     return 0;
-  set_EXT2_info(disk_car,sb,partition,debug,dump_ind);
+  set_EXT2_info(disk_car,sb,partition,verbose,dump_ind);
   partition->part_type_i386=P_LINUX;
   partition->part_type_mac=PMAC_LINUX;
   partition->part_type_sun=PSUN_LINUX;
+  partition->part_type_gpt=GPT_ENT_TYPE_LINUX_DATA;
   partition->part_size=(uint64_t)le32(sb->s_blocks_count)*(EXT2_MIN_BLOCK_SIZE<<le32(sb->s_log_block_size));
   partition->blocksize=EXT2_MIN_BLOCK_SIZE<<le32(sb->s_log_block_size);
-  if(debug>0)
+  guid_cpy(&partition->part_uuid, (const efi_guid_t *)&sb->s_uuid);
+  if(verbose>0)
   {
     log_info("\n");
   }
+  partition->sborg_offset=0x400;
+  partition->sb_size=EXT2_SUPERBLOCK_SIZE;
   if(le16(sb->s_block_group_nr)>0)
   {
-    unsigned long int boot_sector=(le32(sb->s_first_data_block)+le16(sb->s_block_group_nr)*le32(sb->s_blocks_per_group));
-    if(partition->part_offset<(uint64_t)boot_sector*((EXT2_MIN_BLOCK_SIZE/disk_car->sector_size)<<le32(sb->s_log_block_size))*disk_car->sector_size)
+    unsigned long int block_nr=(le32(sb->s_first_data_block)+le16(sb->s_block_group_nr)*le32(sb->s_blocks_per_group));
+    if(partition->part_offset< (uint64_t)block_nr * (EXT2_MIN_BLOCK_SIZE<<le32(sb->s_log_block_size)))
     {
       log_error("recover_EXT2: part_offset problem\n");
       return 1;
     }
-    partition->boot_sector=boot_sector;
-    partition->part_offset-=(uint64_t)boot_sector*((EXT2_MIN_BLOCK_SIZE/disk_car->sector_size)<<le32(sb->s_log_block_size))*disk_car->sector_size;
-    log_warning("recover_EXT2: \"e2fsck -b %lu -B %lu device\" may be needed\n",
-	partition->boot_sector,partition->blocksize);
+    partition->sb_offset=(uint64_t)block_nr * (EXT2_MIN_BLOCK_SIZE<<le32(sb->s_log_block_size));
+    partition->part_offset-=partition->sb_offset;
+    log_warning("recover_EXT2: \"e2fsck -b %lu -B %u device\" may be needed\n",
+        block_nr, partition->blocksize);
   }
   else
   {
-    partition->boot_sector=0;
+    partition->sb_offset=0;
   }
-  if(debug>0)
+  if(verbose>0)
   {
     log_info("recover_EXT2: s_block_group_nr=%u/%u, s_mnt_count=%u/%u, s_blocks_per_group=%u\n",
-    le16(sb->s_block_group_nr), (unsigned int)(le32(sb->s_blocks_count)/le32(sb->s_blocks_per_group)), le16(sb->s_mnt_count),le16(sb->s_max_mnt_count),(unsigned int)le32(sb->s_blocks_per_group));
-    log_info("recover_EXT2: boot_sector=%lu, s_blocksize=%lu\n",partition->boot_sector,partition->blocksize);
-    log_info("recover_EXT2: s_blocks_count %u\n",(unsigned int)le32(sb->s_blocks_count));
-    log_info("recover_EXT2: part_size %lu\n",(long unsigned)(partition->part_size/disk_car->sector_size));
+        le16(sb->s_block_group_nr),
+        (unsigned int)(le32(sb->s_blocks_count)/le32(sb->s_blocks_per_group)),
+        le16(sb->s_mnt_count), le16(sb->s_max_mnt_count),
+        (unsigned int)le32(sb->s_blocks_per_group));
+    log_info("recover_EXT2: s_blocksize=%u\n", partition->blocksize);
+    log_info("recover_EXT2: s_blocks_count %u\n", (unsigned int)le32(sb->s_blocks_count));
+    log_info("recover_EXT2: part_size %lu\n", (long unsigned)(partition->part_size/disk_car->sector_size));
   }
   return 0;
 }
 
-static int test_EXT2(disk_t *disk_car, const struct ext2_super_block *sb,partition_t *partition,const int debug, const int dump_ind)
+static int test_EXT2(disk_t *disk_car, const struct ext2_super_block *sb,partition_t *partition,const int verbose, const int dump_ind)
 {
     /* There is a little offset ... */
   if(le16(sb->s_magic)!=EXT2_SUPER_MAGIC)
@@ -141,7 +148,10 @@ static int test_EXT2(disk_t *disk_car, const struct ext2_super_block *sb,partiti
   if(dump_ind!=0)
   {
     if(partition!=NULL && disk_car!=NULL)
-      log_info("\nEXT2/EXT3 magic value at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
+      log_info("\nEXT2/EXT3 magic value at %u/%u/%u\n",
+	  offset2cylinder(disk_car,partition->part_offset),
+	  offset2head(disk_car,partition->part_offset),
+	  offset2sector(disk_car,partition->part_offset));
     /* There is a little offset ... */
     dump_log(sb,DEFAULT_SECTOR_SIZE);
   }
@@ -151,21 +161,21 @@ static int test_EXT2(disk_t *disk_car, const struct ext2_super_block *sb,partiti
       (le16(sb->s_errors) != EXT2_ERRORS_CONTINUE) &&
       (le16(sb->s_errors) != EXT2_ERRORS_RO) &&
       (le16(sb->s_errors) != EXT2_ERRORS_PANIC))
-    return 5;
+    return 4;
   if ((le16(sb->s_state) & ~(EXT2_VALID_FS | EXT2_ERROR_FS))!=0)
     return 5;
   if (le32(sb->s_blocks_count) == 0) /* reject empty filesystem */
     return 6;
   if(le32(sb->s_log_block_size)>2)  /* block size max = 4096, can be 8192 on alpha */
     return 7;
-  if(partition!=NULL)
-  {
-    if(partition->part_size!=0 && (partition->part_size<le32(sb->s_blocks_count)*(EXT2_MIN_BLOCK_SIZE<<le32(sb->s_log_block_size))))
-      return 8;
-    if(EXT2_HAS_COMPAT_FEATURE(sb,EXT3_FEATURE_COMPAT_HAS_JOURNAL)!=0)
-      partition->upart_type=UP_EXT3;
-    else
-      partition->upart_type=UP_EXT2;
-  }
+  if(partition==NULL)
+    return 0;
+  if(partition->part_size!=0 &&
+      partition->part_size<(uint64_t)le32(sb->s_blocks_count)*(EXT2_MIN_BLOCK_SIZE<<le32(sb->s_log_block_size)))
+    return 8;
+  if(EXT2_HAS_COMPAT_FEATURE(sb,EXT3_FEATURE_COMPAT_HAS_JOURNAL)!=0)
+    partition->upart_type=UP_EXT3;
+  else
+    partition->upart_type=UP_EXT2;
   return 0;
 }
