@@ -31,8 +31,10 @@
 #include <sys/stat.h>
 #endif
 
-#ifdef HAVE_LIBEXT2FS
+#ifdef HAVE_EXT2FS_EXT2_FS_H
 #include "ext2fs/ext2_fs.h"
+#endif
+#ifdef HAVE_EXT2FS_EXT2FS_H
 #include "ext2fs/ext2fs.h"
 #endif
 
@@ -41,9 +43,10 @@
 #include "intrf.h"
 #include "dir.h"
 #include "ext2_dir.h"
+#include "ext2_inc.h"
 #include "log.h"
 
-#ifdef HAVE_LIBEXT2FS
+#if defined(HAVE_LIBEXT2FS)
 #define DIRENT_DELETED_FILE	4
 /*
  * list directory
@@ -51,14 +54,6 @@
 
 #define LONG_OPT	0x0001
 
-struct ext2_dir_struct {
-	file_data_t *dir_list;
-	file_data_t *current_file;
-	ext2_filsys current_fs;
-	int flags;
-};
-
-
 /*
  * I/O Manager routine prototypes
  */
@@ -113,12 +108,12 @@ static io_channel alloc_io_channel(disk_t *disk_car,my_data_t *my_data)
   memset(ioch, 0, sizeof(struct struct_io_channel));
   ioch->magic = EXT2_ET_MAGIC_IO_CHANNEL;
   ioch->manager = my_io_manager;
-  ioch->name = (char *)MALLOC(strlen(my_data->partition->name)+1);
+  ioch->name = (char *)MALLOC(strlen(my_data->partition->fsname)+1);
   if (ioch->name==NULL) {
 	  free(ioch);
 	  return NULL;
   }
-  memcpy(ioch->name, my_data->partition->name,strlen(my_data->partition->name)+1);
+  memcpy(ioch->name, my_data->partition->fsname,strlen(my_data->partition->fsname)+1);
   ioch->private_data = my_data;
   ioch->block_size = 1024; /* The smallest ext2fs block size */
   ioch->read_error = 0;
@@ -166,7 +161,8 @@ static errcode_t my_read_blk(io_channel channel, unsigned long block, int count,
 
   size = (count < 0) ? -count : count * channel->block_size;
 #ifdef DEBUG
-  log_trace("my_read_blk start size=%lu, offset=%lu name=%s\n",(long unsigned)size,(unsigned long)(block*channel->block_size),my_data->partition->name);
+  log_trace("my_read_blk start size=%lu, offset=%lu name=%s\n",
+      (long unsigned)size, (unsigned long)(block*channel->block_size), my_data->partition->fsname);
 #endif
   if(my_data->disk_car->read(my_data->disk_car,size,buf,my_data->partition->part_offset+(uint64_t)block*channel->block_size)!=0)
     return 1;
@@ -270,9 +266,9 @@ static void dir_partition_ext2_close(dir_data_t *dir_data)
 }
 #endif
 
-int dir_partition_ext2_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int debug)
+int dir_partition_ext2_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose)
 {
-#ifdef HAVE_LIBEXT2FS
+#if defined(HAVE_LIBEXT2FS)
   struct ext2_dir_struct *ls=(struct ext2_dir_struct *)MALLOC(sizeof(*ls));
   io_channel ioch;
   my_data_t *my_data;
@@ -285,14 +281,15 @@ int dir_partition_ext2_init(disk_t *disk_car, const partition_t *partition, dir_
   my_data->disk_car=disk_car;
   ioch=alloc_io_channel(disk_car,my_data);
   shared_ioch=&ioch;
-  if(ext2fs_open ("/dev/testdisk", 0, partition->boot_sector, partition->blocksize, my_io_manager, &ls->current_fs)!=0)
+  /* An alternate superblock may be used if the calling function has set an IO redirection */
+  if(ext2fs_open ("/dev/testdisk", 0, 0, 0, my_io_manager, &ls->current_fs)!=0)
   {
     free(ls);
     return -1;
   }
   strncpy(dir_data->current_directory,"/",sizeof(dir_data->current_directory));
   dir_data->current_inode=EXT2_ROOT_INO;
-  dir_data->debug=debug;
+  dir_data->verbose=verbose;
   dir_data->get_dir=ext2_dir;
   dir_data->copy_file=NULL;
   dir_data->close=&dir_partition_ext2_close;
@@ -307,7 +304,7 @@ int dir_partition_ext2_init(disk_t *disk_car, const partition_t *partition, dir_
 const char*td_ext2fs_version(void)
 {
   const char *ext2fs_version="none";
-#ifdef HAVE_LIBEXT2FS
+#if defined(HAVE_LIBEXT2FS)
   ext2fs_get_library_version(&ext2fs_version,NULL);
 #endif
   return ext2fs_version;
