@@ -19,6 +19,6 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
-int dir_partition_ext2_init(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const int debug);
+int dir_partition_ext2_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose);
 const char*td_ext2fs_version(void);
 
