@@ -67,14 +67,14 @@ static void fat16_remove_used_space(disk_t *disk_car,const partition_t *partitio
     if(le16(p16[offset_o])!=0)
     {
       /* Not free */
-      if(end_free+1==partition->part_offset+(uint64_t)(start_data+(prev_cluster-2)*cluster_size)*sector_size)
+      if(end_free+1==partition->part_offset+(start_data+(uint64_t)(prev_cluster-2)*cluster_size)*sector_size)
 	end_free+=cluster_size*sector_size;
       else
       {
 	if(start_free != end_free)
 	  del_search_space(list_search_space, start_free, end_free);
-	start_free=partition->part_offset+(uint64_t)(start_data+(prev_cluster-2)*cluster_size)*sector_size;
-	end_free=partition->part_offset+(uint64_t)(start_data+(prev_cluster-2+1)*cluster_size)*sector_size-1;
+	start_free=partition->part_offset+(start_data+(uint64_t)(prev_cluster-2)*cluster_size)*sector_size;
+	end_free=start_free+(uint64_t)cluster_size*sector_size-1;
       }
     }
   }
@@ -118,8 +118,8 @@ static void fat32_remove_used_space(disk_t *disk_car,const partition_t *partitio
       {
 	if(start_free != end_free)
 	  del_search_space(list_search_space, start_free, end_free);
-	start_free=partition->part_offset+(uint64_t)(start_data+(prev_cluster-2)*cluster_size)*sector_size;
-	end_free=partition->part_offset+(uint64_t)(start_data+(prev_cluster-2+1)*cluster_size)*sector_size-1;
+	start_free=partition->part_offset+(start_data+(uint64_t)(prev_cluster-2)*cluster_size)*sector_size;
+	end_free=start_free+(uint64_t)cluster_size*sector_size-1;
       }
     }
   }
@@ -148,7 +148,7 @@ unsigned int fat_remove_used_space(disk_t *disk_car, const partition_t *partitio
     }
     sector_size=fat_sector_size(fat_header);
     fat_length=le16(fat_header->fat_length)>0?le16(fat_header->fat_length):le32(fat_header->fat32_length);
-    part_size=(sectors(fat_header)>0?sectors(fat_header):fat_header->total_sect);
+    part_size=(sectors(fat_header)>0?sectors(fat_header):le32(fat_header->total_sect));
     start_fat1=le16(fat_header->reserved);
     start_data=start_fat1+fat_header->fats*fat_length+(get_dir_entries(fat_header)*32+sector_size-1)/sector_size;
     no_of_cluster=(part_size-start_data)/fat_header->cluster_size;
