@@ -2,7 +2,7 @@
 
     File: fatp.h
 
-    Copyright (C) 2006 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2006-2007 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,4 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
-
-unsigned int remove_used_space(t_param_disk *disk_car,const t_partition *partition, t_alloc_data *list_search_space);
-
+unsigned int fat_remove_used_space(disk_t *disk_car,const partition_t *partition, alloc_data_t *list_search_space);
