@@ -29,5 +29,5 @@ struct disk_fatx
   uint32_t	unknown;
 } __attribute__ ((__packed__));
 
-int check_FATX(disk_t *disk_car,partition_t *partition,const int debug);
-int recover_FATX(disk_t *disk_car, const struct disk_fatx *fatx_block, partition_t *partition, const int debug, const int dump_ind);
+int check_FATX(disk_t *disk_car,partition_t *partition,const int verbose);
+int recover_FATX(disk_t *disk_car, const struct disk_fatx *fatx_block, partition_t *partition, const int verbose, const int dump_ind);
