@@ -35,11 +35,11 @@
 #include "filegen.h"
 #include "ole.h"
 #include "log.h"
+#include "memmem.h"
 
 static void register_header_check_doc(file_stat_t *file_stat);
 static void file_check_doc(file_recovery_t *file_recovery);
 static int header_check_doc(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
-static const unsigned char * find_in_mem(const unsigned char *haystack, const unsigned int haystack_size, const unsigned char *needle, const unsigned int needle_size);
 static uint64_t test_OLE(FILE *file);
 
 const file_hint_t file_hint_doc= {
@@ -48,7 +48,6 @@ const file_hint_t file_hint_doc= {
   .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
-  .header_check=&header_check_doc,
   .register_header_check=&register_header_check_doc
 };
 
@@ -59,15 +58,6 @@ static void register_header_check_doc(file_stat_t *file_stat)
   register_header_check(0, doc_header,sizeof(doc_header), &header_check_doc, file_stat);
 }
 
-static const unsigned char * find_in_mem(const unsigned char *haystack, const unsigned int haystack_size, const unsigned char *needle, const unsigned int needle_size)
-{
-  unsigned int i;
-  for(i=0;i<haystack_size-needle_size;i++)
-    if(memcmp(&haystack[i],needle,needle_size)==0)
-      return &haystack[i];
-  return NULL;
-}
-
 static void file_check_doc(file_recovery_t *file_recovery)
 {
   uint64_t doc_file_size=test_OLE(file_recovery->handle);
@@ -87,8 +77,10 @@ static int header_check_doc(const unsigned char *buffer, const unsigned int buff
       return 0;
     if(le16(header->uMiniSectorShift)!=6 || le16(header->uSectorShift)!=9)
       return 0;
-    if(le32(header->num_FAT_blocks)==0 || le32(header->num_FAT_blocks)>100)
-      return 0;
+    /*
+       num_FAT_blocks=109+num_extra_FAT_blocks*(512-1);
+       maximum file size is 512+(num_FAT_blocks*128)*512, about 1.6GB
+     */
     if(le32(header->num_FAT_blocks)==0 ||
 	le32(header->num_extra_FAT_blocks)>50 ||
 	le32(header->num_FAT_blocks)>109+le32(header->num_extra_FAT_blocks)*((1<<le16(header->uSectorShift))-1))
@@ -96,70 +88,79 @@ static int header_check_doc(const unsigned char *buffer, const unsigned int buff
     /* TODO read the Root Directory */
     reset_file_recovery(file_recovery_new);
     file_recovery_new->file_check=&file_check_doc;
-    if(find_in_mem(buffer,buffer_size,"S\0c\0e\0n\0e\0",10)!=NULL)
+    if(td_memmem(buffer,buffer_size,"S\0c\0e\0n\0e\0",10)!=NULL)
     {
       file_recovery_new->extension="max";
     }
-    else if(find_in_mem(buffer,buffer_size,"WordDocument",12)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"WordDocument",12)!=NULL)
     {
       file_recovery_new->extension="doc";
     }
-    else if(find_in_mem(buffer,buffer_size,"StarDraw",8)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"StarDraw",8)!=NULL)
     {
       file_recovery_new->extension="sda";
     }
-    else if(find_in_mem(buffer,buffer_size,"StarCalc",8)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"StarCalc",8)!=NULL)
     {
       file_recovery_new->extension="sdc";
     }
-    else if(find_in_mem(buffer,buffer_size,"StarImpress",11)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"StarImpress",11)!=NULL)
     {
       file_recovery_new->extension="sdd";
     }
-    else if(find_in_mem(buffer,buffer_size,"W\0k\0s\0S\0S\0W\0o\0r\0k\0B\0o\0o\0k\0",26)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"W\0k\0s\0S\0S\0W\0o\0r\0k\0B\0o\0o\0k\0",26)!=NULL)
     {
       file_recovery_new->extension="xlr";
     }
-    else if(find_in_mem(buffer,buffer_size,"Worksheet",9)!=NULL ||
-	find_in_mem(buffer,buffer_size,"Book",4)!=NULL || 
-	find_in_mem(buffer,buffer_size,"Workbook",8)!=NULL || 
-	find_in_mem(buffer,buffer_size,"W\0o\0r\0k\0b\0o\0o\0k\0",16)!=NULL || 
-	find_in_mem(buffer,buffer_size,"Calc",4)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"I\0m\0a\0g\0e\0s\0S\0t\0o\0r\0e\0",22)!=NULL)
+    { /* HP Photosmart Photo Printing Album */
+      file_recovery_new->extension="albm";
+    }
+    else if(td_memmem(buffer,buffer_size,"Worksheet",9)!=NULL ||
+	td_memmem(buffer,buffer_size,"Book",4)!=NULL || 
+	td_memmem(buffer,buffer_size,"Workbook",8)!=NULL || 
+	td_memmem(buffer,buffer_size,"W\0o\0r\0k\0b\0o\0o\0k\0",16)!=NULL || 
+	td_memmem(buffer,buffer_size,"Calc",4)!=NULL)
     {
       file_recovery_new->extension="xls";
     }
-    else if(find_in_mem(buffer,buffer_size,"Power",5)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"Power",5)!=NULL ||
+        td_memmem(buffer,buffer_size,"P\0o\0w\0e\0r\0",10)!=NULL)
     {
       file_recovery_new->extension="ppt";
     }
-    else if(find_in_mem(buffer,buffer_size,"AccessObjSiteData",17)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"AccessObjSiteData",17)!=NULL)
     {
       file_recovery_new->extension="mdb";
     }
-    else if(find_in_mem(buffer,buffer_size,"Visio",5)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"Visio",5)!=NULL)
     {
       file_recovery_new->extension="vis";
     }
-    else if(find_in_mem(buffer,buffer_size,"Sfx",3)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"Sfx",3)!=NULL)
     {
       file_recovery_new->extension="sdw";
     }
-    else if(find_in_mem(buffer,buffer_size,"CPicPage",8)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"CPicPage",8)!=NULL)
     {	/* Flash */
       file_recovery_new->extension="fla";
     }
-    else if(find_in_mem(buffer,buffer_size,"Microsoft Publisher",19)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"Microsoft Publisher",19)!=NULL)
     {
       file_recovery_new->extension="pub";
     }
-    else if(find_in_mem(buffer,buffer_size,"C\0O\0N\0T\0E\0N\0T\0S\0",16)!=NULL)
-    {
+    else if(td_memmem(buffer,buffer_size,"C\0O\0N\0T\0E\0N\0T\0S\0",16)!=NULL)
+    { /* Microsoft Works .wps */
       file_recovery_new->extension="wps";
     }
-    else if(find_in_mem(buffer,buffer_size,"MetaStock",9)!=NULL)
+    else if(td_memmem(buffer,buffer_size,"MetaStock",9)!=NULL)
     {
       file_recovery_new->extension="mws";
     }
+    else if(td_memmem(buffer,buffer_size,"_\0_\0n\0a\0m\0e\0i\0d\0_\0v\0e\0r\0s\0i\0o\0n\0001\0.\0000\0",38)!=NULL)
+    { /* Outlook */
+      file_recovery_new->extension="msg";
+    }
     else
       file_recovery_new->extension=file_hint_doc.extension;
     return 1;
@@ -177,7 +178,7 @@ static uint64_t test_OLE(FILE *IN)
   struct OLE_HDR *header=(struct OLE_HDR*)&buffer_header;
   if(!IN)
     return 0;
-  fseek(IN,SEEK_SET,0);
+  fseek(IN,0,SEEK_SET);
   fread(&buffer_header,sizeof(buffer_header),1,IN);	/*reads first sector including OLE header */
   /*
   log_trace("num_FAT_blocks       %u\n",le32(header->num_FAT_blocks));
