@@ -50,7 +50,6 @@ const file_hint_t file_hint_jpg= {
   .min_header_distance=0,
   .max_filesize=50*1024*1024,
   .recover=1,
-  .header_check=&header_check_jpg,
   .register_header_check=&register_header_check_jpg
 };
 
@@ -394,18 +393,18 @@ static void file_check_jpg(file_recovery_t *file_recovery)
       (void) jpeg_read_scanlines(&cinfo, buffer, 1);
     }
     (void) jpeg_finish_decompress(&cinfo);
+    {
+      my_source_mgr * src;
+      src = (my_source_mgr *) cinfo.src;
+      jpeg_size=src->file_size - src->pub.bytes_in_buffer;
+    }
     jpeg_destroy_decompress(&cinfo);
   }
-#endif
+#else
   /* Not accurate */
-//  jpeg_size=ftell(infile);
-//  log_error("JPG offset %llu\n", (long long unsigned)jpeg_size);
-  {
-    my_source_mgr * src;
-    src = (my_source_mgr *) cinfo.src;
-    jpeg_size=src->file_size - src->pub.bytes_in_buffer;
+  jpeg_size=ftell(infile);
+#endif
 //    log_error("JPG size: %llu\n", (long long unsigned)jpeg_size);
-  }
   if(jpeg_size<=0)
     return;
 #if defined(HAVE_LIBJPEG) && defined(HAVE_JPEGLIB_H)
