@@ -31,12 +31,11 @@
 #include "filegen.h"
 #include "common.h"
 #include "log.h"
+#include "memmem.h"
 
 static void register_header_check_mov(file_stat_t *file_stat);
 static int header_check_mov(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
-#ifdef EXPERIMENTAL
 static int data_check_mov(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
-#endif
 
 const file_hint_t file_hint_mov= {
   .extension="mov",
@@ -44,7 +43,6 @@ const file_hint_t file_hint_mov= {
   .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
-  .header_check=&header_check_mov,
   .register_header_check=&register_header_check_mov
 };
 
@@ -53,109 +51,104 @@ static void register_header_check_mov(file_stat_t *file_stat)
   register_header_check(0, NULL,0, &header_check_mov, file_stat);
 }
 
-static const unsigned char * find_in_mem(const unsigned char *haystack, const unsigned int haystack_size, const unsigned char *needle, const unsigned int needle_size);
-
-static const unsigned char * find_in_mem(const unsigned char *haystack, const unsigned int haystack_size, const unsigned char *needle, const unsigned int needle_size)
-{
-  unsigned int i;
-  for(i=0;i<haystack_size-needle_size;i++)
-    if(memcmp(&haystack[i],needle,needle_size)==0)
-      return &haystack[i];
-  return NULL;
-}
-
 static int header_check_mov(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   unsigned int i=0;
   unsigned int prev_atom_skip=0;
-#ifdef EXPERIMENTAL
-  if(file_recovery!=0 && file_recovery->file_stat!=0 && file_recovery->file_stat->file_hint==&file_hint_mov)
-    return 0;
-  file_recovery_new->calculated_file_size=0;
-#endif
   while(i<buffer_size-8)
   {
     unsigned int atom_size;
     atom_size=(buffer[i+0]<<24)+(buffer[i+1]<<16)+(buffer[i+2]<<8)+buffer[i+3];
     if(atom_size<8 || atom_size>1024*1024*1024)
       return 0;
-#ifdef EXPERIMENTAL
-    file_recovery_new->data_check=data_check_mov;
-    file_recovery_new->file_check=&file_check_file;
-    file_recovery_new->calculated_file_size=i+atom_size;
-#endif
     /* check for commun atom type */
     if(buffer[i+4]=='p' && buffer[i+5]=='n' && buffer[i+6]=='o' && buffer[i+7]=='t')
     {
-      if(atom_size < 256)
-      {
-	reset_file_recovery(file_recovery_new);
-	file_recovery_new->extension=file_hint_mov.extension;
-	return 1;
-      }
-      else
+      if(atom_size > 256)
 	return 0;
+      reset_file_recovery(file_recovery_new);
+      file_recovery_new->extension=file_hint_mov.extension;
+      file_recovery_new->data_check=data_check_mov;
+      file_recovery_new->file_check=&file_check_size;
+      file_recovery_new->calculated_file_size=i+atom_size;
+      return 1;
     }
     if(buffer[i+4]=='w' && buffer[i+5]=='i' && buffer[i+6]=='d' && buffer[i+7]=='e')
     {
-      if(atom_size < 256)
-      {
-	reset_file_recovery(file_recovery_new);
-	file_recovery_new->extension=file_hint_mov.extension;
-	return 1;
-      }
-      else
+      if(atom_size > 256)
 	return 0;
+      reset_file_recovery(file_recovery_new);
+      file_recovery_new->extension=file_hint_mov.extension;
+      file_recovery_new->data_check=data_check_mov;
+      file_recovery_new->file_check=&file_check_size;
+      file_recovery_new->calculated_file_size=i+atom_size;
+      return 1;
     }
     if(buffer[i+4]=='m' && buffer[i+5]=='o' && buffer[i+6]=='o' && buffer[i+7]=='v')
     {
-      if(atom_size < 256*256*256)
-      {
-	reset_file_recovery(file_recovery_new);
-	file_recovery_new->extension=file_hint_mov.extension;
-	return 1;
-      }
-      else
+      if(atom_size > 256*256*256)
 	return 0;
+      reset_file_recovery(file_recovery_new);
+      file_recovery_new->extension=file_hint_mov.extension;
+      file_recovery_new->data_check=data_check_mov;
+      file_recovery_new->file_check=&file_check_size;
+      file_recovery_new->calculated_file_size=i+atom_size;
+      return 1;
     }
     if(buffer[i+4]=='f' && buffer[i+5]=='t' && buffer[i+6]=='y' && buffer[i+7]=='p')
     {
-      if(atom_size>buffer_size-i)
-	atom_size=buffer_size-i;
-      if(find_in_mem(&buffer[i+8], atom_size-8, "isom", 4)!=0 ||
-	  find_in_mem(&buffer[i+8], atom_size-8, "mp41", 4)!=0 ||
-	  find_in_mem(&buffer[i+8], atom_size-8, "mp42", 4)!=0 ||
-	  find_in_mem(&buffer[i+8], atom_size-8, "mmp4", 4)!=0 ||
-	  find_in_mem(&buffer[i+8], atom_size-8, "M4A", 3)!=0 ||
-	  find_in_mem(&buffer[i+8], atom_size-8, "M4B", 3)!=0 ||
-	  find_in_mem(&buffer[i+8], atom_size-8, "M4P", 3)!=0)
+      unsigned int search_size=atom_size;
+      if(search_size>buffer_size-i)
+	search_size=buffer_size-i;
+      if(td_memmem(&buffer[i+8], search_size-8, "isom", 4)!=NULL ||
+	  td_memmem(&buffer[i+8], search_size-8, "mp41", 4)!=NULL ||
+	  td_memmem(&buffer[i+8], search_size-8, "mp42", 4)!=NULL ||
+	  td_memmem(&buffer[i+8], search_size-8, "mmp4", 4)!=NULL ||
+	  td_memmem(&buffer[i+8], search_size-8, "M4A", 3)!=NULL ||
+	  td_memmem(&buffer[i+8], search_size-8, "M4B", 3)!=NULL ||
+	  td_memmem(&buffer[i+8], search_size-8, "M4P", 3)!=NULL)
       {
 	reset_file_recovery(file_recovery_new);
 	file_recovery_new->extension="mp4";
+	file_recovery_new->data_check=data_check_mov;
+	file_recovery_new->file_check=&file_check_size;
+	file_recovery_new->calculated_file_size=i+atom_size;
 	return 1;
       }
-      else if(find_in_mem(&buffer[i+8], atom_size-8, "3gp", 3)!=0)
+      else if(td_memmem(&buffer[i+8], search_size-8, "3gp", 3)!=NULL)
       {
 	reset_file_recovery(file_recovery_new);
 	file_recovery_new->extension="3gp";
+	file_recovery_new->data_check=data_check_mov;
+	file_recovery_new->file_check=&file_check_size;
+	file_recovery_new->calculated_file_size=i+atom_size;
 	return 1;
       }
-      else if(find_in_mem(&buffer[i+8], atom_size-8, "3g2", 3)!=0)
+      else if(td_memmem(&buffer[i+8], search_size-8, "3g2", 3)!=NULL)
       {
 	reset_file_recovery(file_recovery_new);
 	file_recovery_new->extension="3g2";
+	file_recovery_new->data_check=data_check_mov;
+	file_recovery_new->file_check=&file_check_size;
+	file_recovery_new->calculated_file_size=i+atom_size;
 	return 1;
       }
-      else if(find_in_mem(&buffer[i+8], atom_size-8, "jp2", 3)!=0)
+      else if(td_memmem(&buffer[i+8], search_size-8, "jp2", 3)!=NULL)
       {
 	reset_file_recovery(file_recovery_new);
 	file_recovery_new->extension="jp2";
+	file_recovery_new->data_check=data_check_mov;
+	file_recovery_new->file_check=&file_check_size;
+	file_recovery_new->calculated_file_size=i+atom_size;
 	return 1;
       }
-      else if(find_in_mem(&buffer[i+8], atom_size-8, "qt", 2)!=0)
+      else if(td_memmem(&buffer[i+8], search_size-8, "qt", 2)!=NULL)
       {
 	reset_file_recovery(file_recovery_new);
 	file_recovery_new->extension="mov";
+	file_recovery_new->data_check=data_check_mov;
+	file_recovery_new->file_check=&file_check_size;
+	file_recovery_new->calculated_file_size=i+atom_size;
 	return 1;
       }
     }
@@ -163,6 +156,9 @@ static int header_check_mov(const unsigned char *buffer, const unsigned int buff
     {
       reset_file_recovery(file_recovery_new);
       file_recovery_new->extension=file_hint_mov.extension;
+      file_recovery_new->data_check=data_check_mov;
+      file_recovery_new->file_check=&file_check_size;
+      file_recovery_new->calculated_file_size=i+atom_size;
       return 1;
     }
     if(buffer[i+4]=='s' && buffer[i+5]=='k' && buffer[i+6]=='i' && buffer[i+7]=='p')
@@ -175,8 +171,6 @@ static int header_check_mov(const unsigned char *buffer, const unsigned int buff
   return 0;
 }
 
-#ifdef EXPERIMENTAL
-/* There are too much atom types */
 static int data_check_mov(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
   while(file_recovery->calculated_file_size + 8 < file_recovery->file_size + buffer_size/2)
@@ -185,35 +179,47 @@ static int data_check_mov(const unsigned char *buffer, const unsigned int buffer
     unsigned int i;
     i=file_recovery->calculated_file_size - file_recovery->file_size + buffer_size/2;
     atom_size=(buffer[i+0]<<24)+(buffer[i+1]<<16)+(buffer[i+2]<<8)+buffer[i+3];
-//    log_trace("file_mov.c: atom %c%c%c%c size %u\n", buffer[i+4],buffer[i+5],buffer[i+6],buffer[i+7], atom_size);
+#ifdef DEBUG_MOV
+    log_trace("file_mov.c: atom %c%c%c%c (0x%02x%02x%02x%02x) size %u, calculated_file_size %llu\n",
+        buffer[i+4],buffer[i+5],buffer[i+6],buffer[i+7], 
+        buffer[i+4],buffer[i+5],buffer[i+6],buffer[i+7], 
+        atom_size,
+        (long long unsigned)file_recovery->calculated_file_size);
+#endif
     if(atom_size>=8 && atom_size<1024*1024*1024 &&
-	((buffer[i+4]=='f' && buffer[i+5]=='r' && buffer[i+6]=='e' && buffer[i+7]=='e') ||
-	 (buffer[i+4]=='s' && buffer[i+5]=='k' && buffer[i+6]=='i' && buffer[i+7]=='p') ||
-	 (buffer[i+4]=='w' && buffer[i+5]=='i' && buffer[i+6]=='d' && buffer[i+7]=='e') ||
-	 (buffer[i+4]=='p' && buffer[i+5]=='n' && buffer[i+6]=='o' && buffer[i+7]=='t') ||
-	 (buffer[i+4]=='m' && buffer[i+5]=='o' && buffer[i+6]=='o' && buffer[i+7]=='v') ||
-	 (buffer[i+4]=='m' && buffer[i+5]=='d' && buffer[i+6]=='a' && buffer[i+7]=='t') ||
-	 (buffer[i+4]=='f' && buffer[i+5]=='t' && buffer[i+6]=='y' && buffer[i+7]=='p') ||
-	 (buffer[i+4]=='t' && buffer[i+5]=='r' && buffer[i+6]=='a' && buffer[i+7]=='k') ||
-	 (buffer[i+4]=='m' && buffer[i+5]=='d' && buffer[i+6]=='i' && buffer[i+7]=='a') ||
-	 (buffer[i+4]=='s' && buffer[i+5]=='t' && buffer[i+6]=='b' && buffer[i+7]=='l') ||
-	 (buffer[i+4]=='c' && buffer[i+5]=='m' && buffer[i+6]=='o' && buffer[i+7]=='v') ||
-	 (buffer[i+4]=='d' && buffer[i+5]=='c' && buffer[i+6]=='o' && buffer[i+7]=='m') ||
-	 (buffer[i+4]=='c' && buffer[i+5]=='m' && buffer[i+6]=='v' && buffer[i+7]=='d') ||
-	 (buffer[i+4]=='j' && buffer[i+5]=='p' && buffer[i+6]=='2' && buffer[i+7]=='h')
-	)
+        ((buffer[i+4]=='f' && buffer[i+5]=='r' && buffer[i+6]=='e' && buffer[i+7]=='e') ||
+         (buffer[i+4]=='s' && buffer[i+5]=='k' && buffer[i+6]=='i' && buffer[i+7]=='p') ||
+         (buffer[i+4]=='w' && buffer[i+5]=='i' && buffer[i+6]=='d' && buffer[i+7]=='e') ||
+         (buffer[i+4]=='p' && buffer[i+5]=='n' && buffer[i+6]=='o' && buffer[i+7]=='t') ||
+         (buffer[i+4]=='m' && buffer[i+5]=='o' && buffer[i+6]=='o' && buffer[i+7]=='v') ||
+         (buffer[i+4]=='m' && buffer[i+5]=='d' && buffer[i+6]=='a' && buffer[i+7]=='t') ||
+         (buffer[i+4]=='f' && buffer[i+5]=='t' && buffer[i+6]=='y' && buffer[i+7]=='p') ||
+         (buffer[i+4]=='t' && buffer[i+5]=='r' && buffer[i+6]=='a' && buffer[i+7]=='k') ||
+         (buffer[i+4]=='m' && buffer[i+5]=='d' && buffer[i+6]=='i' && buffer[i+7]=='a') ||
+         (buffer[i+4]=='s' && buffer[i+5]=='t' && buffer[i+6]=='b' && buffer[i+7]=='l') ||
+         (buffer[i+4]=='c' && buffer[i+5]=='m' && buffer[i+6]=='o' && buffer[i+7]=='v') ||
+         (buffer[i+4]=='d' && buffer[i+5]=='c' && buffer[i+6]=='o' && buffer[i+7]=='m') ||
+         (buffer[i+4]=='c' && buffer[i+5]=='m' && buffer[i+6]=='v' && buffer[i+7]=='d') ||
+         (buffer[i+4]=='j' && buffer[i+5]=='p' && buffer[i+6]=='2' && buffer[i+7]=='h') ||
+         (buffer[i+4]=='P' && buffer[i+5]=='I' && buffer[i+6]=='C' && buffer[i+7]=='T')
+        )
       )
     {
-      file_recovery->calculated_file_size+=atom_size;
+      file_recovery->calculated_file_size+=(uint64_t)atom_size;
     }
     else
     {
-      log_warning("file_mov.c: unknown atom %c%c%c%c\n", buffer[i+4],buffer[i+5],buffer[i+6],buffer[i+7]);
+      if(!(buffer[i+4]==0 && buffer[i+5]==0 && buffer[i+6]==0 && buffer[i+7]==0))
+        log_warning("file_mov.c: unknown atom 0x%02x%02x%02x%02x at %llu\n",
+            buffer[i+4],buffer[i+5],buffer[i+6],buffer[i+7],
+            (long long unsigned)file_recovery->calculated_file_size);
       return 2;
     }
   }
-  file_recovery->calculated_file_size=file_recovery->file_size+(buffer_size/2);
+#ifdef DEBUG_MOV
+  log_trace("file_mov.c: new calculated_file_size %llu\n",
+      (long long unsigned)file_recovery->calculated_file_size);
+#endif
   return 1;
 }
-#endif
 
