@@ -2,7 +2,7 @@
 
     File: file_mysql.c
 
-    Copyright (C) 2006 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2006-2007 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,30 +28,45 @@
 #endif
 #include <stdio.h>
 #include "types.h"
-#include "photorec.h"
+#include "filegen.h"
 
-static const char* header_check_mysql(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery);
-const t_file_hint file_hint_mysql= {
+
+static void register_header_check_mysql(file_stat_t *file_stat);
+static int header_check_mysql(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
+
+const file_hint_t file_hint_mysql= {
   .extension="MYI",
   .description="MySQL",
   .min_header_distance=0,
-  .min_filesize=0,
   .max_filesize=0,
   .recover=1,
-  .header_check=&header_check_mysql,
-  .data_check=NULL,
-  .file_check=NULL
+  .register_header_check=&register_header_check_mysql
 };
 
-static const char* header_check_mysql(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery)
+static const unsigned char mysql_header[4]= {0xfe, 0xfe, 0x07, 0x01};
+static const unsigned char mysql_header_def[2]= {0xfe, 0x01};
+
+static void register_header_check_mysql(file_stat_t *file_stat)
+{
+  register_header_check(0, mysql_header,sizeof(mysql_header), &header_check_mysql, file_stat);
+  register_header_check(0, mysql_header_def,sizeof(mysql_header_def), &header_check_mysql, file_stat);
+}
+
+static int header_check_mysql(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   /* MySQL MISAM compressed data file Version 1 */
   if(buffer[0]==0xfe && buffer[1]==0xfe && buffer[2]==0x07 && buffer[3]==0x01)
-    return "MYI";
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->extension="MYI";
+    return 1;
+  }
   /* MySQL table definition file Version 7 up to 10 */
   if(buffer[0]==0xfe && buffer[1]==0x01 && (buffer[2]>=0x07 && buffer[2]<=0x0A) && buffer[3]==0x09)
-    return "frm";
-  return NULL;
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->extension="frm";
+    return 1;
+  }
+  return 0;
 }
-
-
