@@ -2,7 +2,7 @@
 
     File: file_psd.c
 
-    Copyright (C) 2006 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2006-2007 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,26 +28,38 @@
 #endif
 #include <stdio.h>
 #include "types.h"
-#include "photorec.h"
+#include "filegen.h"
 
-static const char* header_check_psd(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery);
-const t_file_hint file_hint_psd= {
+
+static void register_header_check_psd(file_stat_t *file_stat);
+static int header_check_psd(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
+
+const file_hint_t file_hint_psd= {
   .extension="psd",
   .description="Adobe Photoshop Image",
   .min_header_distance=0,
-  .min_filesize=70,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
-  .header_check=&header_check_psd,
-  .data_check=NULL,
-  .file_check=NULL
+  .register_header_check=&register_header_check_psd
 };
-static const char* header_check_psd(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery)
+
+static const unsigned char psd_header[4]={'8','B','P','S' };
+
+static void register_header_check_psd(file_stat_t *file_stat)
+{
+  register_header_check(0, psd_header,sizeof(psd_header), &header_check_psd, file_stat);
+}
+
+static int header_check_psd(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  const unsigned char psd_header[4]={'8','B','P','S' };
   if(memcmp(buffer,psd_header,sizeof(psd_header))==0)
-    return file_hint_psd.extension;
-  return NULL;
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->min_filesize=70;
+    file_recovery_new->extension=file_hint_psd.extension;
+    return 1;
+  }
+  return 0;
 }
 
 
