@@ -3,17 +3,17 @@
     File: file_txt.c
 
     Copyright (C) 2005-2007 Christophe GRENIER <grenier@cgsecurity.org>
-  
+
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
-  
+
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
-  
+
     You should have received a copy of the GNU General Public License along
     with this program; if not, write the Free Software Foundation, Inc., 51
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
@@ -35,9 +35,8 @@
 #include "common.h"
 #include "filegen.h"
 #include "log.h"
-
-/* Allow a small memory leak */
-#define OPT_MALLOC
+#include "memmem.h"
+#include "log.h"
 
 extern const file_hint_t file_hint_doc;
 extern const file_hint_t file_hint_jpg;
@@ -45,6 +44,7 @@ extern const file_hint_t file_hint_pdf;
 extern const file_hint_t file_hint_zip;
 
 static inline int filtre(unsigned char car);
+static inline int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, const int buf_len);
 
 static void register_header_check_txt(file_stat_t *file_stat);
 static int header_check_txt(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
@@ -52,10 +52,9 @@ static void register_header_check_fasttxt(file_stat_t *file_stat);
 static int header_check_fasttxt(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 
 static int data_check_txt(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
-static inline const unsigned char *find_in_mem(const unsigned char *haystack, const unsigned char * haystack_end,
-    const unsigned char *needle, const unsigned int needle_length);
 static void file_check_html(file_recovery_t *file_recovery);
 static void file_check_emlx(file_recovery_t *file_recovery);
+static void file_check_xml(file_recovery_t *file_recovery);
 
 file_hint_t file_hint_fasttxt= {
   .extension="tx?",
@@ -63,7 +62,6 @@ file_hint_t file_hint_fasttxt= {
   .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
-  .header_check=&header_check_fasttxt,
   .register_header_check=&register_header_check_fasttxt
 };
 
@@ -73,10 +71,10 @@ file_hint_t file_hint_txt= {
   .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
-  .header_check=&header_check_txt,
   .register_header_check=&register_header_check_txt
 };
 
+static const unsigned char header_cls[24]	= {'V','E','R','S','I','O','N',' ','1','.','0',' ','C','L','A','S','S','\r','\n','B','E','G','I','N'};
 static const unsigned char header_imm[13]	= {'M','I','M','E','-','V','e','r','s','i','o','n',':'};
 static const unsigned char header_imm2[13]	= {'R','e','t','u','r','n','-','P','a','t','h',':',' '};
 static const unsigned char header_mail[5]	= {'F','r','o','m',' '};
@@ -87,7 +85,7 @@ static const unsigned char header_sh[9]  	= "#!/bin/sh";
 static const unsigned char header_slk[10]  	= "ID;PSCALC3";
 static const unsigned char header_ram[7]	= "rtsp://";
 static const unsigned char header_xml[14]	= "<?xml version=";
-static const char sign_html[]	= "<html";
+static const char sign_html[5]	= "<html";
 
 static void register_header_check_txt(file_stat_t *file_stat)
 {
@@ -96,6 +94,7 @@ static void register_header_check_txt(file_stat_t *file_stat)
 
 static void register_header_check_fasttxt(file_stat_t *file_stat)
 {
+  register_header_check(0, header_cls,sizeof(header_cls), &header_check_fasttxt, file_stat);
   register_header_check(0, header_imm,sizeof(header_imm), &header_check_fasttxt, file_stat);
   register_header_check(0, header_imm2,sizeof(header_imm2), &header_check_fasttxt, file_stat);
   register_header_check(0, header_mail,sizeof(header_mail), &header_check_fasttxt, file_stat);
@@ -173,195 +172,150 @@ static int filtre(unsigned char car)
 
 /* destination should have an extra byte available for null terminator
    return read size */
-static int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, int buf_len)
+static int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, const int buf_len)
 {
   const unsigned char *p; 	/* pointers to actual position in source buffer */
   unsigned char *q;	/* pointers to actual position in destination buffer */
-  unsigned char bufU[8]; /* buffer for UTF8 character read from source */
-  int i, l; /* counter of remaining bytes available in destination buffer, UTF8 character's length */
+  int i; /* counter of remaining bytes available in destination buffer */
   for (i = buf_len, p = buffer, q = buffer_lower; p-buffer<buf_len && i > 0 && *p!='\0';) 
   {
-    if ((*p & 0x80)!=0x80)
-    {
-      *q = tolower(*p++);
-      if (filtre(*q)==1)
+    const unsigned char *p_org=p;
+    if((*p & 0xf0)==0xe0 && (*(p+1) & 0xc0)==0x80 && (*(p+2) & 0xc0)==0x80)
+    { /* UTF8 l=3 */
+#ifdef DEBUG_TXT
+      log_info("UTF8 l=3 0x%02x 0x%02x 0x02x\n", *p, *(p+1),*(p+2));
+#endif
+      *q = '\0';
+      switch (*p)
       {
-	q++;
-	i--;
+        case 0xE2 : 
+          switch (*(p+1))
+          { 
+            case 0x80 : 
+              switch (*(p+2))
+              { 
+                case 0x93 : (*q) = 150; break;
+                case 0x94 : (*q) = 151; break;
+                case 0x98 : (*q) = 145; break;
+                /* case 0x99 : (*q) = 146; break; */
+                case 0x99 : (*q) = '\''; break;
+                case 0x9A : (*q) = 130; break;
+                case 0x9C : (*q) = 147; break;
+                case 0x9D : (*q) = 148; break;
+                case 0x9E : (*q) = 132; break;
+                case 0xA0 : (*q) = 134; break;
+                case 0xA1 : (*q) = 135; break;
+                case 0xA2 : (*q) = 149; break;
+                case 0xA6 : (*q) = 133; break;
+                case 0xB0 : (*q) = 137; break;
+                case 0xB9 : (*q) = 139; break;
+                case 0xBA : (*q) = 155; break;
+              }
+              break;
+            case 0x82 : 
+              switch (*(p+2))
+              { 
+                case 0xAC : (*q) = 128; break;
+              }
+              break;
+            case 0x84 : 
+              switch (*(p+2))
+              { 
+                case 0xA2 : (*q) = 153; break;
+              }
+              break;
+          }
+          break;
       }
-      else
+      p+=3;
+    }
+    else if((*p & 0xe0)==0xc0 && (*(p+1) & 0xc0)==0x80)
+    { /* UTF8 l=2 */
+      *q = '\0';
+      switch (*p)
       {
-	*q = '\0';
-	return(p-buffer);
+        case 0xC2 : 
+          (*q) = ((*(p+1)) | 0x80) & 0xBF; /* A0-BF and a few 80-9F */
+          if((*q)==0xA0)
+            (*q)=' ';
+          break;
+        case 0xC3 : 
+          switch (*(p+1))
+	  { 
+	    case 0xB3 : (*q) = 162; break;
+	    default:
+			(*q) = (*(p+1)) | 0xC0; /* C0-FF */
+			break;
+	  }
+          break;
+        case 0xC5 : 
+          switch (*(p+1)) { 
+            case 0x92 : (*q) = 140; break;
+            case 0x93 : (*q) = 156; break;
+            case 0xA0 : (*q) = 138; break;
+            case 0xA1 : (*q) = 154; break;
+            case 0xB8 : (*q) = 143; break;
+            case 0xBD : (*q) = 142; break;
+            case 0xBE : (*q) = 158; break;
+          }
+          break;
+        case 0xC6: 
+          switch (*(p+1)) { 
+            case 0x92 : (*q) = 131; break;
+          }
+          break;
+        case 0xCB : 
+          switch (*(p+1)) { 
+            case 0x86 : (*q) = 136; break;
+            case 0x9C : (*q) = 152; break;
+          }
+          break;
       }
+      p+=2;
     }
     else
-    {
-      const unsigned char *p_org=p;
-      l = 0;
-      bufU[l++] = *p++;
-      while (*p >> 6 == 2) /* 2nd-6th bytes in UTF8 character have the most significant bits set to "10" */
-      {
-	if (l < 7)
-	  bufU[l] = *p;
-	l++;
-	p++;
-      }
-      bufU[l > 7 ? 7 : l] = '\0';
-      if (l > 1 && l < 7 && bufU[0] >> 6 == 3) /* the 1st byte in UTF8 character has the most significant bits set to "11" */
-      {
-	*q = '\0'; /* mark that character has not been converted yet */
-	switch (strlen(bufU)) {
-	  case 2: 
-	    switch (bufU[0]) {
-	      case 0xC2 : 
-		(*q) = ((bufU[1]) | 0x80) & 0xBF; /* A0-BF and a few 80-9F */
-		if((*q)==0xA0)
-		  (*q)=' ';
-		break;
-	      case 0xC3 : 
-		(*q) = (bufU[1]) | 0xC0; /* C0-FF */
-		break;
-	      case 0xC5 : 
-		switch (bufU[1]) { 
-		  case 0x92 : (*q) = 140; break;
-		  case 0x93 : (*q) = 156; break;
-		  case 0xA0 : (*q) = 138; break;
-		  case 0xA1 : (*q) = 154; break;
-		  case 0xB8 : (*q) = 143; break;
-		  case 0xBD : (*q) = 142; break;
-		  case 0xBE : (*q) = 158; break;
-		}
-		break;
-	      case 0xC6: 
-		switch (bufU[1]) { 
-		  case 0x92 : (*q) = 131; break;
-		}
-		break;
-	      case 0xCB : 
-		switch (bufU[1]) { 
-		  case 0x86 : (*q) = 136; break;
-		  case 0x9C : (*q) = 152; break;
-		}
-		break;
-	    }
-	    break;
-	  case 3:
-	    switch (bufU[0]) {
-	      case 0xE2 : 
-		switch (bufU[1]) { 
-		  case 0x80 : 
-		    switch (bufU[2]) { 
-		      case 0x93 : (*q) = 150; break;
-		      case 0x94 : (*q) = 151; break;
-		      case 0x98 : (*q) = 145; break;
-		      // case 0x99 : (*q) = 146; break;
-		      case 0x99 : (*q) = '\''; break;
-		      case 0x9A : (*q) = 130; break;
-		      case 0x9C : (*q) = 147; break;
-		      case 0x9D : (*q) = 148; break;
-		      case 0x9E : (*q) = 132; break;
-		      case 0xA0 : (*q) = 134; break;
-		      case 0xA1 : (*q) = 135; break;
-		      case 0xA2 : (*q) = 149; break;
-		      case 0xA6 : (*q) = 133; break;
-		      case 0xB0 : (*q) = 137; break;
-		      case 0xB9 : (*q) = 139; break;
-		      case 0xBA : (*q) = 155; break;
-		    }
-		    break;
-		  case 0x82 : 
-		    switch (bufU[2]) { 
-		      case 0xAC : (*q) = 128; break;
-		    }
-		    break;
-		  case 0x84 : 
-		    switch (bufU[2]) { 
-		      case 0xA2 : (*q) = 153; break;
-		    }
-		    break;
-		}
-		break;
-	    }
-	    break;
-	}
-
-	if (*q!='\0' && filtre(*q)==1)
-	{
-	  q++;
-	  i--;
-	}
-	else
-	{
-#ifdef DEBUG_FILETXT
-	  log_trace("UTF2Lat: UTF character [%s] absent in Latin1 - dropped (%c, 0x%x)\n", bufU,*q,*q);
+    { /* Ascii UCS */
+#ifdef DEBUG_TXT
+      log_info("UTF8 Ascii UCS 0x%02x\n", *p);
 #endif
-	  /* There is already a null but don't care */
-	  *q = '\0';
-	  return(p_org-buffer);
-	}
-      }
-      else
-      {
-	if(l==1 && filtre(bufU[0])==1)
-	{ /* Windows uses forbidden chars ie 0x85 */
-	  i--;
-	  *q=bufU[0];
-	  q++;
-	}
-	else
-	{
-#ifdef DEBUG_FILETXT
-	  log_trace( "UTF2Lat: non-UTF character [%s](0x%x) - dropped\n", bufU,bufU[0]);
-#endif
-	  *q = '\0';
-	  return(p_org-buffer);
-	}
-      }
+      *q = tolower(*p++);
     }
+    if (*q=='\0' || filtre(*q)==0)
+    {
+#ifdef DEBUG_TXT
+      log_warning("UTF2Lat reject 0x%x\n",*q);
+#endif
+      *q = '\0';
+      return(p_org-buffer);
+    }
+    q++;
+    i--;
   }
   *q = '\0';
-/*
-  if (*p)
-  {
-    log_trace("UTF2Lat: buffer too small - string had to be truncated\n");
-  }
-*/
   return(p-buffer);
 }
 
-static inline const unsigned char *find_in_mem(const unsigned char *haystack, const unsigned char * haystack_end,
-    const unsigned char *needle, const unsigned int needle_length)
-{
-  while(haystack!=NULL)
-  {
-    haystack=memchr(haystack,needle[0],haystack_end-haystack);
-    if(haystack!=NULL && haystack<=(haystack_end-needle_length))
-    {
-      if(memcmp(haystack,needle,needle_length)==0)
-	return haystack;
-      haystack++;
-    }
-    else
-      haystack=NULL;
-  };
-  return NULL;
-}
-
 static int header_check_fasttxt(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  const char sign_grisbi[]		= "Version_grisbi";
+  const char sign_grisbi[14]		= "Version_grisbi";
+  const char sign_fst[5]                 = "QBFSD";
+  if(memcmp(buffer,header_cls,sizeof(header_cls))==0)
+  {
+    reset_file_recovery(file_recovery_new);
+    file_recovery_new->data_check=&data_check_txt;
+    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->extension="cls";
+    return 1;
+  }
   /* Incredimail has .imm extension but this extension isn't frequent */
   if(memcmp(buffer,header_imm,sizeof(header_imm))==0 ||
       memcmp(buffer,header_imm2,sizeof(header_imm2))==0 ||
       memcmp(buffer,header_mail,sizeof(header_mail))==0)
   {
     if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
-	file_recovery->file_stat->file_hint==&file_hint_fasttxt &&
-	strcmp(file_recovery->extension,"imm")==0)
-    {
+        file_recovery->file_stat->file_hint==&file_hint_fasttxt &&
+        strcmp(file_recovery->extension,"imm")==0)
       return 0;
-    }
     reset_file_recovery(file_recovery_new);
     file_recovery_new->data_check=NULL;
     file_recovery_new->extension="imm";
@@ -420,16 +374,18 @@ static int header_check_fasttxt(const unsigned char *buffer, const unsigned int
   {
     reset_file_recovery(file_recovery_new);
     file_recovery_new->data_check=&data_check_txt;
-    if(strstr(buffer,sign_grisbi)!=NULL)
+    if(td_memmem(buffer, buffer_size, sign_grisbi, sizeof(sign_grisbi))!=NULL)
       file_recovery_new->extension="gsb";
-    else if(strstr(buffer,sign_html)!=NULL)
+    else if(td_memmem(buffer, buffer_size, sign_fst, sizeof(sign_fst))!=NULL)
+      file_recovery_new->extension="fst";
+    else if(td_memmem(buffer, buffer_size, sign_html, sizeof(sign_html))!=NULL)
     {
       file_recovery_new->extension="html";
       file_recovery_new->file_check=&file_check_html;
     }
     else
       file_recovery_new->extension="xml";
-    file_recovery_new->file_check=&file_check_size;
+    file_recovery_new->file_check=&file_check_xml;
     return 1;
   }
   return 0;
@@ -453,12 +409,14 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
   const char sign_jsp2[]		= "<%=";
   const char sign_php[]			= "<?php";
   const char sign_tex[]			= "\\begin{";
-  const unsigned char *haystack_end;
   {
     unsigned int tmp=0;
     for(i=0;i<10 && isdigit(buffer[i]);i++)
       tmp=tmp*10+buffer[i]-'0';
-    if(buffer[i]==0x0a && memcmp(buffer+i+1, header_imm2, sizeof(header_imm2))==0)
+    if(buffer[i]==0x0a && memcmp(buffer+i+1, header_imm2, sizeof(header_imm2))==0 &&
+        !(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+          file_recovery->file_stat->file_hint==&file_hint_fasttxt &&
+          strcmp(file_recovery->extension,"imm")==0))
     {
       reset_file_recovery(file_recovery_new);
       file_recovery_new->calculated_file_size=tmp+i+1;
@@ -468,19 +426,19 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
       return 1;
     }
   }
-  /* Don't malloc/free memory every time, small memory leak */
   if(buffer_lower_size<buffer_size+16)
   {
     free(buffer_lower);
     buffer_lower=NULL;
   }
+  /* Don't malloc/free memory every time, small memory leak */
   if(buffer_lower==NULL)
   {
     buffer_lower_size=buffer_size+16;
     buffer_lower=MALLOC(buffer_lower_size);
   }
-  haystack_end=buffer_lower+buffer_size;
   i=UTF2Lat(buffer_lower,buffer,buffer_size);
+  /* strncasecmp */
   if(memcmp(buffer_lower,header_bat,sizeof(header_bat))==0)
   {
     reset_file_recovery(file_recovery_new);
@@ -507,22 +465,21 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
   }
   if(buffer[0]=='#' && buffer[1]=='!')
   {
-    unsigned int haystack_size;
+    unsigned int l=i-2;
     const unsigned char *haystack=buffer_lower+2;
     const unsigned char *res;
-    haystack_size=buffer_size-2;
-    res=memchr(haystack,'\n',haystack_size);
+    res=memchr(haystack,'\n',l);
     if(res!=NULL)
-      haystack_end=res;
-    if(find_in_mem(haystack,haystack_end,header_sig_perl,sizeof(header_sig_perl)) != NULL)
+      l=res-haystack;
+    if(td_memmem(haystack, l, header_sig_perl, sizeof(header_sig_perl)) != NULL)
     {
       reset_file_recovery(file_recovery_new);
       file_recovery_new->data_check=&data_check_txt;
-    file_recovery_new->file_check=&file_check_size;
+      file_recovery_new->file_check=&file_check_size;
       file_recovery_new->extension="pl";
       return 1;
     }
-    if(find_in_mem(haystack,haystack_end,header_sig_python,sizeof(header_sig_python)) != NULL)
+    if(td_memmem(haystack, l, header_sig_python, sizeof(header_sig_python)) != NULL)
     {
       reset_file_recovery(file_recovery_new);
       file_recovery_new->data_check=&data_check_txt;
@@ -530,7 +487,7 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
       file_recovery_new->extension="py";
       return 1;
     }
-    if(find_in_mem(haystack,haystack_end,header_sig_ruby,sizeof(header_sig_ruby)) != NULL)
+    if(td_memmem(haystack, l, header_sig_ruby, sizeof(header_sig_ruby)) != NULL)
     {
       reset_file_recovery(file_recovery_new);
       file_recovery_new->data_check=&data_check_txt;
@@ -575,21 +532,20 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
       unsigned int stats[256];
       unsigned int j;
       double ind=0;
-      for(j=0;j<256;j++)
-	stats[j]=0;
+      memset(&stats, 0, sizeof(stats));
       for(j=0;j<i;j++)
-	stats[buffer[j]]++;
+        stats[buffer[j]]++;
       for(j=0;j<256;j++)
-	if(stats[j]>0)
-	  ind+=stats[j]*(stats[j]-1);
+        if(stats[j]>0)
+          ind+=stats[j]*(stats[j]-1);
       ind=ind/i/(i-1);
       if(ind<0.03 || ind>0.90)
-	ext=NULL;
+        ext=NULL;
       else
-	ext=file_hint_txt.extension;
+        ext=file_hint_txt.extension;
     }
     if(ext!=NULL && strcmp(ext,"txt")==0 &&
-      (strstr(buffer_lower,"<br>")!=NULL || strstr(buffer_lower,"<p>")!=NULL))
+        (strstr(buffer_lower,"<br>")!=NULL || strstr(buffer_lower,"<p>")!=NULL))
     {
       ext="html";
     }
@@ -597,61 +553,61 @@ static int header_check_txt(const unsigned char *buffer, const unsigned int buff
     {
       const unsigned char zip_header[4]  = { 'P', 'K', 0x03, 0x04};
       if(strcmp(ext,"html")==0 &&
-	  file_recovery->file_stat->file_hint==&file_hint_txt &&
-	  strstr(file_recovery->filename,"")!=NULL)
+          file_recovery->file_stat->file_hint==&file_hint_txt &&
+          strstr(file_recovery->filename,"")!=NULL)
       {
-	return 0;
+        return 0;
       }
 
       /* Special case: doc, texte files
-	Unix: \n (0xA)
-	Dos: \r\n (0xD 0xA)
-	Doc: \r (0xD)
-      */
+Unix: \n (0xA)
+Dos: \r\n (0xD 0xA)
+Doc: \r (0xD)
+       */
       if(file_recovery->file_stat->file_hint==&file_hint_doc &&
-	  strstr(file_recovery->filename,".doc")!=NULL)
+          strstr(file_recovery->filename,".doc")!=NULL)
       {
-	unsigned int j;
-	unsigned int txt_nl=0;
-	for(j=0;j<i-1;j++)
-	  if(buffer_lower[j]=='\r' && buffer_lower[j+1]!='\n')
-	  {
-	    return 0;
-	  }
-	for(j=0;j<i && j<512;j++)
-	  if(buffer[j]=='\n')
-	    txt_nl=1;
-	if(txt_nl==1)
-	{
-	  /* log_trace(">%s<\ndoc => %s\n",buffer_lower,ext); */
-	  reset_file_recovery(file_recovery_new);
-	  file_recovery_new->data_check=&data_check_txt;
-	  file_recovery_new->file_check=&file_check_size;
-	  file_recovery_new->extension=ext;
-	  return 1;
-	}
+        unsigned int j;
+        unsigned int txt_nl=0;
+        for(j=0;j<i-1;j++)
+          if(buffer_lower[j]=='\r' && buffer_lower[j+1]!='\n')
+          {
+            return 0;
+          }
+        for(j=0;j<i && j<512;j++)
+          if(buffer[j]=='\n')
+            txt_nl=1;
+        if(txt_nl==1)
+        {
+          /* log_trace(">%s<\ndoc => %s\n",buffer_lower,ext); */
+          reset_file_recovery(file_recovery_new);
+          file_recovery_new->data_check=&data_check_txt;
+          file_recovery_new->file_check=&file_check_size;
+          file_recovery_new->extension=ext;
+          return 1;
+        }
       }
       buffer_lower[511]='\0';
       /* Special case: two consecutive HTML files */
       if((strcmp(ext,"html")==0 &&
-	    strstr(buffer_lower,sign_html)!=NULL &&
-	    strstr(file_recovery->filename,".html")!=NULL) ||
-	  /* Text should not be found in JPEG */
-	  (file_recovery->file_stat->file_hint==&file_hint_jpg &&
-	   find_in_mem(buffer,buffer+buffer_size,"8BIM",4)==NULL &&
-	   find_in_mem(buffer,buffer+buffer_size,"adobe",5)==NULL) ||
-	  /* Text should not be found in zip because of compression */
-	  (file_recovery->file_stat->file_hint==&file_hint_zip && find_in_mem(buffer,buffer+buffer_size,zip_header,4)==NULL))
+            strstr(buffer_lower,sign_html)!=NULL &&
+            strstr(file_recovery->filename,".html")!=NULL) ||
+          /* Text should not be found in JPEG */
+          (file_recovery->file_stat->file_hint==&file_hint_jpg &&
+           td_memmem(buffer, buffer_size, "8BIM", 4)==NULL &&
+           td_memmem(buffer, buffer_size, "adobe", 5)==NULL) ||
+          /* Text should not be found in zip because of compression */
+          (file_recovery->file_stat->file_hint==&file_hint_zip && td_memmem(buffer, buffer_size, zip_header, 4)==NULL))
       {
-	reset_file_recovery(file_recovery_new);
-	file_recovery_new->data_check=&data_check_txt;
-	file_recovery_new->file_check=&file_check_size;
-	file_recovery_new->extension=ext;
-	return 1;
+        reset_file_recovery(file_recovery_new);
+        file_recovery_new->data_check=&data_check_txt;
+        file_recovery_new->file_check=&file_check_size;
+        file_recovery_new->extension=ext;
+        return 1;
       }
       return 0;
     }
-/*    log_trace("ext=%s\n",ext); */
+    /*    log_trace("ext=%s\n",ext); */
     if(ext!=NULL)
     {
       reset_file_recovery(file_recovery_new);
@@ -698,26 +654,27 @@ static void file_check_html(file_recovery_t *file_recovery)
   {
     const int read_size=1024;
     int taille;
-    char *buffer;
+    char *buffer_lower;
     int i;
     if(fseek(file_recovery->handle,0,SEEK_SET)<0)
       return;
-    buffer=MALLOC(read_size);
-    taille=fread(buffer,1,read_size,file_recovery->handle);
+    buffer_lower=MALLOC(read_size);
+    taille=fread(buffer_lower,1,read_size,file_recovery->handle);
     if(taille<0)
     {
-      free(buffer);
+      free(buffer_lower);
       return;
     }
+    buffer_lower[taille<read_size?taille:read_size-1]='\0';
+    /* TODO: use strcasestr if available */
     for(i=0;i<taille;i++)
-      buffer[i]=tolower(buffer[i]);
-    buffer[taille<read_size?taille:read_size-1]='\0';
-    if(strstr(buffer, sign_html)==NULL)
+      buffer_lower[i]=tolower(buffer_lower[i]);
+    if(strstr(buffer_lower, sign_html)==NULL)
     {
       log_warning("%s: no header\n",file_recovery->filename);
       file_recovery->file_size=0;
     }
-    free(buffer);
+    free(buffer_lower);
   }
 }
 
@@ -733,3 +690,10 @@ static void file_check_emlx(file_recovery_t *file_recovery)
     file_search_footer(file_recovery, emlx_footer,sizeof(emlx_footer));
   }
 }
+
+static void file_check_xml(file_recovery_t *file_recovery)
+{
+  const unsigned char xml_footer[1]= { '>'};
+  file_search_footer(file_recovery, xml_footer, sizeof(xml_footer));
+  file_allow_nl(file_recovery, NL_BARENL|NL_CRLF|NL_BARECR);
+}
