@@ -38,8 +38,9 @@
 #include "testdisk.h"
 #include "analyse.h"
 #include "log.h"
+#include "guid_cpy.h"
 
-static unsigned int get_geometry_from_list_part_aux(const disk_t *disk_car, const list_part_t *list_part, const int debug);
+static unsigned int get_geometry_from_list_part_aux(const disk_t *disk_car, const list_part_t *list_part, const int verbose);
 
 unsigned long int C_H_S2LBA(const disk_t *disk_car,const unsigned int C, const unsigned int H, const unsigned int S)
 { return ((unsigned long int)C*(disk_car->CHS.head+1)+H)*disk_car->CHS.sector+S-1;
@@ -71,30 +72,37 @@ void offset2CHS(const disk_t *disk_car,const uint64_t offset, CHS_t*CHS)
   CHS->cylinder=pos/(disk_car->CHS.head+1);
 }
 
-void dup_CHS(CHS_t * CHS_dest, const CHS_t * CHS_source)
+void dup_CHS(CHS_t * CHS_dst, const CHS_t * CHS_source)
 {
-  CHS_dest->cylinder=CHS_source->cylinder;
-  CHS_dest->head=CHS_source->head;
-  CHS_dest->sector=CHS_source->sector;
+  CHS_dst->cylinder=CHS_source->cylinder;
+  CHS_dst->head=CHS_source->head;
+  CHS_dst->sector=CHS_source->sector;
 }
 
-void dup_partition_t(partition_t *dest, const partition_t *src)
+void dup_partition_t(partition_t *dst, const partition_t *src)
 {
-  dest->part_offset=src->part_offset;
-  dest->part_size=src->part_size;
-  dest->boot_sector=src->boot_sector;
-  dest->blocksize=src->blocksize;
-  dest->part_type_i386=src->part_type_i386;
-  dest->part_type_sun=src->part_type_sun;
-  dest->part_type_mac=src->part_type_mac;
-  dest->part_type_xbox=src->part_type_xbox;
-  dest->upart_type=src->upart_type;
-  dest->status=src->status;
-  dest->order=src->order;
-  dest->errcode=src->errcode;
-  strncpy(dest->info,src->info,sizeof(dest->info));
-  strncpy(dest->name,src->name,sizeof(dest->name));
-  dest->arch=src->arch;
+#if 0
+  dst->part_offset=src->part_offset;
+  dst->part_size=src->part_size;
+  dst->boot_sector=src->boot_sector;
+  dst->boot_sector_size=src->boot_sector_size;
+  dst->blocksize=src->blocksize;
+  dst->part_type_i386=src->part_type_i386;
+  dst->part_type_sun=src->part_type_sun;
+  dst->part_type_mac=src->part_type_mac;
+  dst->part_type_xbox=src->part_type_xbox;
+  dst->part_type_gpt=src->part_type_gpt;
+  dst->upart_type=src->upart_type;
+  dst->status=src->status;
+  dst->order=src->order;
+  dst->errcode=src->errcode;
+  strncpy(dst->info,src->info,sizeof(dst->info));
+  strncpy(dst->fsname,src->name,sizeof(dst->fsname));
+  strncpy(dst->partname,src->name,sizeof(dst->partname));
+  dst->arch=src->arch;
+#else
+  memcpy(dst, src, sizeof(*src));
+#endif
 }
 
 list_disk_t *insert_new_disk(list_disk_t *list_disk, disk_t *disk_car)
@@ -133,56 +141,39 @@ list_disk_t *insert_new_disk(list_disk_t *list_disk, disk_t *disk_car)
   }
 }
 
-list_part_t *insert_new_partition(list_part_t *list_part, partition_t *part)
+list_part_t *insert_new_partition(list_part_t *list_part, partition_t *part, const int force_insert, int *insert_error)
 {
-  list_part_t *new_list_part;
-#ifdef DEBUG
-  check_list_part(list_part);
-#endif
-  new_list_part=insert_new_partition_aux(list_part, element_new(part),0);
-#ifdef DEBUG
-  check_list_part(new_list_part);
-#endif
-  return new_list_part;
-}
-
-list_part_t *insert_new_partition_aux(list_part_t *list_part, list_part_t *new_element, const unsigned int dont_free_part)
-{
-/*dont_free_part
-  0 if new_element->part is already known, don't insert it and free it
-  1 if new_element->part is already known, don't insert it but free it
-  2 even if new_element->part is already known, insert it
-*/
- /* new partition mustn't be used after insert !*/  list_part_t *prev=NULL;
+  list_part_t *prev=NULL;
   list_part_t *next;
+  *insert_error=0;
   for(next=list_part;;next=next->next)
   { /* prev new next */
     if((next==NULL)||
-      (new_element->part->part_offset<next->part->part_offset) ||
-      (new_element->part->part_offset==next->part->part_offset &&
-       ((new_element->part->part_size<next->part->part_size) ||
-	(new_element->part->part_size==next->part->part_size && (dont_free_part!=2 || new_element->part->boot_sector<next->part->boot_sector)))))
+      (part->part_offset<next->part->part_offset) ||
+      (part->part_offset==next->part->part_offset &&
+       ((part->part_size<next->part->part_size) ||
+	(part->part_size==next->part->part_size && (force_insert==0 || part->sb_offset < next->part->sb_offset)))))
     {
-      if(dont_free_part!=2 &&
-	(next!=NULL)&&(next->part->part_offset==new_element->part->part_offset) &&
-	(next->part->part_size==new_element->part->part_size) &&
-	(next->part->part_type_i386==new_element->part->part_type_i386) &&
-	(next->part->part_type_mac==new_element->part->part_type_mac) &&
-	(next->part->part_type_sun==new_element->part->part_type_sun) &&
-	(next->part->part_type_xbox==new_element->part->part_type_xbox) &&
-	(next->part->upart_type==new_element->part->upart_type || new_element->part->upart_type==UP_UNK))
+      if(force_insert==0 &&
+	(next!=NULL) &&
+	(next->part->part_offset==part->part_offset) &&
+	(next->part->part_size==part->part_size) &&
+	(next->part->part_type_i386==part->part_type_i386) &&
+	(next->part->part_type_mac==part->part_type_mac) &&
+	(next->part->part_type_sun==part->part_type_sun) &&
+	(next->part->part_type_xbox==part->part_type_xbox) &&
+	(next->part->upart_type==part->upart_type || part->upart_type==UP_UNK))
       { /*CGR 2004/05/31*/
 	if(next->part->status==STATUS_DELETED)
 	{
-	  next->part->status=new_element->part->status;
+	  next->part->status=part->status;
 	}
-	if(dont_free_part==0)
-	  free(new_element->part);
-	free(new_element);
+	*insert_error=1;
 	return list_part;
       }
-      else
       { /* prev new_element next */
+	list_part_t *new_element;
+	new_element=element_new(part);
 	new_element->next=next;
 	new_element->prev=prev;
 	if(next!=NULL)
@@ -209,8 +200,8 @@ int delete_list_disk(list_disk_t *list_disk)
     write_used|=element_disk->disk->write_used;
     if(element_disk->disk->clean!=NULL)
       element_disk->disk->clean(element_disk->disk);
-    if(element_disk->disk->device!=NULL)
-      free(element_disk->disk->device);
+    free(element_disk->disk->device);
+    free(element_disk->disk->model);
     free(element_disk->disk);
     free(element_disk);
     element_disk=element_disk_next;
@@ -246,7 +237,7 @@ int check_list_part(list_part_t *list_part)
   return 0;
 }
 
-list_part_t *sorlist_part_t(list_part_t *list_part)
+list_part_t *sort_partition_list(list_part_t *list_part)
 {
   list_part_t *new_list_part=NULL;
   list_part_t *element;
@@ -256,8 +247,12 @@ list_part_t *sorlist_part_t(list_part_t *list_part)
 #endif
   for(element=list_part;element!=NULL;element=next)
   {
+    int insert_error=0;
     next=element->next;
-    new_list_part=insert_new_partition_aux(new_list_part,element,0);
+    new_list_part=insert_new_partition(new_list_part, element->part, 0, &insert_error);
+    if(insert_error>0)
+      free(element->part);
+    free(element);
   }
 #ifdef DEBUG
   check_list_part(new_list_part);
@@ -265,13 +260,26 @@ list_part_t *sorlist_part_t(list_part_t *list_part)
   return new_list_part;
 }
 
-void delete_list_part(list_part_t *list_part)
+list_part_t *gen_sorted_partition_list(const list_part_t *list_part)
+{
+  list_part_t *new_list_part=NULL;
+  const list_part_t *element;
+  for(element=list_part;element!=NULL;element=element->next)
+  {
+    int insert_error=0;
+    if(element->part->status!=STATUS_DELETED)
+      new_list_part=insert_new_partition(new_list_part, element->part, 1, &insert_error);
+  }
+  return new_list_part;
+}
+
+/* Delete the list and its content */
+void part_free_list(list_part_t *list_part)
 {
   list_part_t *element;
 #ifdef DEBUG
   check_list_part(list_part);
 #endif
-  /* Libere la memoire */
   element=list_part;
   while(element!=NULL)
   {
@@ -282,29 +290,77 @@ void delete_list_part(list_part_t *list_part)
   }
 }
 
-void  partition_reset(partition_t *partition)
+/* Free the list but not its content */
+void part_free_list_only(list_part_t *list_part)
+{
+  list_part_t *element;
+#ifdef DEBUG
+  check_list_part(list_part);
+#endif
+  element=list_part;
+  while(element!=NULL)
+  {
+    list_part_t *next=element->next;
+    free(element);
+    element=next;
+  }
+}
+
+int is_part_overlapping(const list_part_t *list_part)
+{
+  const list_part_t *element;
+  /* Test overlapping
+     Must be space between a primary/logical partition and a logical partition for an extended
+  */
+  if(list_part==NULL)
+    return 0;
+  element=list_part;
+  while(1)
+  {
+    const list_part_t *next=element->next;
+    const partition_t *partition=element->part;
+    if(next==NULL)
+      return 0;
+    if( (partition->part_offset + partition->part_size - 1 >= next->part->part_offset)		||
+	((partition->status==STATUS_PRIM ||
+	  partition->status==STATUS_PRIM_BOOT ||
+	  partition->status==STATUS_LOG) &&
+	 next->part->status==STATUS_LOG &&
+	 partition->part_offset + partition->part_size - 1 + 1 >= next->part->part_offset))
+      return 1;
+    element=next;
+  }
+  return 0;
+}
+
+void  partition_reset(partition_t *partition, const arch_fnct_t *arch)
 {
 /* partition->lba=0; Don't reset lba, used by search_part */
   partition->part_size=(uint64_t)0;
-  partition->boot_sector=0;
+  partition->sborg_offset=0;
+  partition->sb_offset=0;
+  partition->sb_size=0;
   partition->blocksize=0;
   partition->part_type_i386=P_NO_OS;
   partition->part_type_sun=PSUN_UNK;
   partition->part_type_mac=PMAC_UNK;
   partition->part_type_xbox=PXBOX_UNK;
+  partition->part_type_gpt=GPT_ENT_TYPE_UNUSED;
+  guid_cpy(&partition->part_uuid, &GPT_ENT_TYPE_UNUSED);
   partition->upart_type=UP_UNK;
   partition->status=STATUS_DELETED;
   partition->order=NO_ORDER;
   partition->errcode=BAD_NOERR;
-  partition->name[0]='\0';
+  partition->fsname[0]='\0';
+  partition->partname[0]='\0';
   partition->info[0]='\0';
-  partition->arch=NULL;
+  partition->arch=arch;
 }
 
-partition_t *partition_new()
+partition_t *partition_new(const arch_fnct_t *arch)
 {
   partition_t *partition=(partition_t *)MALLOC(sizeof(*partition));
-  partition_reset(partition);
+  partition_reset(partition, arch);
   return partition;
 }
 
@@ -317,7 +373,7 @@ list_part_t *element_new(partition_t *part)
   return new_element;
 }
 
-static unsigned int get_geometry_from_list_part_aux(const disk_t *disk_car, const list_part_t *list_part, const int debug)
+static unsigned int get_geometry_from_list_part_aux(const disk_t *disk_car, const list_part_t *list_part, const int verbose)
 {
   const list_part_t *element;
   unsigned int nbr=0;
@@ -340,7 +396,7 @@ static unsigned int get_geometry_from_list_part_aux(const disk_t *disk_car, cons
   if(nbr>0)
   {
     log_info("get_geometry_from_list_part_aux head=%u nbr=%u\n",disk_car->CHS.head+1,nbr);
-    if(debug>1)
+    if(verbose>1)
     {
       for(element=list_part;element!=NULL;element=element->next)
       {
@@ -358,7 +414,7 @@ static unsigned int get_geometry_from_list_part_aux(const disk_t *disk_car, cons
   return nbr;
 }
 
-unsigned int get_geometry_from_list_part(const disk_t *disk_car, const list_part_t *list_part, const int debug)
+unsigned int get_geometry_from_list_part(const disk_t *disk_car, const list_part_t *list_part, const int verbose)
 {
   const unsigned int head_list[]={8,16,32,64,128,240,255,0};
   unsigned int nbr_max;
@@ -367,11 +423,11 @@ unsigned int get_geometry_from_list_part(const disk_t *disk_car, const list_part
   unsigned int head_max=disk_car->CHS.head;
   disk_t *new_disk_car=MALLOC(sizeof(*new_disk_car));
   memcpy(new_disk_car,disk_car,sizeof(*new_disk_car));
-  nbr_max=get_geometry_from_list_part_aux(new_disk_car, list_part, debug);
+  nbr_max=get_geometry_from_list_part_aux(new_disk_car, list_part, verbose);
   for(h_index=0;head_list[h_index]!=0;h_index++)
   {
     new_disk_car->CHS.head=head_list[h_index]-1;
-    nbr=get_geometry_from_list_part_aux(new_disk_car, list_part, debug);
+    nbr=get_geometry_from_list_part_aux(new_disk_car, list_part, verbose);
     if(nbr>=nbr_max)
     {
       nbr_max=nbr;
