@@ -2,7 +2,7 @@
 
     File: fnctdsk.h
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,15 +30,17 @@ void offset2CHS(const disk_t *disk_car,const uint64_t offset, CHS_t*CHS);
 void dup_CHS(CHS_t * CHS_dest, const CHS_t * CHS_source);
 list_part_t *element_new(partition_t *part);
 list_disk_t *insert_new_disk(list_disk_t *list_disk, disk_t *disk_car);
-list_part_t *insert_new_partition(list_part_t *list_part, partition_t *part);
-list_part_t *insert_new_partition_aux(list_part_t *list_part, list_part_t *new_element, const unsigned int dont_free_part);
+list_part_t *insert_new_partition(list_part_t *list_part, partition_t *part, const int force_insert, int *insert_error);
 list_part_t *remove_partition(list_part_t *list_part, list_part_t *element2removed);
-list_part_t *sorlist_part_t(list_part_t *list_part);
-void delete_list_part(list_part_t *list_part);
-void  partition_reset(partition_t *partition);
-partition_t *partition_new(void);
+list_part_t *sort_partition_list(list_part_t *list_part);
+list_part_t *gen_sorted_partition_list(const list_part_t *list_part);
+void part_free_list(list_part_t *list_part);
+void part_free_list_only(list_part_t *list_part);
+void  partition_reset(partition_t *partition, const arch_fnct_t *arch);
+partition_t *partition_new(const arch_fnct_t *arch);
 int check_list_part(list_part_t *list_part);
-unsigned int get_geometry_from_list_part(const disk_t *disk_car, const list_part_t *list_part, const int debug);
+unsigned int get_geometry_from_list_part(const disk_t *disk_car, const list_part_t *list_part, const int verbose);
 int delete_list_disk(list_disk_t *list_disk);
 const char *size_to_unit(uint64_t disk_size, char *buffer);
+int is_part_overlapping(const list_part_t *list_part);
 
