@@ -19,5 +19,5 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
-t_param_disk *new_diskcache(t_param_disk *disk_car, const unsigned int cache_size_min);
+disk_t *new_diskcache(disk_t *disk_car, const unsigned int cache_size_min);
 
