@@ -51,28 +51,28 @@ struct hfs_mdb {
 					  file or directory created */
   uint16_t drFreeBks;                  /* 0x22 number of free allocation blocks */
   uint8_t  drVN[28];                   /* 0x24 the volume label */
-  uint32_t drVolBkUp;                  /* fs backup date/time */
-  uint16_t drVSeqNum;                  /* backup sequence number */
-  uint32_t drWrCnt;                    /* fs write count */
-  uint32_t drXTClpSiz;                 /* clumpsize for the extents B-tree */
-  uint32_t drCTClpSiz;                 /* clumpsize for the catalog B-tree */
-  uint16_t drNmRtDirs;                 /* number of directories in
+  uint32_t drVolBkUp;                  /* 0x40 fs backup date/time */
+  uint16_t drVSeqNum;                  /* 0x44 backup sequence number */
+  uint32_t drWrCnt;                    /* 0x46 fs write count */
+  uint32_t drXTClpSiz;                 /* 0x4a clumpsize for the extents B-tree */
+  uint32_t drCTClpSiz;                 /* 0x4e clumpsize for the catalog B-tree */
+  uint16_t drNmRtDirs;                 /* 0x52 number of directories in
 					  the root directory */
-  uint32_t drFilCnt;                   /* number of files in the fs */
-  uint32_t drDirCnt;                   /* number of directories in the fs */
-  uint8_t  drFndrInfo[32];             /* data used by the Finder */
-  uint16_t drEmbedSigWord;             /* embedded volume signature */
-  uint32_t drEmbedExtent;              /* starting block number (xdrStABN)
+  uint32_t drFilCnt;                   /* 0x54 number of files in the fs */
+  uint32_t drDirCnt;                   /* 0x58 number of directories in the fs */
+  uint8_t  drFndrInfo[32];             /* 0x5c data used by the Finder */
+  uint16_t drEmbedSigWord;             /* 0x7c embedded volume signature */
+  uint32_t drEmbedExtent;              /* 0x7e starting block number (xdrStABN)
 					  and number of allocation blocks
 					  (xdrNumABlks) occupied by embedded
 					  volume */
-  uint32_t drXTFlSize;                 /* bytes in the extents B-tree */
-  hfs_extent_rec drXTExtRec;           /* extents B-tree's first 3 extents */
-  uint32_t drCTFlSize;                 /* bytes in the catalog B-tree */
-  hfs_extent_rec drCTExtRec;      /* catalog B-tree's first 3 extents */
+  uint32_t drXTFlSize;                 /* 0x82 bytes in the extents B-tree */
+  hfs_extent_rec drXTExtRec;           /* 0x86 extents B-tree's first 3 extents */
+  uint32_t drCTFlSize;                 /* 0x92 bytes in the catalog B-tree */
+  hfs_extent_rec drCTExtRec;           /* 0x96 catalog B-tree's first 3 extents */
 } __attribute__ ((__packed__));
-int check_HFS(t_param_disk *disk_car,t_partition *partition,const int debug);
-int test_HFS(t_param_disk *disk_car, const hfs_mdb_t *hfs_mdb,t_partition *partition,const int debug, const int dump_ind);
-int recover_HFS(t_param_disk *disk_car, const hfs_mdb_t *hfs_mdb,t_partition *partition,const int debug, const int dump_ind, const int backup);
+int check_HFS(disk_t *disk_car,partition_t *partition,const int verbose);
+int test_HFS(disk_t *disk_car, const hfs_mdb_t *hfs_mdb,partition_t *partition,const int verbose, const int dump_ind);
+int recover_HFS(disk_t *disk_car, const hfs_mdb_t *hfs_mdb,partition_t *partition,const int verbose, const int dump_ind, const int backup);
 
 #endif /* _HFS_H */
