@@ -25,10 +25,10 @@
 #define INTER_STRUCTURE	13
 #define INTER_BAD_PART	10
 
-int do_curses_testdisk(int debug, int dump_ind, const t_list_disk *list_disk, const int saveheader, const char *cmd_device, char **current_cmd);
-int interface_write(t_param_disk *disk_car,t_list_part *list_part,const int can_search_deeper, const int can_ask_minmax_ext, char **current_cmd, unsigned int *menu);
-
-t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const int debug, char **current_cmd);
-void interface_list(t_param_disk *disk_car, const int debug,const int test_recovery, const int saveheader, const int backup);
-int interface_superblock(t_param_disk *disk_car,t_list_part *list_part,char**current_cmd);
+int do_curses_testdisk(int verbose, int dump_ind, const list_disk_t *list_disk, const int saveheader, const char *cmd_device, char **current_cmd);
+int interface_write(disk_t *disk_car,list_part_t *list_part,const int can_search_deeper, const int can_ask_minmax_ext, int *no_confirm, char **current_cmd, unsigned int *menu);
 
+list_part_t *ask_structure(disk_t *disk_car,list_part_t *list_part, const int verbose, char **current_cmd);
+void interface_list(disk_t *disk_car, const int verbose, const int saveheader, const int backup, char **current_cmd);
+int interface_superblock(disk_t *disk_car,list_part_t *list_part,char**current_cmd);
+int ask_testdisk_log_creation(void);
