@@ -23,6 +23,6 @@
 #define JFS_SUPERBLOCK_SIZE 512
 
 #define L2BPERDMAP 13      /* l2 num of blks per dmap */
-int check_JFS(t_param_disk *disk_car,t_partition *partition,const int debug);
-int recover_JFS(t_param_disk *disk_car, const struct jfs_superblock *sb,t_partition *partition,const int debug, const int dump_ind);
-int set_JFS_info(t_param_disk *disk_car, const struct jfs_superblock *sb,t_partition *partition,const int debug, const int dump_ind);
+int check_JFS(disk_t *disk_car,partition_t *partition,const int verbose);
+int recover_JFS(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int verbose, const int dump_ind);
+int set_JFS_info(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int verbose, const int dump_ind);
