@@ -32,7 +32,7 @@
 #include "log.h"
 
 /* #define DEBUG_LIST_APPEND_BLOCK */
-void list_append_block(alloc_list_t *list, const uint64_t offset, const uint64_t blocksize, const int data)
+void list_append_block(alloc_list_t *list, const uint64_t offset, const uint64_t blocksize, const unsigned int data)
 {
   alloc_list_t *prev;
 #ifdef DEBUG_LIST_APPEND_BLOCK
