@@ -32,7 +32,7 @@ struct struct_mediaspace
   unsigned int data;
 };
 
-void list_append_block(alloc_list_t *list, const uint64_t offset, const uint64_t blocksize, const int data);
+void list_append_block(alloc_list_t *list, const uint64_t offset, const uint64_t blocksize, const unsigned int data);
 void list_truncate(alloc_list_t *list, uint64_t size);
 void td_list_delete(alloc_list_t *list);
 
@@ -65,7 +65,7 @@ struct td_list_head {
 #define TD_LIST_HEAD(name) \
 	struct td_list_head name = TD_LIST_HEAD_INIT(name)
 
-#define INIT_TD_LIST_HEAD(ptr) do { \
+#define TD_INIT_LIST_HEAD(ptr) do { \
 	(ptr)->next = (ptr); (ptr)->prev = (ptr); \
 } while (0)
 
@@ -144,7 +144,7 @@ static inline void td_list_del(struct td_list_head *entry)
 static inline void td_list_del_init(struct td_list_head *entry)
 {
 	__td_list_del(entry->prev, entry->next);
-	INIT_TD_LIST_HEAD(entry);
+	TD_INIT_LIST_HEAD(entry);
 }
 
 /**
@@ -234,7 +234,7 @@ static inline void td_list_splice_init(struct td_list_head *list,
 {
 	if (!td_list_empty(list)) {
 		__td_list_splice(list, head);
-		INIT_TD_LIST_HEAD(list);
+		TD_INIT_LIST_HEAD(list);
 	}
 }
 
@@ -242,7 +242,7 @@ static inline void td_list_splice_init(struct td_list_head *list,
  * td_list_entry - get the struct for this entry
  * @ptr:	the &struct td_list_head pointer.
  * @type:	the type of the struct this is embedded in.
- * @member:	the name of the list_struct within the struct.
+ * @member:	the name of the td_list_struct within the struct.
  */
 #define td_list_entry(ptr, type, member) \
 	((type *)((char *)(ptr)-(unsigned long)(&((type *)0)->member)))
@@ -279,7 +279,7 @@ static inline void td_list_splice_init(struct td_list_head *list,
  * td_list_for_each_entry	-	iterate over list of given type
  * @pos:	the type * to use as a loop counter.
  * @head:	the head for your list.
- * @member:	the name of the list_struct within the struct.
+ * @member:	the name of the td_list_struct within the struct.
  */
 #define td_list_for_each_entry(pos, head, member)				\
 	for (pos = td_list_entry((head)->next, typeof(*pos), member);	\
@@ -290,7 +290,7 @@ static inline void td_list_splice_init(struct td_list_head *list,
  * td_list_for_each_entry_reverse - iterate backwards over list of given type.
  * @pos:	the type * to use as a loop counter.
  * @head:	the head for your list.
- * @member:	the name of the list_struct within the struct.
+ * @member:	the name of the td_list_struct within the struct.
  */
 #define td_list_for_each_entry_reverse(pos, head, member)			\
 	for (pos = td_list_entry((head)->prev, typeof(*pos), member);	\
@@ -302,7 +302,7 @@ static inline void td_list_splice_init(struct td_list_head *list,
  *			td_list_for_each_entry_continue
  * @pos:	the type * to use as a start point
  * @head:	the head of the list
- * @member:	the name of the list_struct within the struct.
+ * @member:	the name of the td_list_struct within the struct.
  */
 #define td_list_prepare_entry(pos, head, member) \
 	((pos) ? : td_list_entry(head, typeof(*pos), member))
@@ -312,7 +312,7 @@ static inline void td_list_splice_init(struct td_list_head *list,
  *			continuing after existing point
  * @pos:	the type * to use as a loop counter.
  * @head:	the head for your list.
- * @member:	the name of the list_struct within the struct.
+ * @member:	the name of the td_list_struct within the struct.
  */
 #define td_list_for_each_entry_continue(pos, head, member) 		\
 	for (pos = td_list_entry(pos->member.next, typeof(*pos), member);	\
@@ -324,7 +324,7 @@ static inline void td_list_splice_init(struct td_list_head *list,
  * @pos:	the type * to use as a loop counter.
  * @n:		another type * to use as temporary storage
  * @head:	the head for your list.
- * @member:	the name of the list_struct within the struct.
+ * @member:	the name of the td_list_struct within the struct.
  */
 #define td_list_for_each_entry_safe(pos, n, head, member)			\
 	for (pos = td_list_entry((head)->next, typeof(*pos), member),	\
