@@ -87,9 +87,9 @@ typedef struct {
 } pv_disk_v2_t;
 
 #define pv_disk_t pv_disk_v2_t
-int check_LVM(disk_t *disk_car,partition_t *partition,const int debug);
-int test_LVM(disk_t *disk_car, const pv_disk_t *pv,partition_t *partition,const int debug, const int dump_ind);
-int recover_LVM(disk_t *disk_car, const pv_disk_t *pv,partition_t *partition,const int debug, const int dump_ind);
+int check_LVM(disk_t *disk_car,partition_t *partition,const int verbose);
+int test_LVM(disk_t *disk_car, const pv_disk_t *pv,partition_t *partition,const int verbose, const int dump_ind);
+int recover_LVM(disk_t *disk_car, const pv_disk_t *pv,partition_t *partition,const int verbose, const int dump_ind);
 
 #define LVM2_LABEL	"LVM2 001"
 #define LABEL_ID 	"LABELONE"
@@ -116,8 +116,8 @@ struct lvm2_pv_header {
   struct lvm2_disk_locn disk_areas_xl[0];      /* Two lists */
 } __attribute__ ((packed));
 
-int check_LVM2(disk_t *disk_car,partition_t *partition,const int debug);
-int test_LVM2(disk_t *disk_car, const struct lvm2_label_header *lh,partition_t *partition,const int debug, const int dump_ind);
-int recover_LVM2(disk_t *disk_car, const unsigned char *buf,partition_t *partition,const int debug, const int dump_ind);
+int check_LVM2(disk_t *disk_car,partition_t *partition,const int verbose);
+int test_LVM2(disk_t *disk_car, const struct lvm2_label_header *lh,partition_t *partition,const int verbose, const int dump_ind);
+int recover_LVM2(disk_t *disk_car, const unsigned char *buf,partition_t *partition,const int verbose, const int dump_ind);
 
 #endif /* _LVM_H */
