@@ -32,9 +32,9 @@
 #include "types.h"
 #include "common.h"
 #include "netware.h"
-static int test_netware(disk_t *disk_car, const struct disk_netware *netware_block,partition_t *partition,const int debug, const int dump_ind);
+static int test_netware(disk_t *disk_car, const struct disk_netware *netware_block,partition_t *partition,const int verbose, const int dump_ind);
 
-static int test_netware(disk_t *disk_car, const struct disk_netware *netware_block,partition_t *partition,const int debug, const int dump_ind)
+static int test_netware(disk_t *disk_car, const struct disk_netware *netware_block,partition_t *partition,const int verbose, const int dump_ind)
 {
   if(memcmp(netware_block->magic,"Nw_PaRtItIoN",12)==0)
   {
@@ -44,7 +44,7 @@ static int test_netware(disk_t *disk_car, const struct disk_netware *netware_blo
   return 1;
 }
 
-int check_netware(disk_t *disk_car,partition_t *partition,const int debug)
+int check_netware(disk_t *disk_car,partition_t *partition,const int verbose)
 {
   unsigned char *buffer=MALLOC(DEFAULT_SECTOR_SIZE);
   if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, partition->part_offset)!=0)
@@ -52,7 +52,7 @@ int check_netware(disk_t *disk_car,partition_t *partition,const int debug)
     free(buffer);
     return 1;
   }
-  if(test_netware(disk_car,(const struct disk_netware *)buffer,partition,debug,0)!=0)
+  if(test_netware(disk_car,(const struct disk_netware *)buffer,partition,verbose,0)!=0)
   {
     free(buffer);
     return 1;
@@ -66,8 +66,8 @@ int recover_netware(disk_t *disk_car, const struct disk_netware *netware_block,p
   if(test_netware(disk_car,netware_block,partition,0,0)!=0)
     return 1;
   partition->part_type_i386=P_NETWARE;
-  partition->part_size=(uint64_t)netware_block->nbr_sectors*disk_car->sector_size;
-  partition->name[0]='\0';
+  partition->part_size=(uint64_t)le32(netware_block->nbr_sectors) * disk_car->sector_size;
+  partition->fsname[0]='\0';
   partition->info[0]='\0';
   return 0;
 }
