@@ -49,10 +49,15 @@
 #ifdef HAVE_MACHINE_ENDIAN_H
 #include <machine/endian.h>
 #endif
+#ifdef HAVE_ICONV_H
+#include <iconv.h>
+#endif
+#include <ctype.h>      /* isalpha */
 #include <stdarg.h>
 #include "types.h"
 
 #ifdef HAVE_LIBNTFS
+#include <ntfs/volume.h>
 #include <ntfs/attrib.h>
 #ifdef HAVE_NTFS_VERSION_H
 #include <ntfs/version.h>
@@ -64,7 +69,6 @@
 #include "ntfs.h"
 #include "dir.h"
 #include "ntfs_dir.h"
-#include "io_redir.h"
 #include "ntfs_utl.h"
 #include "ntfs_inc.h"
 #include "log.h"
@@ -137,6 +141,31 @@ static time_t ntfs2utc (s64 ntfstime)
   return (ntfstime - (NTFS_TIME_OFFSET)) / 10000000;
 }
 
+#ifdef HAVE_ICONV
+static int ntfs_ucstoutf8(iconv_t cd, const ntfschar *ins, int ins_len, char **outs, int outs_len)
+{
+    const char *inp;
+    char *outp;
+    size_t inb_left, outb_left;
+    if (cd == (iconv_t)(-1))
+      return -1;
+
+    outp = *outs;
+    inp = (const char *) ins;
+    inb_left = ins_len << 1;    // ntfschar is 16-bit
+    outb_left = outs_len - 1;   // reserve 1 byte for NUL
+
+    if (iconv(cd, &inp, &inb_left, &outp, &outb_left) == (size_t)(-1))
+    {
+      // Regardless of the value of errno
+      log_error("ntfs_ucstoutf8: iconv failed\n");
+      return -1;
+    }
+    *outp = '\0';
+    return 0;
+}
+#endif
+
 /**
  * ntfs_td_list_entry
  * FIXME: Should we print errors as we go along? (AIA)
@@ -154,10 +183,18 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
     return -1;
   }
 
+#ifdef HAVE_ICONV
+  if (ntfs_ucstoutf8(ls->cd, name, name_len, &filename, MAX_PATH) < 0 &&
+      ntfs_ucstombs (name, name_len, &filename, MAX_PATH) < 0) {
+    log_error("Cannot represent filename in current locale.\n");
+    goto free;
+  }
+#else
   if (ntfs_ucstombs (name, name_len, &filename, MAX_PATH) < 0) {
     log_error("Cannot represent filename in current locale.\n");
     goto free;
   }
+#endif
 
   result = 0;					/* These are successful */
   if (filename[0] == '$')			/* system */
@@ -227,9 +264,9 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
       new_file->next=NULL;
       /* log_debug("fat: new file %s de=%p size=%u\n",new_file->name,de,de->size); */
       if(ls->current_file!=NULL)
-	ls->current_file->next=new_file;
+        ls->current_file->next=new_file;
       else
-	ls->dir_list=new_file;
+        ls->dir_list=new_file;
       ls->current_file=new_file;
     }
 
@@ -246,32 +283,6 @@ free:
   return result;
 }
 
-/**
- * ntfs_mount - open ntfs volume
- * @name:	name of device/file to open
- * @rwflag:	optional mount flags
- *
- * This function mounts an ntfs volume. @name should contain the name of the
- * device/file to mount as the ntfs volume.
- *
- * @rwflags is an optional second parameter. The same flags are used as for
- * the mount system call (man 2 mount). Currently only the following flag
- * is implemented:
- *	MS_RDONLY	- mount volume read-only
- *
- * The function opens the device or file @name and verifies that it contains a
- * valid bootsector. Then, it allocates an ntfs_volume structure and initializes
- * some of the values inside the structure from the information stored in the
- * bootsector. It proceeds to load the necessary system files and completes
- * setting up the structure.
- *
- * Return the allocated volume structure on success and NULL on error with
- * errno set to the error code.
- *
- * Note, that a copy is made of @name, and hence it can be discarded as
- * soon as the function returns.
- */
-
 static file_data_t *ntfs_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int cluster)
 {
   ntfs_inode *inode;
@@ -339,9 +350,13 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
       block_size = index_get_size(inode);
     else
       block_size = 0;
-    new_file=MALLOC(strlen(dir_data->local_dir)+strlen(dir_data->current_directory)+1);
-    strcpy(new_file,dir_data->local_dir);
-    strcat(new_file,dir_data->current_directory);
+    {
+      int l1=strlen(dir_data->local_dir);
+      int l2=strlen(dir_data->current_directory);
+      new_file=MALLOC(l1+l2+1);
+      l1=filename_convert(new_file,dir_data->local_dir,l1+1);
+      filename_convert(new_file+l1,dir_data->current_directory,l2+1);
+    }
     f_out=create_file(new_file);
     if(!f_out)
     {
@@ -396,20 +411,20 @@ static void dir_partition_ntfs_close(dir_data_t *dir_data)
   partition=ls->my_data->partition;
   ntfs_umount(ls->vol, FALSE);
   free(ls->my_data);
-  if(partition->boot_sector!=0)
-    io_redir_del_redir(disk_car,partition->part_offset);
+#ifdef HAVE_ICONV
+  if (ls->cd != (iconv_t)(-1))
+    iconv_close(ls->cd);
+#endif
   free(ls);
 }
 #endif
 
-int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int debug)
+int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose)
 {
 #ifdef HAVE_LIBNTFS
   struct ntfs_device *dev;
   my_data_t *my_data=NULL;
   ntfs_volume *vol=NULL;
-  if(partition->boot_sector!=0)
-    io_redir_add_redir(disk_car,partition->part_offset,DEFAULT_SECTOR_SIZE,partition->part_offset+(uint64_t)partition->boot_sector*disk_car->sector_size,NULL);
 #ifdef NTFS_LOG_LEVEL_VERBOSE
   ntfs_log_set_levels(NTFS_LOG_LEVEL_VERBOSE);
   ntfs_log_set_handler(ntfs_log_handler_stderr);
@@ -424,10 +439,18 @@ int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_
     my_data->offset=0;
     dev->d_private=my_data;
     /* Call ntfs_device_mount() to do the actual mount. */
+#ifdef MS_RDONLY
     vol = ntfs_device_mount(dev, MS_RDONLY);
+#else
+    vol = ntfs_device_mount(dev, NTFS_MNT_RDONLY);
+#endif
 #ifdef HAVE_NTFS_VOLUME_STARTUP
     if(!vol) {
+#ifdef MS_RDONLY
       vol = ntfs_volume_startup(dev, MS_RDONLY);
+#else
+      vol = ntfs_volume_startup(dev, NTFS_MNT_RDONLY);
+#endif
       if(vol)
       {
 	log_warning("NTFS filesystem need to be repaired.\n");
@@ -436,8 +459,6 @@ int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_
 #endif
   }
   if (!vol) {
-    if(partition->boot_sector!=0)
-      io_redir_del_redir(disk_car,partition->part_offset);
     free(my_data);
     ntfs_device_free(dev);
     return -1;
@@ -451,9 +472,15 @@ int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_
     ls->current_file=NULL;
     ls->vol=vol;
     ls->my_data=my_data;
+#ifdef HAVE_ICONV
+    if ((ls->cd = iconv_open("UTF-8", "UTF-16LE")) == (iconv_t)(-1))
+    {
+      log_error("ntfs_ucstoutf8: iconv_open failed\n");
+    }
+#endif
     strncpy(dir_data->current_directory,"/",sizeof(dir_data->current_directory));
     dir_data->current_inode=FILE_root;
-    dir_data->debug=debug;
+    dir_data->verbose=verbose;
     dir_data->get_dir=ntfs_dir;
     dir_data->copy_file=ntfs_copy;
     dir_data->close=&dir_partition_ntfs_close;
