@@ -3,17 +3,17 @@
     File: photorec.c
 
     Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
-  
+
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
-  
+
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
-  
+
     You should have received a copy of the GNU General Public License along
     with this program; if not, write the Free Software Foundation, Inc., 51
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
@@ -53,12 +53,19 @@
 #include <sys/wait.h>
 #endif
 #include <errno.h>
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "intrf.h"
 #include "godmode.h"
 #include "fnctdsk.h"
+#ifdef HAVE_NCURSES
 #include "intrfn.h"
+#else
+#include <stdio.h>
+#endif
 #include "dir.h"
 #include "filegen.h"
 #include "photorec.h"
@@ -66,9 +73,12 @@
 #include "hdcache.h"
 #include "fatp.h"
 #include "ntfsp.h"
+#include "ext2p.h"
 #include "ewf.h"
 #include "log.h"
 #include "phrecn.h"
+#include "hdaccess.h"
+#include "sudo.h"
 
 /* #define DEBUG_FILE_FINISH */
 /* #define DEBUG_UPDATE_SEARCH_SPACE */
@@ -95,22 +105,29 @@ extern const file_hint_t file_hint_bmp;
 extern const file_hint_t file_hint_bz2;
 extern const file_hint_t file_hint_cab;
 extern const file_hint_t file_hint_cam;
+extern const file_hint_t file_hint_cm;
 extern const file_hint_t file_hint_crw;
 extern const file_hint_t file_hint_ctg;
 extern const file_hint_t file_hint_cwk;
 extern const file_hint_t file_hint_dat;
 extern const file_hint_t file_hint_dbf;
+extern const file_hint_t file_hint_dim;
 extern const file_hint_t file_hint_dir;
 extern const file_hint_t file_hint_djv;
 extern const file_hint_t file_hint_doc;
+extern const file_hint_t file_hint_dpx;
 extern const file_hint_t file_hint_dsc;
+extern const file_hint_t file_hint_dss;
 extern const file_hint_t file_hint_dta;
 extern const file_hint_t file_hint_dump;
 extern const file_hint_t file_hint_dv;
+extern const file_hint_t file_hint_dwg;
 extern const file_hint_t file_hint_elf;
+extern const file_hint_t file_hint_emf;
 extern const file_hint_t file_hint_evt;
 extern const file_hint_t file_hint_exe;
 extern const file_hint_t file_hint_ext2_sb;
+extern const file_hint_t file_hint_fcp;
 extern const file_hint_t file_hint_fh10;
 extern const file_hint_t file_hint_fh5;
 extern const file_hint_t file_hint_flac;
@@ -122,7 +139,10 @@ extern const file_hint_t file_hint_imb;
 extern const file_hint_t file_hint_indd;
 extern const file_hint_t file_hint_itunes;
 extern const file_hint_t file_hint_jpg;
+extern const file_hint_t file_hint_kdb;
+extern const file_hint_t file_hint_lnk;
 extern const file_hint_t file_hint_max;
+extern const file_hint_t file_hint_mb;
 extern const file_hint_t file_hint_mdb;
 extern const file_hint_t file_hint_mdf;
 extern const file_hint_t file_hint_mid;
@@ -135,8 +155,10 @@ extern const file_hint_t file_hint_mus;
 extern const file_hint_t file_hint_mysql;
 extern const file_hint_t file_hint_njx;
 extern const file_hint_t file_hint_ogg;
+extern const file_hint_t file_hint_one;
 extern const file_hint_t file_hint_orf;
 extern const file_hint_t file_hint_pap;
+extern const file_hint_t file_hint_pct;
 extern const file_hint_t file_hint_pcx;
 extern const file_hint_t file_hint_pdf;
 extern const file_hint_t file_hint_png;
@@ -144,6 +166,7 @@ extern const file_hint_t file_hint_prc;
 extern const file_hint_t file_hint_ps;
 extern const file_hint_t file_hint_psd;
 extern const file_hint_t file_hint_pst;
+extern const file_hint_t file_hint_ptb;
 extern const file_hint_t file_hint_qbb;
 extern const file_hint_t file_hint_qdf;
 extern const file_hint_t file_hint_qxd;
@@ -153,29 +176,31 @@ extern const file_hint_t file_hint_rar;
 extern const file_hint_t file_hint_raw;
 extern const file_hint_t file_hint_rdc;
 extern const file_hint_t file_hint_reg;
+extern const file_hint_t file_hint_res;
 extern const file_hint_t file_hint_riff;
 extern const file_hint_t file_hint_rm;
 extern const file_hint_t file_hint_rns;
 extern const file_hint_t file_hint_rpm;
 extern const file_hint_t file_hint_sit;
+extern const file_hint_t file_hint_skp;
+extern const file_hint_t file_hint_spe;
 extern const file_hint_t file_hint_spss;
 extern const file_hint_t file_hint_stuffit;
 extern const file_hint_t file_hint_swf;
 extern const file_hint_t file_hint_tar;
+extern const file_hint_t file_hint_tib;
 extern const file_hint_t file_hint_tiff;
 extern const file_hint_t file_hint_fasttxt;
 extern const file_hint_t file_hint_txt;
+extern const file_hint_t file_hint_vmdk;
+extern const file_hint_t file_hint_wpd;
 extern const file_hint_t file_hint_wmf;
 extern const file_hint_t file_hint_x3f;
 extern const file_hint_t file_hint_xcf;
 extern const file_hint_t file_hint_xm;
+extern const file_hint_t file_hint_xsv;
 extern const file_hint_t file_hint_zip;
 
-#ifdef DEAD_CODE
-static void get_prev_sector(alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset, const unsigned int blocksize);
-static void get_prev_header(alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset);
-#endif
-
 static alloc_data_t *update_search_space(const file_recovery_t *file_recovery, alloc_data_t *list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize);
 static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, uint64_t start, uint64_t end, alloc_data_t **new_current_search_space, uint64_t *offset);
 
@@ -255,28 +280,22 @@ static alloc_data_t *update_search_space(const file_recovery_t *file_recovery, a
   struct td_list_head *search_walker = NULL;
 #ifdef DEBUG_UPDATE_SEARCH_SPACE
   log_trace("update_search_space\n");
-  td_list_for_each(search_walker, &list_search_space->list)
-  {
-    alloc_data_t *cur_free_space;
-    cur_free_space=td_list_entry(search_walker, alloc_data_t, list);
-    log_trace(" %llu-%llu",(long long unsigned)(cur_free_space->start/512),
-	(long long unsigned)(cur_free_space->end/512));
-  }
-  log_trace("\n");
+  info_list_search_space(list_search_space, NULL, DEFAULT_SECTOR_SIZE, 0, 1);
 #endif
+
   td_list_for_each(search_walker, &list_search_space->list)
   {
     alloc_data_t *current_search_space;
     current_search_space=td_list_entry(search_walker, alloc_data_t, list);
     if(current_search_space->start <= file_recovery->location.start &&
-	file_recovery->location.start <= current_search_space->end)
+        file_recovery->location.start <= current_search_space->end)
     {
       *offset=file_recovery->location.start;
       *new_current_search_space=current_search_space;
       for(element=&file_recovery->location;element!=NULL;element=element->next)
       {
-	uint64_t end=(element->end-(element->start%blocksize)+blocksize-1+1)/blocksize*blocksize+(element->start%blocksize)-1;
-	list_search_space=update_search_space_aux(list_search_space, element->start, end, new_current_search_space, offset);
+        uint64_t end=(element->end-(element->start%blocksize)+blocksize-1+1)/blocksize*blocksize+(element->start%blocksize)-1;
+        list_search_space=update_search_space_aux(list_search_space, element->start, end, new_current_search_space, offset);
       }
       return list_search_space;
     }
@@ -293,10 +312,10 @@ static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, ui
 {
   struct td_list_head *search_walker = NULL;
 #ifdef DEBUG_UPDATE_SEARCH_SPACE
-    log_trace("update_search_space_aux offset=%llu remove [%llu-%llu]\n",
-	(long long unsigned)((*offset)/512),
-	(unsigned long long)(start/512),
-	(unsigned long long)(end/512));
+  log_trace("update_search_space_aux offset=%llu remove [%llu-%llu]\n",
+      (long long unsigned)((*offset)/512),
+      (unsigned long long)(start/512),
+      (unsigned long long)(end/512));
 #endif
   if(start >= end)
     return list_search_space;
@@ -306,34 +325,35 @@ static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, ui
     current_search_space=td_list_entry(search_walker, alloc_data_t, list);
 #ifdef DEBUG_UPDATE_SEARCH_SPACE
     log_trace("update_search_space_aux offset=%llu remove [%llu-%llu] in [%llu-%llu]\n",
-	(long long unsigned)((*offset)/512),
-	(unsigned long long)(start/512),
-	(unsigned long long)(end/512),
-	(unsigned long long)(current_search_space->start/512),
-	(unsigned long long)(current_search_space->end/512));
+        (long long unsigned)((*offset)/512),
+        (unsigned long long)(start/512),
+        (unsigned long long)(end/512),
+        (unsigned long long)(current_search_space->start/512),
+        (unsigned long long)(current_search_space->end/512));
 #endif
     if(current_search_space->start==start)
     {
       if(end<current_search_space->end)
       { /* current_search_space->start==start end<current_search_space->end */
-	if(offset!=NULL && new_current_search_space!=NULL &&
-	    current_search_space->start<=*offset && *offset<=end)
-	{
-	  *new_current_search_space=current_search_space;
-	  *offset=end+1;
-	}
-	current_search_space->start=end+1;
-	return list_search_space;
+        if(offset!=NULL && new_current_search_space!=NULL &&
+            current_search_space->start<=*offset && *offset<=end)
+        {
+          *new_current_search_space=current_search_space;
+          *offset=end+1;
+        }
+        current_search_space->start=end+1;
+        current_search_space->file_stat=NULL;
+        return list_search_space;
       }
       /* current_search_space->start==start current_search_space->end<=end */
       start=current_search_space->end+1;
       if(list_search_space==current_search_space)
-	list_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
+        list_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
       if(offset!=NULL && new_current_search_space!=NULL &&
-	  current_search_space->start<=*offset && *offset<=current_search_space->end)
+          current_search_space->start<=*offset && *offset<=current_search_space->end)
       {
-	*new_current_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
-	*offset=(*new_current_search_space)->start;
+        *new_current_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
+        *offset=(*new_current_search_space)->start;
       }
       td_list_del(search_walker);
       free(current_search_space);
@@ -346,24 +366,24 @@ static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, ui
 #endif
       if(current_search_space->start<start)
       { /* current_search_space->start<start current_search_space->end==end */
-	if(offset!=NULL && new_current_search_space!=NULL &&
-	  start<=*offset && *offset<=current_search_space->end)
-	{
-	  *new_current_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
-	  *offset=(*new_current_search_space)->start;
-	}
-	current_search_space->end=start-1;
-	return list_search_space;
+        if(offset!=NULL && new_current_search_space!=NULL &&
+            start<=*offset && *offset<=current_search_space->end)
+        {
+          *new_current_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
+          *offset=(*new_current_search_space)->start;
+        }
+        current_search_space->end=start-1;
+        return list_search_space;
       }
       /* start<=current_search_space->start current_search_space->end==end */
       end=current_search_space->start-1;
       if(list_search_space==current_search_space)
-	list_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
+        list_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
       if(offset!=NULL && new_current_search_space!=NULL &&
-	  current_search_space->start<=*offset && *offset<=current_search_space->end)
+          current_search_space->start<=*offset && *offset<=current_search_space->end)
       {
-	*new_current_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
-	*offset=(*new_current_search_space)->start;
+        *new_current_search_space=td_list_entry(current_search_space->list.next, alloc_data_t, list);
+        *offset=(*new_current_search_space)->start;
       }
       td_list_del(search_walker);
       free(current_search_space);
@@ -372,7 +392,7 @@ static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, ui
     if(start < current_search_space->start && current_search_space->start <= end)
     {
       list_search_space=update_search_space_aux(list_search_space,
-	  start, current_search_space->start-1,  new_current_search_space, offset);
+          start, current_search_space->start-1,  new_current_search_space, offset);
       return update_search_space_aux(list_search_space, current_search_space->start, end, new_current_search_space, offset);
     }
     if(start <= current_search_space->end && current_search_space->end < end)
@@ -390,9 +410,9 @@ static alloc_data_t *update_search_space_aux(alloc_data_t *list_search_space, ui
       current_search_space->end=start-1;
       td_list_add(&new_free_space->list,search_walker);
       if(offset!=NULL && new_current_search_space!=NULL &&
-	  new_free_space->start<=*offset && *offset<=new_free_space->end)
+          new_free_space->start<=*offset && *offset<=new_free_space->end)
       {
-	*new_current_search_space=new_free_space;
+        *new_current_search_space=new_free_space;
       }
       return update_search_space_aux(list_search_space, start, end, new_current_search_space, offset);
     }
@@ -464,10 +484,10 @@ unsigned int photorec_mkdir(const char *recup_dir, const unsigned int initial_di
 #ifdef __MINGW32__
     if(mkdir(working_recup_dir)!=0 && errno==EEXIST)
 #else
-      if(mkdir(working_recup_dir, 0700)!=0 && errno==EEXIST)
+      if(mkdir(working_recup_dir, 0775)!=0 && errno==EEXIST)
 #endif
 #else
-#error You need a mkdir function!
+#warn You need a mkdir function!
 #endif
       {
 	dir_num++;
@@ -489,36 +509,6 @@ unsigned int photorec_mkdir(const char *recup_dir, const unsigned int initial_di
   return dir_num;
 }
 
-#ifdef DEAD_CODE
-static void get_prev_sector(alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset, const unsigned int blocksize)
-{
-  /*
-  log_trace("get_prev_sector %lu (%lu-%lu)\n", (long unsigned int)((*offset)/blocksize),
-      (unsigned long int)((*current_search_space)->start/blocksize),
-      (unsigned long int)((*current_search_space)->end/blocksize));
-  */
-  if((*current_search_space) == list_search_space)
-    return ;
-  if(! ((*current_search_space)->start <= *offset && (*offset)<=(*current_search_space)->end))
-  {
-    log_critical("BUG: get_prev_sector stop everything\n");
-    log_close();
-    exit(1);
-  }
-  if((*offset)-blocksize >= (*current_search_space)->start)
-    *offset-=blocksize;
-  else
-    get_prev_header(list_search_space, current_search_space, offset);
-}
-
-static void get_prev_header(alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset)
-{
-  if((*current_search_space) != list_search_space)
-    *current_search_space=td_list_entry((*current_search_space)->list.prev, alloc_data_t, list);
-  *offset=(*current_search_space)->start;
-}
-#endif
-
 int get_prev_file_header(alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset)
 {
   int nbr;
@@ -583,6 +573,10 @@ unsigned int remove_used_space(disk_t *disk_car, const partition_t *partition, a
 #ifdef HAVE_LIBNTFS
   else if(partition->upart_type==UP_NTFS)
     return ntfs_remove_used_space(disk_car, partition, list_search_space);
+#endif
+#ifdef HAVE_LIBEXT2FS
+  else if(partition->upart_type==UP_EXT2 || partition->upart_type==UP_EXT3)
+    return ext2_remove_used_space(disk_car, partition, list_search_space);
 #endif
   return 0;
 }
@@ -625,9 +619,9 @@ void write_stats_log(const file_stat_t *file_stats)
     {
       file_nbr+=new_file_stats[i].recovered;
       log_info("%s: %u/%u recovered\n",
-	  (new_file_stats[i].file_hint->extension!=NULL?
-	   new_file_stats[i].file_hint->extension:""),
-	  new_file_stats[i].recovered, new_file_stats[i].recovered+new_file_stats[i].not_recovered);
+          (new_file_stats[i].file_hint->extension!=NULL?
+           new_file_stats[i].file_hint->extension:""),
+          new_file_stats[i].recovered, new_file_stats[i].recovered+new_file_stats[i].not_recovered);
     }
   }
   free(new_file_stats);
@@ -663,9 +657,9 @@ void write_stats_stdout(const file_stat_t *file_stats)
     {
       file_nbr+=file_stats[i].recovered;
       printf("%s: %u/%u recovered\n",
-	  (file_stats[i].file_hint->extension!=NULL?
-	   file_stats[i].file_hint->extension:""),
-	  file_stats[i].recovered, file_stats[i].recovered+file_stats[i].not_recovered);
+          (file_stats[i].file_hint->extension!=NULL?
+           file_stats[i].file_hint->extension:""),
+          file_stats[i].recovered, file_stats[i].recovered+file_stats[i].not_recovered);
     }
   }
   if(file_nbr>1)
@@ -681,10 +675,10 @@ void write_stats_stdout(const file_stat_t *file_stats)
 partition_t *new_whole_disk(const disk_t *disk_car)
 {
   partition_t *fake_partition;
-  fake_partition=partition_new();
+  fake_partition=partition_new(disk_car->arch);
   fake_partition->part_offset=0;
   fake_partition->part_size=disk_car->disk_size;
-  strncpy(fake_partition->name,"Whole disk",sizeof(fake_partition->name)-1);
+  strncpy(fake_partition->fsname,"Whole disk",sizeof(fake_partition->fsname)-1);
   return fake_partition;
 }
 
@@ -698,55 +692,6 @@ struct info_cluster_offset
   unsigned int nbr;
 };
 
-unsigned int find_blocksize_cluster(list_cluster_t *list_cluster, const unsigned int default_blocksize, uint64_t *offset)
-{
-  unsigned int cluster_size;
-  unsigned int cluster_size_best=default_blocksize;
-  unsigned int nbr_max=0;
-  for(cluster_size=default_blocksize;cluster_size<=128*512;cluster_size*=2)
-  {
-    struct td_list_head *dir_walker = NULL;
-    cluster_offset_t cluster_offset[1000];
-    unsigned int nbr_sol=0;
-    td_list_for_each(dir_walker,&list_cluster->list)
-    {
-      list_cluster_t *info;
-      info=td_list_entry(dir_walker, list_cluster_t, list);
-      if(info->cluster>=2 && (info->cluster-2)*cluster_size<info->offset)
-      {
-	unsigned int sol_cur;
-	unsigned int found=0;
-	unsigned int offset_tmp;
-	offset_tmp=info->offset-(info->cluster-2)*cluster_size;
-	for(sol_cur=0;sol_cur<nbr_sol && !found;sol_cur++)
-	{
-	  if(cluster_offset[sol_cur].offset==offset_tmp)
-	  {
-	    cluster_offset[sol_cur].nbr++;
-	    found=1;
-	  }
-	}
-	if(!found && nbr_sol<1000)
-	{
-	  cluster_offset[nbr_sol].offset=offset_tmp;
-	  cluster_offset[nbr_sol].nbr=1;
-	  nbr_sol++;
-	}
-      }
-    }
-    {
-      unsigned int sol_cur;
-      for(sol_cur=0;sol_cur<nbr_sol;sol_cur++)
-	if(nbr_max<cluster_offset[sol_cur].nbr)
-	{
-	  nbr_max=cluster_offset[sol_cur].nbr;
-	  cluster_size_best=cluster_size;
-	  *offset=cluster_offset[sol_cur].offset;
-	}
-    }
-  }
-  return cluster_size_best;
-}
 
 unsigned int find_blocksize(alloc_data_t *list_file, const unsigned int default_blocksize, uint64_t *offset)
 {
@@ -771,7 +716,7 @@ unsigned int find_blocksize(alloc_data_t *list_file, const unsigned int default_
       alloc_data_t *current_file;
       current_file=td_list_entry(search_walker, alloc_data_t, list);
       if(current_file->start%blocksize!=*offset)
-	blocksize_ok=0;
+        blocksize_ok=0;
     }
   }
   blocksize=blocksize<<1;
@@ -791,7 +736,7 @@ alloc_data_t * update_blocksize(unsigned int blocksize, alloc_data_t *list_searc
     {
       td_list_del(search_walker);
       if(list_search_space==current_search_space)
-	list_search_space=td_list_entry(search_walker_next, alloc_data_t, list);
+        list_search_space=td_list_entry(search_walker_next, alloc_data_t, list);
       free(current_search_space);
     }
   }
@@ -801,7 +746,9 @@ alloc_data_t * update_blocksize(unsigned int blocksize, alloc_data_t *list_searc
 int main( int argc, char **argv )
 {
   int i;
-  int help=0, create_log=0, debug=0;
+  int use_sudo=0;
+  int help=0, verbose=0;
+  int create_log=TD_LOG_NONE;
   int run_setlocale=1;
   int testdisk_mode=TESTDISK_O_RDONLY|TESTDISK_O_READAHEAD_32K;
   const char *recup_dir=NULL;
@@ -836,22 +783,29 @@ int main( int argc, char **argv )
     { .enable=1, .file_hint=&file_hint_bz2  },
     { .enable=1, .file_hint=&file_hint_cab  },
     { .enable=1, .file_hint=&file_hint_cam  },
+    { .enable=1, .file_hint=&file_hint_cm  },
     { .enable=1, .file_hint=&file_hint_crw  },
     { .enable=1, .file_hint=&file_hint_ctg  },
     { .enable=1, .file_hint=&file_hint_cwk  },
     { .enable=1, .file_hint=&file_hint_dat  },
     { .enable=0, .file_hint=&file_hint_dbf  },
+    { .enable=1, .file_hint=&file_hint_dim  },
     { .enable=1, .file_hint=&file_hint_dir  },
     { .enable=1, .file_hint=&file_hint_djv  },
     { .enable=1, .file_hint=&file_hint_doc  },
+    { .enable=1, .file_hint=&file_hint_dpx  },
     { .enable=1, .file_hint=&file_hint_dsc  },
+    { .enable=1, .file_hint=&file_hint_dss  },
     { .enable=0, .file_hint=&file_hint_dta  },
     { .enable=1, .file_hint=&file_hint_dump },
     { .enable=1, .file_hint=&file_hint_dv   },
+    { .enable=1, .file_hint=&file_hint_dwg  },
     { .enable=1, .file_hint=&file_hint_elf  },
+    { .enable=1, .file_hint=&file_hint_emf  },
     { .enable=1, .file_hint=&file_hint_evt  },
     { .enable=1, .file_hint=&file_hint_exe  },
     { .enable=1, .file_hint=&file_hint_ext2_sb },
+    { .enable=1, .file_hint=&file_hint_fcp  },
     { .enable=1, .file_hint=&file_hint_fh10  },
     { .enable=1, .file_hint=&file_hint_fh5  },
     { .enable=1, .file_hint=&file_hint_flac },
@@ -862,8 +816,11 @@ int main( int argc, char **argv )
     { .enable=1, .file_hint=&file_hint_imb  },
     { .enable=1, .file_hint=&file_hint_indd  },
     { .enable=1, .file_hint=&file_hint_itunes  },
+    { .enable=1, .file_hint=&file_hint_kdb  },
+    { .enable=1, .file_hint=&file_hint_lnk  },
     { .enable=1, .file_hint=&file_hint_jpg  },
     { .enable=1, .file_hint=&file_hint_max  },
+    { .enable=1, .file_hint=&file_hint_mb   },
     { .enable=1, .file_hint=&file_hint_mdb  },
     { .enable=1, .file_hint=&file_hint_mdf  },
     { .enable=1, .file_hint=&file_hint_mid  },
@@ -876,8 +833,10 @@ int main( int argc, char **argv )
     { .enable=1, .file_hint=&file_hint_mysql },
     { .enable=1, .file_hint=&file_hint_njx  },
     { .enable=1, .file_hint=&file_hint_ogg  },
+    { .enable=1, .file_hint=&file_hint_one  },
     { .enable=1, .file_hint=&file_hint_orf  },
     { .enable=1, .file_hint=&file_hint_pap  },
+    { .enable=1, .file_hint=&file_hint_pct  },
     { .enable=1, .file_hint=&file_hint_pcx  },
     { .enable=1, .file_hint=&file_hint_pdf  },
     { .enable=1, .file_hint=&file_hint_png  },
@@ -885,6 +844,7 @@ int main( int argc, char **argv )
     { .enable=1, .file_hint=&file_hint_ps   },
     { .enable=1, .file_hint=&file_hint_psd  },
     { .enable=1, .file_hint=&file_hint_pst  },
+    { .enable=1, .file_hint=&file_hint_ptb  },
     { .enable=1, .file_hint=&file_hint_qbb  },
     { .enable=1, .file_hint=&file_hint_qdf  },
     { .enable=1, .file_hint=&file_hint_qxd  },
@@ -894,29 +854,34 @@ int main( int argc, char **argv )
     { .enable=1, .file_hint=&file_hint_raw  },
     { .enable=1, .file_hint=&file_hint_rdc  },
     { .enable=1, .file_hint=&file_hint_reg  },
+    { .enable=1, .file_hint=&file_hint_res  },
     { .enable=1, .file_hint=&file_hint_riff },
     { .enable=1, .file_hint=&file_hint_rm   },
     { .enable=1, .file_hint=&file_hint_rns  },
     { .enable=1, .file_hint=&file_hint_rpm  },
     { .enable=1, .file_hint=&file_hint_sit  },
+    { .enable=1, .file_hint=&file_hint_skp  },
+    { .enable=1, .file_hint=&file_hint_spe  },
     { .enable=1, .file_hint=&file_hint_spss },
     { .enable=1, .file_hint=&file_hint_stuffit  },
     { .enable=1, .file_hint=&file_hint_swf  },
     { .enable=1, .file_hint=&file_hint_tar  },
+    { .enable=1, .file_hint=&file_hint_tib  },
     { .enable=1, .file_hint=&file_hint_tiff },
     { .enable=1, .file_hint=&file_hint_fasttxt  },
     { .enable=1, .file_hint=&file_hint_txt  },
+    { .enable=1, .file_hint=&file_hint_vmdk },
+    { .enable=1, .file_hint=&file_hint_wpd  },
+    { .enable=1, .file_hint=&file_hint_wmf  },
     { .enable=1, .file_hint=&file_hint_x3f  },
     { .enable=1, .file_hint=&file_hint_xcf  },
     { .enable=1, .file_hint=&file_hint_xm   },
-    { .enable=1, .file_hint=&file_hint_wmf  },
+    { .enable=1, .file_hint=&file_hint_xsv  },
     { .enable=1, .file_hint=&file_hint_zip  },
     { .enable=0, .file_hint=NULL }
   };
-
-#ifdef TESTING
-  srand(1);
-#endif
+  /* random (weak is ok) is need fot GPT */
+  srand(time(NULL));
 #ifdef HAVE_SIGACTION
   /* set up the signal handler for SIGHUP */
   action.sa_handler  = sighup_hdlr;
@@ -931,24 +896,28 @@ int main( int argc, char **argv )
   for(i=1;i<argc;i++)
   {
     if((strcmp(argv[i],"/log")==0) ||(strcmp(argv[i],"-log")==0))
-      create_log=1;
+    {
+      if(create_log==TD_LOG_NONE)
+        create_log=log_open("photorec.log", TD_LOG_APPEND, 0, "PhotoRec", argc, argv);
+    }
     else if((strcmp(argv[i],"/debug")==0) || (strcmp(argv[i],"-debug")==0))
     {
-      debug++;
-      create_log=1;
+      verbose++;
+      if(create_log==TD_LOG_NONE)
+        create_log=log_open("photorec.log", TD_LOG_APPEND, 0, "PhotoRec", argc, argv);
     }
     else if(((strcmp(argv[i],"/d")==0)||(strcmp(argv[i],"-d")==0)) &&(i+1<argc))
     {
       int len=strlen(argv[i+1]);
       if(argv[i+1][len-1]=='\\' || argv[i+1][len-1]=='/')
       {
-	char *new_recup_dir=MALLOC(len+strlen(DEFAULT_RECUP_DIR)+1);
-	strcpy(new_recup_dir,argv[i+1]);
-	strcat(new_recup_dir,DEFAULT_RECUP_DIR);
-	recup_dir=new_recup_dir;	/* small memory leak */
+        char *new_recup_dir=MALLOC(len+strlen(DEFAULT_RECUP_DIR)+1);
+        strcpy(new_recup_dir,argv[i+1]);
+        strcat(new_recup_dir,DEFAULT_RECUP_DIR);
+        recup_dir=new_recup_dir;	/* small memory leak */
       }
       else
-	recup_dir=argv[i+1];
+        recup_dir=argv[i+1];
       i++;
     }
     else if((strcmp(argv[i],"/all")==0) || (strcmp(argv[i],"-all")==0))
@@ -963,62 +932,49 @@ int main( int argc, char **argv )
     else if(strcmp(argv[i],"/cmd")==0)
     {
       if(i+2>=argc)
-	help=1;
+        help=1;
       else
       {
-	disk_t *disk_car;
-	cmd_device=argv[++i];
-	cmd_run=argv[++i];
-	/* There is no log currently */
-	disk_car=file_test_availability(cmd_device,debug,arch,testdisk_mode);
-	if(disk_car==NULL)
-	{
-	  printf("\nUnable to open file or device %s\n",cmd_device);
-	  help=1;
-	}
-	else
-	  list_disk=insert_new_disk(list_disk,disk_car);
+        disk_t *disk_car;
+        cmd_device=argv[++i];
+        cmd_run=argv[++i];
+        /* There is no log currently */
+        disk_car=file_test_availability(cmd_device,verbose,arch,testdisk_mode);
+        if(disk_car==NULL)
+        {
+          printf("\nUnable to open file or device %s\n",cmd_device);
+          help=1;
+        }
+        else
+          list_disk=insert_new_disk(list_disk,disk_car);
       }
     }
     else
     {
-      disk_t *disk_car=file_test_availability(argv[i],debug,arch,testdisk_mode);
+      disk_t *disk_car=file_test_availability(argv[i],verbose,arch,testdisk_mode);
       if(disk_car==NULL)
       {
-	printf("\nUnable to open file or device %s\n",argv[i]);
-	help=1;
+        printf("\nUnable to open file or device %s\n",argv[i]);
+        help=1;
       }
       else
-	list_disk=insert_new_disk(list_disk,disk_car);
+        list_disk=insert_new_disk(list_disk,disk_car);
     }
   }
   if(help!=0)
   {
     printf("\nUsage: photorec [/log] [/debug] [/d recup_dir] [file or device]\n"\
-	    "\n" \
-	    "/log          : create a photorec.log file\n" \
-	    "/debug        : add debug information\n" \
-	    "\n" \
-	    "PhotoRec searches various file formats (JPEG, Office...), it stores them\n" \
-	    "in recup_dir directory.\n" \
-	    "\n" \
-	    "If you have problems with PhotoRec or bug reports, please contact me.\n");
+        "\n" \
+        "/log          : create a photorec.log file\n" \
+        "/debug        : add debug information\n" \
+        "\n" \
+        "PhotoRec searches various file formats (JPEG, Office...), it stores them\n" \
+        "in recup_dir directory.\n" \
+        "\n" \
+        "If you have problems with PhotoRec or bug reports, please contact me.\n");
     return 0;
   }
-  if(create_log>0)
-    log_open("photorec.log",(create_log==1?"a":"w"),"PhotoRec",argc,argv);
-  log_info("PhotoRec %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
-  log_info(TESTDISK_OS);
-  log_info(" (ewf lib: %s)\n", td_ewf_version());
-#if 0
-#if defined(HAVE_LIBJPEG) && defined(JPEG_LIB_VERSION)
-  log_info(", libjpeg: %u)\n", JPEG_LIB_VERSION);
-#else
-  log_info(", libjpeg: none)\n");
-#endif
-#endif
-  log_info("\n");
-  printf("Please wait...\n");
+  aff_buffer(BUFFER_RESET,"Q");
 #ifdef HAVE_SETLOCALE
   if(run_setlocale>0)
   {
@@ -1032,17 +988,42 @@ int main( int argc, char **argv )
     }
   }
 #endif
-  aff_buffer(BUFFER_RESET,"Q");
-  /* Scan for available device only if no device or image has been supplied in parameter */
-  if(list_disk==NULL)
-    list_disk=hd_parse(list_disk,debug,arch,testdisk_mode);
-#ifdef DJGPP
-  for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
+#ifdef HAVE_NCURSES
+  /* ncurses need locale for correct unicode support */
+  if(start_ncurses("PhotoRec", argv[0]))
+    return 1;
+#endif
+  create_log=log_open("photorec.log", create_log, 1, "PhotoRec", argc, argv);
+  log_info("PhotoRec %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
+  log_info(TESTDISK_OS);
+  log_info(" (ewf lib: %s)\n", td_ewf_version());
+#if 0
+#if defined(HAVE_LIBJPEG) && defined(JPEG_LIB_VERSION)
+  log_info(", libjpeg: %u)\n", JPEG_LIB_VERSION);
+#else
+  log_info(", libjpeg: none)\n");
+#endif
+#endif
+  log_info("\n");
+#if defined(__CYGWIN__) || defined(__MINGW32__) || defined(DJGPP)
+#else
+#ifdef HAVE_GETEUID
+  if(geteuid()!=0)
   {
-    printf("%s\n",element_disk->disk->description(element_disk->disk));
+    log_warning("User is not root!\n");
   }
 #endif
-  hd_update_all_geometry(list_disk,0,debug);
+#endif
+#ifdef HAVE_NCURSES
+  aff_copy(stdscr);
+  wmove(stdscr,5,0);
+  wprintw(stdscr, "Please wait...\n");
+  wrefresh(stdscr);
+#endif
+  /* Scan for available device only if no device or image has been supplied in parameter */
+  if(list_disk==NULL)
+    list_disk=hd_parse(list_disk,verbose,arch,testdisk_mode);
+  hd_update_all_geometry(list_disk,0,verbose);
   /* Activate the cache, even if photorec has its own */
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
     element_disk->disk=new_diskcache(element_disk->disk,testdisk_mode);
@@ -1050,16 +1031,19 @@ int main( int argc, char **argv )
   log_info("Hard disk list\n");
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
   {
-    printf("%s, sector size=%u\n",element_disk->disk->description(element_disk->disk),element_disk->disk->sector_size);
-    log_info("%s, sector size=%u\n",element_disk->disk->description(element_disk->disk),element_disk->disk->sector_size);
+    disk_t *disk=element_disk->disk;
+    if(disk->model==NULL)
+      log_info("%s, sector size=%u\n",
+	  disk->description(disk), disk->sector_size);
+    else
+      log_info("%s, sector size=%u - %s\n",
+	  disk->description(disk), disk->sector_size, disk->model);
   }
-  printf("\n");
   log_info("\n");
-  if(start_ncurses("PhotoRec", argv[0])==0)
-  {
-    do_curses_photorec(debug,recup_dir,list_disk,list_file_enable,cmd_device,cmd_run);
-    end_ncurses();
-  }
+  use_sudo=do_curses_photorec(verbose, recup_dir, list_disk, list_file_enable, cmd_device, &cmd_run);
+#ifdef HAVE_NCURSES
+  end_ncurses();
+#endif
   delete_list_disk(list_disk);
   log_info("PhotoRec exited normally.\n");
   if(log_close()!=0)
@@ -1070,6 +1054,10 @@ int main( int argc, char **argv )
   {
     printf("PhotoRec exited normally.\n");
   }
+#ifdef SUDO_BIN
+  if(use_sudo>0)
+    run_sudo(argc, argv);
+#endif
   return 0;
 }
 
@@ -1098,9 +1086,9 @@ void file_search_footer(file_recovery_t *file_recovery, const unsigned char*foot
     {
       if(buffer[i]==footer[0] && memcmp(buffer+i,footer,footer_length)==0)
       {
-	file_recovery->file_size=file_size+i+footer_length;
-	free(buffer);
-	return;
+        file_recovery->file_size=file_size+i+footer_length;
+        free(buffer);
+        return;
       }
     }
     memcpy(buffer+read_size,buffer,footer_length-1);
@@ -1136,9 +1124,9 @@ void file_search_lc_footer(file_recovery_t *file_recovery, const unsigned char*f
     {
       if(buffer[i]==footer[0] && memcmp(buffer+i,footer,footer_length)==0)
       {
-	file_recovery->file_size=file_size+i+footer_length;
-	free(buffer);
-	return;
+        file_recovery->file_size=file_size+i+footer_length;
+        free(buffer);
+        return;
       }
     }
     memcpy(buffer+read_size,buffer,footer_length-1);
@@ -1168,7 +1156,7 @@ void file_check_size(file_recovery_t *file_recovery)
 /* file_finish() returns
    -1: file not recovered, file_size=0 offset_error!=0
     0: file not recovered
-    1: file recovered	   
+    1: file recovered
  */
 int file_finish(file_recovery_t *file_recovery, const char *recup_dir, const int paranoid, unsigned int *file_nbr,
     const unsigned int blocksize, alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset,
@@ -1181,6 +1169,7 @@ int file_finish(file_recovery_t *file_recovery, const char *recup_dir, const int
       (unsigned long int)((*current_search_space)->end/blocksize));
   log_debug("file_recovery->offset_error=%llu\n", (long long unsigned)file_recovery->offset_error);
   log_debug("file_recovery->handle %s NULL\n", (file_recovery->handle!=NULL?"!=":"=="));
+  info_list_search_space(list_search_space, NULL, DEFAULT_SECTOR_SIZE, 0, 1);
 #endif
   if(file_recovery->handle)
   {
@@ -1188,41 +1177,41 @@ int file_finish(file_recovery_t *file_recovery, const char *recup_dir, const int
     {
       if(file_recovery->file_stat!=NULL && file_recovery->file_check!=NULL && paranoid>0)
       { /* Check if recovered file is valid */
-	file_recovery->file_check(file_recovery);
+        file_recovery->file_check(file_recovery);
       }
       if(file_recovery->file_stat!=NULL && file_recovery->file_size> 0 &&
-	  file_recovery->file_size < file_recovery->min_filesize)
+          file_recovery->file_size < file_recovery->min_filesize)
       { 
-	log_info("File too small ( %llu < %llu), reject it\n",
-	    (long long unsigned) file_recovery->file_size,
-	    (long long unsigned) file_recovery->min_filesize);
-	file_recovery->file_size=0;
-	file_recovery->file_size_on_disk=0;
+        log_info("File too small ( %llu < %llu), reject it\n",
+            (long long unsigned) file_recovery->file_size,
+            (long long unsigned) file_recovery->min_filesize);
+        file_recovery->file_size=0;
+        file_recovery->file_size_on_disk=0;
       }
       /* FIXME: need to adapt read_size to volume size to avoid this */
       if(file_recovery->file_size > disk_car->disk_size)
-	file_recovery->file_size = disk_car->disk_size;
+        file_recovery->file_size = disk_car->disk_size;
       if(file_recovery->file_size > disk_car->disk_real_size)
-	file_recovery->file_size = disk_car->disk_real_size;
+        file_recovery->file_size = disk_car->disk_real_size;
 #ifdef HAVE_FTRUNCATE
       fflush(file_recovery->handle);
       if(ftruncate(fileno(file_recovery->handle), file_recovery->file_size)<0)
       {
-	log_critical("ftruncate failed.\n");
+        log_critical("ftruncate failed.\n");
       }
 #endif
     }
     fclose(file_recovery->handle);
     file_recovery->handle=NULL;
-//    log_debug("%s %llu\n",file_recovery->filename,(long long unsigned)file_recovery->file_size);
+    //    log_debug("%s %llu\n",file_recovery->filename,(long long unsigned)file_recovery->file_size);
     if(file_recovery->file_size>0)
     {
       if((++(*file_nbr))%MAX_FILES_PER_DIR==0)
       {
-	*dir_num=photorec_mkdir(recup_dir,*dir_num+1);
+        *dir_num=photorec_mkdir(recup_dir,*dir_num+1);
       }
       if(paranoid==0 || (status!=STATUS_EXT2_ON_SAVE_EVERYTHING && status!=STATUS_EXT2_OFF_SAVE_EVERYTHING))
-	file_recovery->file_stat->recovered++;
+        file_recovery->file_stat->recovered++;
     }
     else
     {
@@ -1239,8 +1228,8 @@ int file_finish(file_recovery_t *file_recovery, const char *recup_dir, const int
       /* File hasn't been sucessfully recovered, remember where it begins */
       list_free_add(file_recovery, list_search_space);
       if((*current_search_space)!=list_search_space &&
-	  !((*current_search_space)->start <= *offset && *offset <= (*current_search_space)->end))
-	*current_search_space=td_list_entry((*current_search_space)->list.next, alloc_data_t, list);
+          !((*current_search_space)->start <= *offset && *offset <= (*current_search_space)->end))
+        *current_search_space=td_list_entry((*current_search_space)->list.next, alloc_data_t, list);
     }
     else if(status!=STATUS_EXT2_ON_SAVE_EVERYTHING && status!=STATUS_EXT2_OFF_SAVE_EVERYTHING && status!=STATUS_FIND_OFFSET)
     {
@@ -1258,6 +1247,41 @@ int file_finish(file_recovery_t *file_recovery, const char *recup_dir, const int
   log_debug("file_finish end %lu (%lu-%lu)\n\n", (long unsigned int)((*offset)/blocksize),
       (unsigned long int)((*current_search_space)->start/blocksize),
       (unsigned long int)((*current_search_space)->end/blocksize));
+  info_list_search_space(list_search_space, NULL, DEFAULT_SECTOR_SIZE, 0, 1);
 #endif
   return file_recovered;
 }
+
+void info_list_search_space(const alloc_data_t *list_search_space, const alloc_data_t *current_search_space, const unsigned int sector_size, const int keep_corrupted_file, const int verbose)
+{
+  struct td_list_head *search_walker = NULL;
+  unsigned long int nbr_headers=0;
+  uint64_t sectors_with_unknown_data=0;
+  td_list_for_each(search_walker,&list_search_space->list)
+  {
+    alloc_data_t *tmp;
+    tmp=td_list_entry(search_walker, alloc_data_t, list);
+    if(tmp->file_stat!=NULL)
+    {
+      nbr_headers++;
+      tmp->file_stat->not_recovered++;
+    }
+    sectors_with_unknown_data+=(tmp->end-tmp->start+sector_size-1)/sector_size;
+    if(verbose>0)
+    {
+      if(tmp==current_search_space)
+        log_info("* ");
+      log_info("%lu-%lu: %s\n",(long unsigned)(tmp->start/sector_size),
+          (long unsigned)(tmp->end/sector_size),
+          (tmp->file_stat!=NULL && tmp->file_stat->file_hint!=NULL?
+           (tmp->file_stat->file_hint->extension?
+            tmp->file_stat->file_hint->extension:""):
+           "(null)"));
+    }
+  }
+  log_info("%llu sectors contains unknown data, %lu invalid files found %s.\n",
+      (long long unsigned)sectors_with_unknown_data, (long unsigned)nbr_headers,
+      (keep_corrupted_file>0?"but saved":"and rejected"));
+}
+
+
