@@ -463,15 +463,12 @@ static void dir_partition_reiser_close(dir_data_t *dir_data)
 }
 #endif
 
-int dir_partition_reiser_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int debug)
+int dir_partition_reiser_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose)
 {
 #ifdef HAVE_LIBREISERFS
   dal_t *dal;
   reiserfs_fs_t *fs;
-  reiserfs_path_node_t *leaf;
   my_data_t *my_data;
-  int32_t offset = 0;
-  int error = 0;
   my_data=MALLOC(sizeof(*my_data));
   my_data->partition=partition;
   my_data->disk_car=disk_car;
@@ -510,7 +507,7 @@ int dir_partition_reiser_init(disk_t *disk_car, const partition_t *partition, di
     ls->flags = 0; /*DIRENT_FLAG_INCLUDE_EMPTY; */
     strncpy(dir_data->current_directory,"/",sizeof(dir_data->current_directory));
     dir_data->current_inode=2;
-    dir_data->debug=debug;
+    dir_data->verbose=verbose;
     dir_data->get_dir=reiser_dir;
     dir_data->copy_file=NULL;
     dir_data->close=&dir_partition_reiser_close;
