@@ -29,7 +29,7 @@ struct backup_disk
   list_part_t *list_part;
 };
 
-int save_header(disk_t *disk_car,partition_t *partition, const int debug);
-int partition_save(disk_t *disk_car, list_part_t *list_part, const int debug);
-backup_disk_t *partition_load(const disk_t *disk_car, const int debug);
+int save_header(disk_t *disk_car,partition_t *partition, const int verbose);
+int partition_save(disk_t *disk_car, list_part_t *list_part, const int verbose);
+backup_disk_t *partition_load(const disk_t *disk_car, const int verbose);
 
