@@ -23,6 +23,9 @@
 #include <config.h>
 #endif
 #include <stdio.h>
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>	/* unlink */
 #endif
@@ -49,46 +52,35 @@
 #define SESSION_MAXSIZE 40960
 #define SESSION_FILENAME "photorec.ses"
 
-static int session_create_empty(void);
-
-static int session_create_empty()
-{
-  int res;
-  FILE *f_session;
-  char *buffer;
-  f_session=fopen(SESSION_FILENAME,"wb");
-  if(!f_session)
-  {
-    log_critical("Can't create empty photorec.ses file: %s\n",strerror(errno));
-    return -1;
-  }
-  buffer=MALLOC(SESSION_MAXSIZE);
-  memset(buffer,0,SESSION_MAXSIZE);
-  res=fwrite(buffer,SESSION_MAXSIZE,1,f_session);
-  free(buffer);
-  fclose(f_session);
-  return (res<=0?-1:0);
-}
-
 int session_load(char **cmd_device, char **current_cmd, alloc_data_t *list_free_space)
 {
   FILE *f_session;
   char *buffer;
   char *pos;
   int taille;
+  struct stat stat_rec;
+  unsigned int buffer_size;
   f_session=fopen(SESSION_FILENAME,"rb");
   if(!f_session)
   {
-    log_info("Can't read photorec.ses file: %s\n",strerror(errno));
-    session_create_empty();
+    log_info("Can't open photorec.ses file: %s\n",strerror(errno));
+    session_save(NULL, NULL, NULL, NULL, 0, 0);
     return -1;
   }
-  buffer=MALLOC(SESSION_MAXSIZE);
-  taille=fread(buffer,1,SESSION_MAXSIZE,f_session);
+  if(fstat(fileno(f_session), &stat_rec)<0)
+    buffer_size=SESSION_MAXSIZE;
+  else
+    buffer_size=stat_rec.st_size;
+  buffer=MALLOC(buffer_size+1);
+  taille=fread(buffer,1,buffer_size,f_session);
+  buffer[taille]='\0';
   fclose(f_session);
-  buffer[(taille<SESSION_MAXSIZE?taille:SESSION_MAXSIZE-1)]='\0';
   pos=buffer;
-  if(*pos=='#')
+  if(*pos!='#')
+  {
+    free(buffer);
+    return -1;
+  }
   {
     time_t my_time;
     char *info=NULL;
@@ -134,37 +126,35 @@ int session_load(char **cmd_device, char **current_cmd, alloc_data_t *list_free_
       long unsigned start,end;
       if(sscanf(pos,"%lu-%lu\n",&start,&end)==2)
       {
-	if(start<=end)
-	{
-	  alloc_data_t *new_free_space;
-	  new_free_space=(alloc_data_t*)MALLOC(sizeof(*new_free_space));
-	  /* Temporary storage, values need to be multiply by sector_size */
-	  new_free_space->start=start;
-	  new_free_space->end=end;
-	  new_free_space->file_stat=NULL;
-	  td_list_add_tail(&new_free_space->list, &list_free_space->list);
+        if(start<=end)
+        {
+          alloc_data_t *new_free_space;
+          new_free_space=(alloc_data_t*)MALLOC(sizeof(*new_free_space));
+          /* Temporary storage, values need to be multiply by sector_size */
+          new_free_space->start=start;
+          new_free_space->end=end;
+          new_free_space->file_stat=NULL;
+          td_list_add_tail(&new_free_space->list, &list_free_space->list);
 #ifdef DEBUG
-	  log_trace(">%lu-%lu<\n",start,end);
+          log_trace(">%lu-%lu<\n",start,end);
 #endif
-	}
-	pos=strstr(pos,"\n");
-	if(pos!=NULL)
-	  pos++;
+        }
+        pos=strstr(pos,"\n");
+        if(pos!=NULL)
+          pos++;
       }
       else
-	pos=NULL;
+        pos=NULL;
     } while(pos!=NULL);
   }
   free(buffer);
   return 0;
 }
 
-int session_save(alloc_data_t *list_free_space, disk_t *disk_car, const partition_t *partition, const file_enable_t *files_enable, const unsigned int blocksize, const int debug)
+int session_save(alloc_data_t *list_free_space, disk_t *disk_car, const partition_t *partition, const file_enable_t *files_enable, const unsigned int blocksize, const int verbose)
 {
-  struct td_list_head *free_walker = NULL;
   FILE *f_session;
-  unsigned int i;
-  if(debug>1)
+  if(verbose>1)
   {
     log_trace("session_save\n");
   }
@@ -174,21 +164,39 @@ int session_save(alloc_data_t *list_free_space, disk_t *disk_car, const partitio
     log_critical("Can't create photorec.ses file: %s\n",strerror(errno));
     return -1;
   }
-  fprintf(f_session,"#%u\n%s %s,%u,blocksize,%u,fileopt,",(unsigned int)time(NULL), disk_car->device, disk_car->arch->part_name_option, partition->order,blocksize);
-  for(i=0;files_enable[i].file_hint!=NULL;i++)
+  if(disk_car!=NULL)
   {
-    if(files_enable[i].file_hint->extension!=NULL && files_enable[i].file_hint->extension[0]!='\0')
+    struct td_list_head *free_walker = NULL;
+    unsigned int i;
+    fprintf(f_session,"#%u\n%s %s,%u,blocksize,%u,fileopt,",(unsigned int)time(NULL), disk_car->device, disk_car->arch->part_name_option, partition->order,blocksize);
+    for(i=0;files_enable[i].file_hint!=NULL;i++)
     {
-      fprintf(f_session,"%s,%s,",files_enable[i].file_hint->extension,(files_enable[i].enable!=0?"enable":"disable"));
+      if(files_enable[i].file_hint->extension!=NULL && files_enable[i].file_hint->extension[0]!='\0')
+      {
+	fprintf(f_session,"%s,%s,",files_enable[i].file_hint->extension,(files_enable[i].enable!=0?"enable":"disable"));
+      }
+    }
+    fprintf(f_session,"search,inter\n");
+    td_list_for_each(free_walker, &list_free_space->list)
+    {
+      alloc_data_t *current_free_space;
+      current_free_space=td_list_entry(free_walker, alloc_data_t, list);
+      fprintf(f_session,"%lu-%lu\n",(long unsigned)(current_free_space->start/disk_car->sector_size),
+	  (long unsigned)(current_free_space->end/disk_car->sector_size));
     }
   }
-  fprintf(f_session,"search,inter\n");
-  td_list_for_each(free_walker, &list_free_space->list)
-  {
-    alloc_data_t *current_free_space;
-    current_free_space=td_list_entry(free_walker, alloc_data_t, list);
-    fprintf(f_session,"%lu-%lu\n",(long unsigned)(current_free_space->start/disk_car->sector_size),
-	(long unsigned)(current_free_space->end/disk_car->sector_size));
+  { /* Reserve some space */
+    int res;
+    char *buffer;
+    buffer=MALLOC(SESSION_MAXSIZE);
+    memset(buffer,0,SESSION_MAXSIZE);
+    res=fwrite(buffer,1,SESSION_MAXSIZE,f_session);
+    free(buffer);
+    if(res<SESSION_MAXSIZE)
+    {
+      fclose(f_session);
+      return -1;
+    }
   }
   fclose(f_session);
   return 0;
