@@ -36,7 +36,7 @@
 #include "fnctdsk.h"
 static int set_Linux_SWAP_info(const union swap_header *swap_header,partition_t *partition);
 
-int check_Linux_SWAP(disk_t *disk_car,partition_t *partition,const int debug)
+int check_Linux_SWAP(disk_t *disk_car,partition_t *partition,const int verbose)
 {
   unsigned char *buffer=(unsigned char*)MALLOC(SWAP_SIZE);
   if(disk_car->read(disk_car,SWAP_SIZE, buffer, partition->part_offset)!=0)
@@ -44,7 +44,7 @@ int check_Linux_SWAP(disk_t *disk_car,partition_t *partition,const int debug)
     free(buffer);
     return 1;
   }
-  if(test_Linux_SWAP(disk_car,(union swap_header*)buffer,partition,debug,0)!=0)
+  if(test_Linux_SWAP(disk_car,(union swap_header*)buffer,partition,verbose,0)!=0)
   {
     free(buffer);
     return 1;
@@ -56,7 +56,7 @@ int check_Linux_SWAP(disk_t *disk_car,partition_t *partition,const int debug)
 
 static int set_Linux_SWAP_info(const union swap_header *swap_header,partition_t *partition)
 {
-  partition->name[0]='\0';
+  partition->fsname[0]='\0';
   switch(partition->upart_type)
   {
     case UP_LINSWAP:
@@ -73,7 +73,7 @@ static int set_Linux_SWAP_info(const union swap_header *swap_header,partition_t
   return 0;
 }
 
-int test_Linux_SWAP(disk_t *disk_car, const union swap_header *swap_header,partition_t *partition,const int debug, const int dump_ind)
+int test_Linux_SWAP(disk_t *disk_car, const union swap_header *swap_header,partition_t *partition,const int verbose, const int dump_ind)
 {
   if(memcmp(swap_header->magic.magic,"SWAP-SPACE",10)==0)
   {
@@ -90,14 +90,15 @@ int test_Linux_SWAP(disk_t *disk_car, const union swap_header *swap_header,parti
   return 1;
 }
 
-int recover_Linux_SWAP(disk_t *disk_car, const union swap_header *swap_header,partition_t *partition,const int debug, const int dump_ind)
+int recover_Linux_SWAP(disk_t *disk_car, const union swap_header *swap_header,partition_t *partition,const int verbose, const int dump_ind)
 {
-  if(test_Linux_SWAP(disk_car,swap_header,partition,debug,dump_ind)!=0)
+  if(test_Linux_SWAP(disk_car,swap_header,partition,verbose,dump_ind)!=0)
     return 1;
   set_Linux_SWAP_info(swap_header,partition);
   partition->part_type_i386=P_LINSWAP;
   partition->part_type_sun=PSUN_LINSWAP;
   partition->part_type_mac=PMAC_SWAP;
+  partition->part_type_gpt=GPT_ENT_TYPE_LINUX_SWAP;
   switch(partition->upart_type)
   {
     case UP_LINSWAP:
