@@ -41,6 +41,6 @@ union swap_header {
   } info;
 };
 
-int check_Linux_SWAP(t_param_disk *disk_car,t_partition *partition,const int debug);
-int test_Linux_SWAP(t_param_disk *disk_car, const union swap_header *swap_header,t_partition *partition,const int debug, const int dump_ind);
-int recover_Linux_SWAP(t_param_disk *disk_car, const union swap_header *swap_header,t_partition *partition,const int debug, const int dump_ind);
+int check_Linux_SWAP(disk_t *disk_car,partition_t *partition,const int verbose);
+int test_Linux_SWAP(disk_t *disk_car, const union swap_header *swap_header,partition_t *partition,const int verbose, const int dump_ind);
+int recover_Linux_SWAP(disk_t *disk_car, const union swap_header *swap_header,partition_t *partition,const int verbose, const int dump_ind);
