@@ -300,9 +300,15 @@ struct ufs2_csum_total {
 	uint64_t   cs_spare[3];	/* future expansion */
 } __attribute__ ((__packed__));
 
-/*
- * This is the actual superblock, as it is laid out on the disk.
- */
+struct fs_u11_u1_st {
+  int8_t	fs_fsmnt[UFS_MAXMNTLEN];/* name mounted on */
+  uint32_t	fs_cgrotor;	/* last cg searched */
+  uint32_t	fs_csp[UFS_MAXCSBUFS];/*list of fs_cs info buffers */
+  uint32_t	fs_maxcluster;
+  uint32_t	fs_cpc;		/* cyl per cycle in postbl */
+  uint16_t	fs_opostbl[16][8]; /* old rotation block list head */
+} __attribute__ ((__packed__));
+
 struct fs_u11_u2_st {
 	int8_t  fs_fsmnt[UFS2_MAXMNTLEN];	/* name mounted on */
 	uint8_t   fs_volname[UFS2_MAXVOLLEN]; /* volume name */
@@ -327,6 +333,9 @@ struct fs_u11_u2_st {
 	uint32_t    fs_pendinginodes;/*inodes in process of being freed */
 } __attribute__ ((__packed__));
 
+/*
+ * This is the actual superblock, as it is laid out on the disk.
+ */
 struct ufs_super_block {
 	uint32_t	fs_link;	/* UNUSED */
 	uint32_t	fs_rlink;	/* UNUSED */
@@ -404,16 +413,9 @@ struct ufs_super_block {
 	int8_t	fs_ronly;	/* mounted read-only flag */
 	int8_t	fs_flags;	/* currently unused flag */
 	union {
-		struct {
-			int8_t	fs_fsmnt[UFS_MAXMNTLEN];/* name mounted on */
-			uint32_t	fs_cgrotor;	/* last cg searched */
-			uint32_t	fs_csp[UFS_MAXCSBUFS];/*list of fs_cs info buffers */
-			uint32_t	fs_maxcluster;
-			uint32_t	fs_cpc;		/* cyl per cycle in postbl */
-			uint16_t	fs_opostbl[16][8]; /* old rotation block list head */
-		} fs_u1 __attribute__ ((__packed__));
+		struct fs_u11_u1_st fs_u1;
 		struct fs_u11_u2_st fs_u2;
-	}  fs_u11 __attribute__ ((__packed__));
+	}  fs_u11;
 	union {
 		struct {
 			uint32_t	fs_sparecon[53];/* reserved for future constants */
@@ -463,6 +465,6 @@ struct ufs_super_block {
 #define UFS_DYNAMICPOSTBLFMT	1	/* dynamic rotational table format */
 
 
-int check_ufs(disk_t *disk_car,partition_t *partition,const int debug);
-int recover_ufs(disk_t *disk_car, const struct ufs_super_block *sb, partition_t *partition,const int debug, const int dump_ind);
+int check_ufs(disk_t *disk_car,partition_t *partition,const int verbose);
+int recover_ufs(disk_t *disk_car, const struct ufs_super_block *sb, partition_t *partition,const int verbose, const int dump_ind);
 
